/*
 * NAME:	cics_types.h - CICS typedefs.
 * VERSION:	1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 *
 * DESCRIPTION:
 *		This header provides the basic CICS typedefs. These
 *		are used for user written monitoring functions, statistics
 *		formatting and user data conversion.
 */

#ifndef CICS_TYPES_H

#define CICS_TYPES_H			/* multiple inclusion defence	 */


/*
 * OBJECT: typedefs - Common CICS typedefs.
 */

typedef unsigned long	u_long_t;	/* unsigned 32-bit		 */
typedef signed long	long_t;		/* signed 32-bit		 */
typedef unsigned short	u_short_t;	/* unsigned 16-bit		 */
typedef int		bool_t;		/* boolean type			 */
typedef unsigned int	u_int_t;	/* unsigned natural machine size */
typedef unsigned char	u_byte_t;	/* unsigned 8-bit		 */
typedef signed char	byte_t;		/* signed 8-bit			 */

typedef struct {
    u_long_t	High;
    u_long_t	Low;
} u_xlong_t;				/* unsigned 64-bit		 */


#if !defined(TRUE) || !defined(FALSE)
#define FALSE	((bool_t)0)		/* A zero value for FALSE	 */
#define TRUE	(!FALSE)		/* A non zero value for TRUE	 */
#endif


#if !defined(CONST)
#define CONST
#endif
#if !defined(IN)
#define IN 	CONST			/* Function Input parameter	 */
#endif
#if !defined(OUT)
#define OUT				/* Function Output parameter	 */
#endif
#if !defined(INOUT)
#define INOUT				/* Function Input and Output	 */
#endif

/*
 * DESCRIPTION:
 *
 *	Common CICS typedefs used by Monitoring, Data Conversion and
 *	Statistics formatting. There are definitions for standard types and
 *	a set of defines which are used to indicate function parameter usage.
 *
 *	The parameter usage flags used in function definitions/declaration
 *	have the following meanings :-
 *	:dl tsize=18.
 *	:dt.IN
 *	:dd.defines a parameter which is read into the function;
 *	:dt.OUT
 *	:dd.defines a parameter which is modified by the function
 *	to pass information out of the function (a pointer);
 *	:dt.INOUT
 *	:dd.defines a parameter which is read into the function and
 *	which is modified by the function to pass information
 *	out of the function (a pointer).
 *	:edl.
 */

#endif /* CICS_TYPE_H */

