/*
 *
 * NAME:		cics3270.c -  CICS EPI 3270 Translation Tables
 *
 * VERSION:		1.1
 *
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *  The values in this file refer to ASCII definition of 3270 datastream.
 *
 */

#include <cicstype.h>

/*
 * Table for translating binary values to displayable characters
 *
 * This table is used in converting:
 *
 * - a binary 12-bit buffer address to a cursor address.
 * - standard field attributes and buffer addresses following orders
 *   in inbound streams to ASCII displayable characters.
 *
 * For instance on a 80 * 24 screen, cursor is at row 3 column 25.
 * Buffer address is ((3-1)*80) + (25-1) = 184 = 0xB8
 * This splits into 2 6-bit values of 0x02 and 0x38
 * Two byte buffer address is 0x42,0x38
 */

cics_ubyte_t cics_epi_bin_to_ASCII[0x40] =
{
   0x20,           /*  0x00 -> ' ' */
   0x41,           /*  0x01 -> 'A' */
   0x42,           /*  0x02 -> 'B' */
   0x43,           /*  0x03 -> 'C' */
   0x44,           /*  0x04 -> 'D' */
   0x45,           /*  0x05 -> 'E' */
   0x46,           /*  0x06 -> 'F' */
   0x47,           /*  0x07 -> 'G' */
   0x48,           /*  0x08 -> 'H' */
   0x49,           /*  0x09 -> 'I' */
   0x5B,           /*  0x0A -> '[' */
   0x2E,           /*  0x0B -> '.' */
   0x3C,           /*  0x0C -> '<' */
   0x28,           /*  0x0D -> '(' */
   0x2B,           /*  0x0E -> '+' */
   0x21,           /*  0x0F -> '!' */
   0x26,           /*  0x10 -> '&' */
   0x4A,           /*  0x11 -> 'J' */
   0x4B,           /*  0x12 -> 'K' */
   0x4C,           /*  0x13 -> 'L' */
   0x4D,           /*  0x14 -> 'M' */
   0x4E,           /*  0x15 -> 'N' */
   0x4F,           /*  0x16 -> 'O' */
   0x50,           /*  0x17 -> 'P' */
   0x51,           /*  0x18 -> 'Q' */
   0x52,           /*  0x19 -> 'R' */
   0x5D,           /*  0x1A -> ']' */
   0x24,           /*  0x1B -> '$' */
   0x2A,           /*  0x1C -> '*' */
   0x29,           /*  0x1D -> ')' */
   0x3B,           /*  0x1E -> ';' */
   0x5E,           /*  0x1F -> '^' */
   0x2D,           /*  0x20 -> '-' */
   0x2F,           /*  0x21 -> '/' */
   0x53,           /*  0x22 -> 'S' */
   0x54,           /*  0x23 -> 'T' */
   0x55,           /*  0x24 -> 'U' */
   0x56,           /*  0x25 -> 'V' */
   0x57,           /*  0x26 -> 'W' */
   0x58,           /*  0x27 -> 'X' */
   0x59,           /*  0x28 -> 'Y' */
   0x5A,           /*  0x29 -> 'Z' */
   0x7C,           /*  0x2A -> '|' */
   0x2C,           /*  0x2B -> ',' */
   0x25,           /*  0x2C -> '%' */
   0x5F,           /*  0x2D -> '_' */
   0x3E,           /*  0x2E -> '>' */
   0x3F,           /*  0x2F -> '?' */
   0x30,           /*  0x30 -> '0' */
   0x31,           /*  0x31 -> '1' */
   0x32,           /*  0x32 -> '2' */
   0x33,           /*  0x33 -> '3' */
   0x34,           /*  0x34 -> '4' */
   0x35,           /*  0x35 -> '5' */
   0x36,           /*  0x36 -> '6' */
   0x37,           /*  0x37 -> '7' */
   0x38,           /*  0x38 -> '8' */
   0x39,           /*  0x39 -> '9' */
   0x3A,           /*  0x3A -> ':' */
   0x23,           /*  0x3B -> '#' */
   0x40,           /*  0x3C -> '@' */
   0x27,           /*  0x3D -> '\''*/
   0x3D,           /*  0x3E -> '=' */
   0x22            /*  0x3F -> '"' */
};

/*
 * Table for translating displayable values to binary.
 *
 * This table is used in converting:
 *
 * -  write control characters from displayable characters to binary values.
 * -  standard field attributes and buffer addresses following orders
 *    in outbound streams from displayable characters to binary values.
 *
 * An 0xff in the table means the character is not convertable.
 */

cics_ubyte_t cics_epi_ASCII_to_bin[0x80] =
{
   0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, /* 0x00 to 0x1F */
   0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, /* are not      */
   0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, /* convertable  */
   0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
   0x00,                                           /* ' ' -> 0x00 */
   0x0F,                                           /* '!' -> 0x0F */
   0x3F,                                           /* '"' -> 0x3F */
   0x3B,                                           /* '#' -> 0x3B */
   0x1B,                                           /* '$' -> 0x1B */
   0x2C,                                           /* '%' -> 0x2C */
   0x10,                                           /* '&' -> 0x10 */
   0x3D,                                           /* '\' -> 0x3D'*/
   0x0D,                                           /* '(' -> 0x0D */
   0x1D,                                           /* ')' -> 0x1D */
   0x1C,                                           /* '*' -> 0x1C */
   0x0E,                                           /* '+' -> 0x0E */
   0x2B,                                           /* ',' -> 0x2B */
   0x20,                                           /* '-' -> 0x20 */
   0x0B,                                           /* '.' -> 0x0B */
   0x21,                                           /* '/' -> 0x21 */
   0x30,                                           /* '0' -> 0x30 */
   0x31,                                           /* '1' -> 0x31 */
   0x32,                                           /* '2' -> 0x32 */
   0x33,                                           /* '3' -> 0x33 */
   0x34,                                           /* '4' -> 0x34 */
   0x35,                                           /* '5' -> 0x35 */
   0x36,                                           /* '6' -> 0x36 */
   0x37,                                           /* '7' -> 0x37 */
   0x38,                                           /* '8' -> 0x38 */
   0x39,                                           /* '9' -> 0x39 */
   0x3A,                                           /* ':' -> 0x3A */
   0x1E,                                           /* ';' -> 0x1E */
   0x0C,                                           /* '<' -> 0x0C */
   0x3E,                                           /* '=' -> 0x3E */
   0x2E,                                           /* '>' -> 0x2E */
   0x2F,                                           /* '?' -> 0x2F */
   0x3C,                                           /* '@' -> 0x3C */
   0x01,                                           /* 'A' -> 0x01 */
   0x02,                                           /* 'B' -> 0x02 */
   0x03,                                           /* 'C' -> 0x03 */
   0x04,                                           /* 'D' -> 0x04 */
   0x05,                                           /* 'E' -> 0x05 */
   0x06,                                           /* 'F' -> 0x06 */
   0x07,                                           /* 'G' -> 0x07 */
   0x08,                                           /* 'H' -> 0x08 */
   0x09,                                           /* 'I' -> 0x09 */
   0x11,                                           /* 'J' -> 0x11 */
   0x12,                                           /* 'K' -> 0x12 */
   0x13,                                           /* 'L' -> 0x13 */
   0x14,                                           /* 'M' -> 0x14 */
   0x15,                                           /* 'N' -> 0x15 */
   0x16,                                           /* 'O' -> 0x16 */
   0x17,                                           /* 'P' -> 0x17 */
   0x18,                                           /* 'Q' -> 0x18 */
   0x19,                                           /* 'R' -> 0x19 */
   0x22,                                           /* 'S' -> 0x22 */
   0x23,                                           /* 'T' -> 0x23 */
   0x24,                                           /* 'U' -> 0x24 */
   0x25,                                           /* 'V' -> 0x25 */
   0x26,                                           /* 'W' -> 0x26 */
   0x27,                                           /* 'X' -> 0x27 */
   0x28,                                           /* 'Y' -> 0x28 */
   0x29,                                           /* 'Z' -> 0x29 */
   0x0A,                                           /* '[' -> 0x0A */
   0xFF,                                           /* '\\' not convertable */
   0x1A,                                           /* ']' -> 0x1A */
   0x1F,                                           /* '^' -> 0x1F */
   0x2D,                                           /* '_' -> 0x2D */
   0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, /* 0x60 to 0x7F */
   0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, /* are not      */
   0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, /* convertable  */
   0xFF, 0xFF, 0xFF, 0xFF, 0x2A, 0xFF, 0xFF, 0xFF  /* except '|' -> 0x2A */
};
