/*
 *
 * NAME:		cics3270.h -  CICS EPI 3270 constant definitions
 *
 * VERSION:		1.2
 *
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *  The values in this file refer to ASCII definition of 3270 datastream.
 *
 */

#ifndef _CICS3270_H

#define _CICS3270_H


#include <cicstype.h>

/* Certain of the constants are already defined in these supplied files */

#include <dfhaid.h>
#include <dfhbmsca.h>

/*
 *  Inbound Datastreams (to CICS) consist of
 *
 *  - An AID (1 byte). The key pressed.
 *  - A cursor address (2 bytes). This is a 3270 buffer address (12 bit)
 *    and should be calculated as specified below.
 *  - A data buffer (variable length). This consists of displayable data with
 *    embedded control characters defined below.
 */

/* Values for the AID fields */

#define  CICS_EPI_AID_ENTER           DFHENTER
#define  CICS_EPI_AID_CLEAR           DFHCLEAR
#define  CICS_EPI_AID_CLEAR_PARTITION DFHCLRP
#define  CICS_EPI_AID_SEL_PEN_ATT     DFHPEN
#define  CICS_EPI_AID_OP_ID           DFHOPID
#define  CICS_EPI_AID_MAG_RDR_NUM     DFHMSRE
#define  CICS_EPI_AID_TRIGGER         DFHTRIG
#define  CICS_EPI_AID_PA1             DFHPA1
#define  CICS_EPI_AID_PA2             DFHPA2
#define  CICS_EPI_AID_PA3             DFHPA3
#define  CICS_EPI_AID_PF1             DFHPF1
#define  CICS_EPI_AID_PF2             DFHPF2
#define  CICS_EPI_AID_PF3             DFHPF3
#define  CICS_EPI_AID_PF4             DFHPF4
#define  CICS_EPI_AID_PF5             DFHPF5
#define  CICS_EPI_AID_PF6             DFHPF6
#define  CICS_EPI_AID_PF7             DFHPF7
#define  CICS_EPI_AID_PF8             DFHPF8
#define  CICS_EPI_AID_PF9             DFHPF9
#define  CICS_EPI_AID_PF10            DFHPF10
#define  CICS_EPI_AID_PF11            DFHPF11
#define  CICS_EPI_AID_PF12            DFHPF12
#define  CICS_EPI_AID_PF13            DFHPF13
#define  CICS_EPI_AID_PF14            DFHPF14
#define  CICS_EPI_AID_PF15            DFHPF15
#define  CICS_EPI_AID_PF16            DFHPF16
#define  CICS_EPI_AID_PF17            DFHPF17
#define  CICS_EPI_AID_PF18            DFHPF18
#define  CICS_EPI_AID_PF19            DFHPF19
#define  CICS_EPI_AID_PF20            DFHPF20
#define  CICS_EPI_AID_PF21            DFHPF21
#define  CICS_EPI_AID_PF22            DFHPF22
#define  CICS_EPI_AID_PF23            DFHPF23
#define  CICS_EPI_AID_PF24            DFHPF24

/*
 *  Table for translating binary values to displayable characters is         
 *  defined in cics3270.c                                                   
 *
 *  This table is used in converting:                                      
 *
 *  - a binary 12-bit buffer address to a cursor address.                     
 *  - standard field attributes and buffer addresses following orders in     
 *    inbound streams to ASCII displayable characters.                      
 */

extern cics_ubyte_t cics_epi_bin_to_ASCII[0x40];


/*
 *                                                                         
 *  Outbound Datastreams (from CICS) consist of                           
 *                                                                       
 *  - Command (1 byte). Either a write or read command                  
 *  - Write Control Character (WCC) (1 byte - optional).               
 *  - A data buffer (variable length - optional)                      
 *                                                                   
 */

/* Values for the Command field - Note read modify, read modify all and */
/* write structured field are not supported.                            */

#define  CICS_EPI_CMD_WRITE            0x31
#define  CICS_EPI_CMD_ERASE_WRITE      0x35
#define  CICS_EPI_CMD_ERASE_WRITE_ALT  0x3D
#define  CICS_EPI_CMD_EAU              0x3F
#define  CICS_EPI_CMD_READ_BUFFER      0x32

/*
 * The WCC is transmitted in the 3270 data stream as a displayable character  
 * and should be translated using the table below to a binary                
 * value. This binary value is a bit array with the bits defined below in   
 * Intel bit convention (not as per 3270 manual).                          
 *                                                                        
 * For example receiving an 'A' translates to a binary value of 0x01 and 
 * means that the reset MDT bit is set.                                 
 */

#define  CICS_EPI_WCC_PRINTER_BITA        5   /* 3270 Manual value 2 */
#define  CICS_EPI_WCC_PRINTER_BITB        4   /* 3270 Manual value 3 */
#define  CICS_EPI_WCC_START_PRINT_BIT     3   /* 3270 Manual value 4 */
#define  CICS_EPI_WCC_ALARM_BIT           2   /* 3270 Manual value 5 */
#define  CICS_EPI_WCC_KEYBOARD_RESET_BIT  1   /* 3270 Manual value 6 */
#define  CICS_EPI_WCC_RESET_MDT_BIT       0   /* 3270 Manual value 7 */


/*
 *  Table for translating displayable values to binary is defined in   
 *  cics3270.c.                                                       
 *                                                                   
 *  This table is used in converting:                               
 *                                                                 
 *  -  write control characters from displayable characters to binary values.
 *  -  standard field attributes and buffer addresses following orders      
 *     in outbound streams from displayable characters to binary values.   
 */

extern cics_ubyte_t cics_epi_ASCII_to_bin[0x80];


/*                                                                        
 *  3270 Control characters consist of                                   
 *                                                                      
 *  - Orders                                                           
 *  - Standard field attributes                                       
 *  - Buffer addresses                                               
 *  - Extended field attribute types and values                     
 */

/*
 * 3270 orders occur in both inbound and outbound data buffers.               
 *                                                                           
 * Note the following orders are not supported                              
 * -  Graphic Escape (GE)                                                  
 * -  Substitute     (SUB)                                                
 * -  Duplicate      (DUP)                                               
 * -  Field Mark     (FM)                                               
 * -  Eight Ones     (EO)                                              
 */

#define  CICS_EPI_ORDER_SF  0x1D      /* Start Field                          */
#define  CICS_EPI_ORDER_SFE 0x10      /* Start Field Extended (Not a standard */
                                      /* 3270 value                           */
#define  CICS_EPI_ORDER_SBA 0x11      /* Set Buffer Address                   */
#define  CICS_EPI_ORDER_SA  0x1F      /* Set Attribute (Not a standard        */
                                      /* 3270 value)                          */
#define  CICS_EPI_ORDER_MF  0x1A      /* Modify Field (Not a standard         */
                                      /* 3270 value)                          */
#define  CICS_EPI_ORDER_IC  0x13      /* Insert Cursor                        */
#define  CICS_EPI_ORDER_PT  0x09      /* Program Tab                          */
#define  CICS_EPI_ORDER_RA  0x14      /* Repeat to Address                    */
#define  CICS_EPI_ORDER_EUA 0x12      /* Erase Unprotected to Address         */
#define  CICS_EPI_ORDER_FF  DFHBMPFF  /* Form Feed                            */
#define  CICS_EPI_ORDER_CR  DFHBMDCR  /* Carriage Return                      */
#define  CICS_EPI_ORDER_NL  DFHBMPNL  /* New Line                             */
#define  CICS_EPI_ORDER_EM  DFHBMPEM  /* End of Medium                        */

/*
 * The following orders have succeeding bytes                         
 *                                                                   
 *                              Number  Contents                    
 *                              of                                 
 *                              bytes                             
 *                                                               
 * Start Field                  1       Standard field attribute.       
 * Start Field Extended         1+2*N   N extended attribute type/value pairs
 * Set Buffer Address           2       Buffer address                      
 * Set Attribute                2       Extended attribute type/value pair 
 * Modify Field                 1+2*N   N extended attribute type/value pairs 
 * Repeat to Address            2       Buffer address + character           
 * Erase Unprotected to Address 2       Buffer address                      
 *                                                                         
 * -  A standard field attribute is transmitted in the 3270 data stream as a 
 *    displayable character and should be translated using                  
 *    cics_epi_buffer_address_table to a binary value. It is then a bit array 
 *    as defined below.                                                      
 * -  Buffer addresses should be converted from displayable to binary values
 *    in outbound streams and vice-versa in outbound streams, using the table 
 *    above.                                                                 
 * -  Extended attribute types and values are defined below.                
 */

/*
 * Standard field attributes, after conversion to a binary value is a bit    
 * array with the bits defined below in Intel bit convention (not as per    
 * 3270 manual).                                                           
 *                                                                        
 * For example receiving an 'A' translates to a binary value of 0x01 and 
 * means that the reset MDT bit is set                                  
 */

#define  CICS_EPI_ATTR_PROTECTED_BIT     5   /* 3270 Manual value 2 */
#define  CICS_EPI_ATTR_ALPHANUMERIC_BIT  4   /* 3270 Manual value 3 */
#define  CICS_EPI_ATTR_DISPLAY_BIT0      3   /* 3270 Manual value 4 */
#define  CICS_EPI_ATTR_DISPLAY_BIT1      2   /* 3270 Manual value 5 */
#define  CICS_EPI_ATTR_RESERVED_BIT      1   /* 3270 Manual value 6 */
#define  CICS_EPI_ATTR_MDT_BIT           0   /* 3270 Manual value 7 */

/* 
 * Some common values of standard field attributes from DFHDMSCA.H 
 */

#define  CICS_EPI_FLD_ATT_BMASK DFHBMASK /* Autoskip                          */
#define  CICS_EPI_FLD_ATT_BMUNP DFHBMUNP /* Unprotected                       */
#define  CICS_EPI_FLD_ATT_BMUNN DFHBMUNN /* Unprotected, numeric              */
#define  CICS_EPI_FLD_ATT_BMPRO DFHBMPRO /* Protected                         */
#define  CICS_EPI_FLD_ATT_BMBRY DFHBMBRY /* High Intensity                    */
#define  CICS_EPI_FLD_ATT_BMDAR DFHBMDAR /* Zero intensity, non-print         */
#define  CICS_EPI_FLD_ATT_BMFSE DFHBMFSE /* MDT set                           */
#define  CICS_EPI_FLD_ATT_BMPRF DFHBMPRF /* Protected, MDT set                */
#define  CICS_EPI_FLD_ATT_BMASF DFHBMASF /* Autoskip, MDT set                 */
#define  CICS_EPI_FLD_ATT_BMASB DFHBMASB /* Autoskip, High intensity          */
#define  CICS_EPI_FLD_ATT_UNNOD DFHUNNOD /* Unprotected, Non-display,         */
                                         /* Non-print, Non-detectable, MDT    */
#define  CICS_EPI_FLD_ATT_UNIMD DFHUNIMD /* Unprotected, Intensify, MDT       */
#define  CICS_EPI_FLD_ATT_UNNUM DFHUNNUM /* Unprotected, Numeric, MDT         */
#define  CICS_EPI_FLD_ATT_UNINT DFHUNINT /* Unprotected, Numeric, Intensify,  */
                                         /* MDT                               */
#define  CICS_EPI_FLD_ATT_UNNON DFHUNNON /* Unprotected, Numeric, Non-display,*/
                                         /* Non-print, Non-detectable, MDT    */
#define  CICS_EPI_FLD_ATT_PROTI DFHPROTI /* Protected                         */
#define  CICS_EPI_FLD_ATT_PROTN DFHPROTN /* Non-display, Non-print            */

/*
 * Extended attribute types                                                  
 *                                                                          
 * Note: The background colour and transparency types are not supported.   
 * SOSI is for DBCS support only                                          
 */

#define  CICS_EPI_ATT_TYPE_ALL        DFHALL   /* All character attributes  */
#define  CICS_EPI_ATT_TYPE_HILITE     DFHHLT   /* Extended highlighting     */
#define  CICS_EPI_ATT_TYPE_COLOR      DFHCOLOR /* Foreground colour         */
#define  CICS_EPI_ATT_TYPE_CHARSET    DFHPS    /* Character set             */
#define  CICS_EPI_ATT_TYPE_3270       DFH3270  /* 3270 field attribute      */
#define  CICS_EPI_ATT_TYPE_VALIDATION DFHVAL   /* Field validation          */
#define  CICS_EPI_ATT_TYPE_OUTLINE    DFHOUTLN /* Field outlining           */
#define  CICS_EPI_ATT_TYPE_SOSI       0xfe     /* Shift Out/Shift In        */

/*
 * Extended attribute values for colour - Note Black, Deep Blue, Orange,
 * Purple, Pale Green, Pale Turquoise, Grey and White are not supported
 */

#define  CICS_EPI_COLOR_DEFAULT DFHDFCOL       /* Default colour            */
#define  CICS_EPI_COLOR_BLUE    DFHBLUE        /* Blue                      */
#define  CICS_EPI_COLOR_RED     DFHRED         /* Red                       */
#define  CICS_EPI_COLOR_PINK    DFHPINK        /* Pink                      */
#define  CICS_EPI_COLOR_GREEN   DFHGREEN       /* Green                     */
#define  CICS_EPI_COLOR_TURQ    DFHTURQ        /* Turquoise                 */
#define  CICS_EPI_COLOR_YELLOW  DFHYELLO       /* Yellow                    */
#define  CICS_EPI_COLOR_NEUTRAL DFHNEUTR       /* Neutral                   */

/* 
 * Extended attribute values for extended highlight 
 */

#define  CICS_EPI_HILITE_DEFAULT    DFHDFHI  /* Default highlight           */
#define  CICS_EPI_HILITE_NORMAL     '0'      /* Normal (as field attribute) */
#define  CICS_EPI_HILITE_BLINK      DFHBLINK /* Blink                       */
#define  CICS_EPI_HILITE_REVERSE    DFHREVRS /* Reverse video               */
#define  CICS_EPI_HILITE_UNDERSCORE DFHUNDLN /* Underscore                  */

/* 
 * Extended attribute values for character set - DBC2 '8' is the only
 * non default set supported 
 */

#define  CICS_EPI_CHAR_SET_DEFAULT  0x00       /* Default                   */
#define  CICS_EPI_CHAR_SET_DBCS     '8'        /* DBCS                      */

/* 
 * Extended attribute values for 3270 field attribute as for standard field
 * attributes (no conversion necessary).
 */

/* 
 * Extended attribute values for field validation is a bit array with the
 * bits defined below in Intel bit convention (not as per the 3270 manual).
 * (No conversion necessary.)                                              
 */

#define  CICS_EPI_FLD_VAL_MAND_FILL_BIT   2    /* 3270 Manual value 5       */
#define  CICS_EPI_FLD_VAL_MAND_ENTRY_BIT  1    /* 3270 Manual value 6       */
#define  CICS_EPI_FLD_VAL_TRIGGER_BIT     0    /* 3270 Manual value 7       */

/* 
 * Extended attribute values for field outlining may be calculated by ANDing
 * together the following constants.                                        
 */

#define  CICS_EPI_OUTLINE_NONE   0x00
#define  CICS_EPI_OUTLINE_UNDER  0x01
#define  CICS_EPI_OUTLINE_RIGHT  0x02
#define  CICS_EPI_OUTLINE_OVER   0x04
#define  CICS_EPI_OUTLINE_LEFT   0x08

/* 
 * Extended attribute values for SOSI.
 */

#define  CICS_EPI_SOSI_DISABLED  0x00
#define  CICS_EPI_SOSI_ENABLED   0x01

#endif /* _CICS3270_H */

