/*
 * NAME:        cics_eci.h - External CICS Interface header
 *
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *  This file contains all the External CICS Interface (ECI)
 *  public definitions.
 */

#ifndef _CICS_ECI_H

#define _CICS_ECI_H


#include <cicstype.h>
#include <cics_packon.h>

/*
 * ECI callback notification function definition.                             
 */
typedef void (CICS_EciCallback_t)(cics_sshort_t);
typedef CICS_EciCallback_t CICSPTR *CICS_EciNotify_t;

/*
 * Parameter block for the ECI.                                               
 * Note: Text strings are NOT null-terminated.
 */

typedef struct
{
   CICS_EciNotify_t eci_callback;      /* Call back notification function     */
   cics_ptr_t      eci_commarea;       /* Ptr to commarea or NULL.            */
   cics_slong_t    eci_luw_token;      /* Must be set to zero for a new LUW   */
   cics_sshort_t   eci_call_type;      /* Call_type values:                   */
   cics_sshort_t   eci_commarea_length;/* Should be 0 if eci_commarea is NULL */
   cics_sshort_t   eci_timeout;        /* In seconds.                         */
   cics_sshort_t   eci_sys_return_code;/* Extra error code if problems arise. */
   cics_sshort_t   eci_extend_mode;    /* Valid values defined below.         */
   cics_sshort_t   eci_message_qualifier;/* This value returned on NOTIFY_MSG */
   cics_sshort_t   eci_version;        /* ECI version identifier:             */
   cics_char_t     eci_transid[4];     /*                                     */
   cics_char_t     eci_abend_code[4];  /* Returned to the caller.             */
   cics_char_t     eci_sysid[4];       /* Optional CICS sysid                 */
   cics_char_t     eci_program_name[8];/*                                     */
   cics_char_t     eci_userid[8];      /*                                     */
   cics_char_t     eci_password[8];    /*                                     */
   cics_char_t     eci_system_name[8]; /* System to execute ECI call          */
   cics_char_t     eci_userid2[16];    /* Extended userid                     */
   cics_char_t     eci_password2[16];  /* Extended password                   */
} ECI_PARMS;

typedef ECI_PARMS CICSPTR *PECI_PARMS;

/*
 * Values for eci_call_type
 */
#define ECI_SYNC			516 /* Synchronous call               */
#define ECI_ASYNC			517 /* Async call, notify by callback */
#define ECI_GET_REPLY			520 /* Get reply to ASYNC call        */
#define ECI_GET_REPLY_WAIT		521 /* As above but wait for reply    */
#define ECI_GET_SPECIFIC_REPLY		528 /* Get specific ASYNC reply       */
#define ECI_GET_SPECIFIC_REPLY_WAIT	529 /* As above but wait for reply    */
#define ECI_STATE_SYNC			522 /* Synchronous request for status */
#define ECI_STATE_ASYNC			523 /* Async request for status       */

/*
 * Values for eci_extend_mode
 */
#define ECI_NO_EXTEND        0   /* These values are to be used with    */
#define ECI_EXTENDED         1   /* call_types other that ECI_GET_xxx & */
#define ECI_COMMIT           2   /* ECI_STATE_xxx.                      */
#define ECI_CANCEL           ECI_COMMIT
#define ECI_BACKOUT          3   /*                                     */
#define ECI_STATE_IMMEDIATE  4   /* All ECI_STATE_xxx call-types.       */
#define ECI_STATE_CHANGED    5   /* ECI_STATE_ASYNC_SEM &               */
#define ECI_STATE_CANCEL     6   /* ECI_STATE_ASYNC_MSG call_types only.*/

/*
 * Values for eci_version
 */
#define ECI_VERSION_1        1
#define ECI_VERSION_MAX      1

/*
 * Datatype returned by the CICS_EciListSystems function                      
 */
#define CICS_ECI_SYSTEM_MAX        8
#define CICS_ECI_DESCRIPTION_MAX  60
typedef struct
{
   cics_char_t SystemName[CICS_ECI_SYSTEM_MAX+1];
   cics_char_t Description[CICS_ECI_DESCRIPTION_MAX+1];
} CICS_EciSystem_t;


/*
 * Function prototypes for ECI.                                               
 */
cics_sshort_t CICS_ExternalCall(ECI_PARMS CICSPTR *EciParms);
cics_sshort_t CICS_EciListSystems( cics_char_t      CICSPTR *NameSpace,
				   cics_ushort_t    CICSPTR *Systems,
				   CICS_EciSystem_t CICSPTR *List);

/*
 * ECI return codes.                                                          
 */
#define ECI_NO_ERROR                  0
#define ECI_ERR_INVALID_DATA_LENGTH  -1
#define ECI_ERR_INVALID_EXTEND_MODE  -2
#define ECI_ERR_NO_CICS              -3
#define ECI_ERR_CICS_DIED            -4
#define ECI_ERR_REQUEST_TIMEOUT      -5
#define ECI_ERR_NO_REPLY	     ECI_ERR_REQUEST_TIMEOUT
#define ECI_ERR_RESPONSE_TIMEOUT     -6
#define ECI_ERR_TRANSACTION_ABEND    -7
#define ECI_ERR_EXEC_NOT_RESIDENT    -8
#define ECI_ERR_LUW_TOKEN            ECI_ERR_EXEC_NOT_RESIDENT
#define ECI_ERR_SYSTEM_ERROR         -9
#define ECI_ERR_NULL_WIN_HANDLE      -10
#define ECI_ERR_NULL_MESSAGE_ID      -12
#define ECI_ERR_THREAD_CREATE_ERROR  -13
#define ECI_ERR_INVALID_CALL_TYPE    -14
#define ECI_ERR_ALREADY_ACTIVE       -15
#define ECI_ERR_RESOURCE_SHORTAGE    -16
#define ECI_ERR_NO_SESSIONS          -17
#define ECI_ERR_NULL_SEM_HANDLE      -18
#define ECI_ERR_INVALID_DATA_AREA    -19
#define ECI_ERR_NULL_CALLBACK        -20
#define ECI_ERR_INVALID_VERSION      -21
#define ECI_ERR_UNKNOWN_SERVER       -22
#define ECI_ERR_CALL_FROM_CALLBACK   -23
#define ECI_ERR_INVALID_TRANSID      -24
#define ECI_ERR_MORE_SYSTEMS         -25
#define ECI_ERR_NO_SYSTEMS           -26
#define ECI_ERR_SECURITY_ERROR       -27
#define ECI_ERR_MAX_SYSTEMS          -28
#define ECI_ERR_MAX_SESSIONS         -29
#define ECI_ERR_ROLLEDBACK           -30


/*
 * ECI LuwId value for new LUWs.                                              
 */
#define ECI_LUW_NEW  0L


/*
 * Commarea layout for ECI_STATE_xxx CallType requests other than when the    
 * ExtendMode is ECI_STATE_CANCEL.                                            
 *                                                                            
 * It should be supplied with valid values for a request where the ExtendMode 
 * is ECI_STATE_CHANGED.  In this case a response will be returned only when  
 * the status is different to that which was supplied.                        
 *                                                                            
 * It will be returned with the current status in these fields except where   
 * the ExtendMode is ECI_STATE_CANCEL.                                        
 */
typedef struct
{
   cics_sshort_t   ConnectionType;              /* What am I connected to ?   */
   cics_sshort_t   CicsServerStatus;            /* What is the server state?  */
   cics_sshort_t   CicsClientStatus;            /* What is the client state?  */
} ECI_STATUS;

/*
 * Values for ConnectionType
 */
#define ECI_CONNECTED_NOWHERE        0    /* Connected to nothing.      */
#define ECI_CONNECTED_TO_SERVER      1    /* Connected to server.       */
#define ECI_CONNECTED_TO_CLIENT      2    /* Connected to client.       */

/*
 * Values for CicsServerStatus
 */
#define ECI_SERVERSTATE_UNKNOWN      0    /* Server state unknown.      */
#define ECI_SERVERSTATE_UP           1    /* Server available.          */
#define ECI_SERVERSTATE_DOWN         2    /* Server unavailable.        */

/*
 * Values for CicsClientStatus
 */
#define ECI_CLIENTSTATE_UNKNOWN      0    /* Client state unknown.      */
#define ECI_CLIENTSTATE_UP           1    /* Client available.          */
#define ECI_CLIENTSTATE_INAPPLICABLE 2    /* Client state inapplicable  */

#define ECI_STATUS_LENGTH sizeof(ECI_STATUS)      /* For the CommareaLength   */

#include <cics_packoff.h>

#endif /* _CICS_ECI_H */

