/*
 *  NAME: summariseStatistic.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 * void summariseStatistic( Monitor_t * Monitor,
 *                          unsigned int StatisticNumber ,
 *                          const char * StatisticName
 *                        )
 *
 *  DESCRIPTION:
 *
 *      Summarise (across all terminals) the distribution of a statistic.
 *
 *
 *  CAVEATS/WARNINGS:
 *
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */
#include <epiload.h>
#include <epiload_int.h>
void summariseStatistic( Monitor_t * Monitor,
                          unsigned int StatisticNumber ,
                          const char * StatisticName
                          )
{
   if (StatisticNumber < StatisticCount)
   {
      int s1 = 0 ;
      int sok[32] = {0,0,0,0,0,0,0,0,
                     0,0,0,0,0,0,0,0,
                     0,0,0,0,0,0,0,0,
                     0,0,0,0,0,0,0,0} ;

      double sx = 0.0 ;
      double sxx = 0.0 ;
      int terminals = Monitor -> TerminalsCreated ;
      Terminal_t * TermArray = Monitor -> FirstTerminal ;
      int x ;
      for (x = 0 ; x < terminals ; x += 1 )
      {
         Statistic_t * Statistic = &(TermArray[x].Statistic[StatisticNumber]) ;
         int cs1 = Statistic -> current . s1 ;
         double csx = Statistic -> current . sx ;
         double csxx = Statistic -> current . sxx ;
         int ps1 = Statistic -> previous.s1 ;
         double psx = Statistic -> previous.sx ;
         double psxx = Statistic -> previous.sxx ;
         int q ;

         s1 += (cs1 - ps1) ;
         sx += (csx - psx) ;
         sxx += (csxx - psxx) ;
         for (q=0;q<32 ;q += 1 ) {
           int sokq = sok[q] ;
           int csok = Statistic -> current . sok[q] ;
           int psok = Statistic -> previous . sok[q] ;
           Statistic -> previous . sok[q] = csok ;
           sok[q] = sokq + csok - psok ;
         } /* endfor */

         Statistic -> previous.s1 = cs1 ;
         Statistic -> previous.sx = csx ;
         Statistic -> previous.sxx = csxx ;

      } /* endfor */
      {
        double invs1 = 1.0 / s1 ;
        double mean = sx * invs1 ;
        double variance = ( sxx * invs1 ) - ( mean * mean ) ;
        double stdev = sqrt(variance) ;
        int q ;

        fprintf(stderr, "%s: Count=%i Mean=%f StDev=%f\n",
                        StatisticName, s1, mean, stdev) ;
        for (q=0; q<32; q+=1 ) {
           int sokq = sok[q] ;
           double okfraction =  sokq * invs1 ;
           if (sokq > 0 ) {
              fprintf(stderr, "%s: Attribute %i : %i/%i=%f\n",
                        StatisticName, q, sokq, s1, okfraction) ;
           } /* endif */
        } /* endfor */
      }
   } /* endif */
}
