/*
 *  NAME: addTerminal.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 *    void addTerminal (Terminal_t * Terminal)
 *
 *  DESCRIPTION:
 *
 *      EPI workload driver function to autoinstall a terminal to
 *      an appropriate CICS region. This would not normally be called by
 *      user code; it would normally be called internally within the
 *      workload driver only.
 *
 *  CAVEATS/WARNINGS:
 *
 *      Will 'exit' the process on failure
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */

#include <epiload.h>
#include <epiload_int.h>
void addTerminal(Terminal_t * Terminal)
{
  cics_ushort_t TIndex ;
  Monitor_t * Monitor = TerminalMon(Terminal) ;

  /*
   * Limit the number of concurrent signers-on
   */
  pthread_mutex_lock(&(Monitor->Mutex)) ;
  while (Monitor -> TerminalsAdding >= concurrentAdd )
  {
     pthread_cond_wait(&(Monitor->AddSeq),&(Monitor->Mutex)) ;
  } /* endwhile */
  Monitor -> TerminalsAdding += 1 ;
  pthread_mutex_unlock(&(Monitor->Mutex)) ;

  /*
   * Issue the 'autoinstall' call
   */

  fprintf(stderr, "Installing to %s\n", TerminalRegion(Terminal)) ;
  CHECK(
        CICS_EpiAddTerminal,
                           (NULL,       /* No Namespace needed       */
                            (char *) TerminalRegion(Terminal),     /* Use region specified */
                            NULL,       /* Autoinstall the terminal  */
                            (char *) TerminalModel(Terminal),       /* Appropriate terminal model */
                            NULL,       /* No event notification     */
                            NULL,       /* TermDetails not needed    */
                            &TIndex), /* Receives new TermIndex   */
        CICS_EPI_NORMAL,
        CICS_EPI_TERM_INDEX_NONE

       ) ;

  /*
   * Allow the next guy to sign on
   */
  pthread_mutex_lock(&(Monitor->Mutex)) ;
  Monitor -> TerminalsAdding -= 1 ;
  pthread_cond_signal(&(Monitor->AddSeq)) ;
  pthread_mutex_unlock(&(Monitor->Mutex)) ;

  TerminalTermIndex(Terminal) = TIndex ;
}
