/*
 *  NAME: bytes_3270.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 * void bytes_3270(Datastream_t *Data, const void *Bytes, unsigned int Length)
 *
 *  DESCRIPTION:
 *
 * Adds a byte string to an inbound 3270 datastream
 * See 'text_3270' if you want to add a string-with-nul-terminator rather
 * than a string-with-length
 *
 *  CAVEATS/WARNINGS:
 *
 * If the datastream overflows the buffer, the text will be truncated.
 *
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */

#include <epiload.h>
#include <epiload_int.h>
void bytes_3270(Datastream_t *Data, const void *Bytes, unsigned int Length)
{
   unsigned int CurrentSize = Data->CurrentSize ;
   unsigned int NextSize = CurrentSize+Length;
   if (NextSize <= max3270ds)
   {
      memcpy(&(Data->Stream[CurrentSize]),Bytes,Length) ;
      Data->CurrentSize = NextSize ;
   } /* endif */
}
