/*
 *  NAME: collectStatistic.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 * void collectStatistic(Terminal_t * Terminal,
 *                       unsigned int StatisticNumber,
 *                       double Observation,
 *                       unsigned int Observation_OK
 *                       )
 *
 *  DESCRIPTION:
 *
 *      Collect a sample value of a statistic
 *
 *      The sample value may be accompanied by a number of attribute flags
 *      (one per bit, up to 32). These indicate whether the sample was OK
 *      according to some criterion; typically meeting a required response
 *      time. For efficiency, any unused bits should be the high-order
 *      bits.
 *
 *
 *  CAVEATS/WARNINGS:
 *
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */
#include <epiload.h>
#include <epiload_int.h>
void collectStatistic(Terminal_t * Terminal,
                       unsigned int StatisticNumber,
                       double Observation,
                       unsigned int Observation_OK
                       )
{
   if (StatisticNumber < StatisticCount)
   {
      Statistic_t * Statistic = &(Terminal->Statistic[StatisticNumber]) ;
      int new_s1 = Statistic -> current.s1 + 1 ;
      double new_sx = Statistic -> current.sx + Observation ;
      double new_sxx = Statistic -> current.sxx + (Observation*Observation) ;
      int x ;

      for (x=0; x<32 ; x += 1) {
         int sok = Statistic -> current.sok[x] + 1 ;
         if (Observation_OK & 1) {
           Statistic -> current.sok[x] = sok ;
         } /* endif */
         if (0 == Observation_OK) {
            break ;
         } /* endif */
         Observation_OK >>= 1 ;
      } /* endfor */

      Statistic -> current.s1 = new_s1 ;
      Statistic -> current.sx = new_sx ;
      Statistic -> current.sxx = new_sxx ;

   } /* endif */
}
