/*
 *  NAME: deleteTerminal.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 *    void deleteTerminal (Terminal_t * Terminal)
 *
 *  DESCRIPTION:
 *
 *      EPI workload driver function to sign off a terminal from
 *      its CICS region. This would not normally be called by
 *      user code; it would normally be called internally within the
 *      workload driver only.
 *
 *  CAVEATS/WARNINGS:
 *
 *      Will 'exit' the process on failure
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */

#include <epiload.h>
#include <epiload_int.h>
void deleteTerminal(Terminal_t * Terminal)
{
  int TermIndex=TerminalTermIndex(Terminal) ;
  Monitor_t * Monitor = TerminalMon(Terminal) ;

  /*
   * Limit the number of concurrent signers-off
   */

  pthread_mutex_lock(&(Monitor->Mutex)) ;
  while (Monitor -> TerminalsAdding >= concurrentAdd )
  {
     pthread_cond_wait(&(Monitor->AddSeq),&(Monitor->Mutex)) ;
  } /* endwhile */
  Monitor -> TerminalsAdding += 1 ;
  pthread_mutex_unlock(&(Monitor->Mutex)) ;

  /*
   * Issue the 'uninstall' (CSSF) request
   */

  CHECK(CICS_EpiDelTerminal,(TermIndex),CICS_EPI_NORMAL,TermIndex) ;
  while (! Terminal -> Terminated)
  {
     processEventNoCallback(Terminal) ;
  } /* endwhile */

  /*
   * Allow the next guy to sign off
   */

  pthread_mutex_lock(&(Monitor->Mutex)) ;
  Monitor -> TerminalsAdding -= 1 ;
  pthread_cond_signal(&(Monitor->AddSeq)) ;
  pthread_mutex_unlock(&(Monitor->Mutex)) ;
}
