/*
 *  NAME: displayData.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 *    void displayData (const void * Data, int Size)
 *    static void displayLine(const char * Data, int Start, int Size)
 *
 *  DESCRIPTION:
 *
 *      EPI workload driver function to display a (3270-ASCII) datastream
 *      received from a CICS region. This is called only for EPI-workload
 *      applications built for diagnostic mode.
 *
 *      This would not normally be called by
 *      user code; it would normally be called internally within the
 *      workload driver only.
 *
 *  CAVEATS/WARNINGS:
 *
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */

#include <epiload.h>
#include <epiload_int.h>
static void displayLine(const char * Data, int Start, int Size)
{
   int Index ;
   int Char ;
   fprintf(stderr,"%04x -", Start) ;
   for (Index = 0 ; Index < Size ; Index += 1 )
   {
      fprintf(stderr, " %02x", Data[Index]) ;
   } /* endfor */
   for (Index = Size; Index < 16 ; Index += 1 ) {
      fprintf(stderr, "   ") ;
   } /* endfor */
   fprintf(stderr, " [") ;
   for (Index = 0 ; Index < Size ; Index += 1 )
   {
      Char = Data[Index] ;
      if (Char >= 0x20) {
        fprintf(stderr, "%c", Char) ;
      } else {
        fprintf(stderr,".") ;
      } /* endif */
   } /* endfor */
   fprintf(stderr, "]\n") ;

}

void displayData(const void * Data, int Size)
{
   const char * CData = (const char *) Data ;

   if (diagnose)
   {
      fprintf(stderr,"\"");
      fwrite(Data,1,Size,stderr) ;
      fprintf(stderr,"\"\n");
   } /* endif */

   if (diagnosex)
   {
      int Index ;
      for ( Index = 0 ; Index < Size-16; Index += 16 )
      {
         displayLine(&(CData[Index]), Index, 16) ;
      }
      displayLine(&(CData[Index]), Index, Size-Index) ;
   } /* endif */
}
