/*
 *  NAME: main.c
 *
 *
 *  VERSION: 1.3
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 * int main(int argc, char ** argv, char ** envp)
 *
 *  DESCRIPTION:
 *
 *      'main' function for EPI workload simulator
 *
 *
 *  CAVEATS/WARNINGS:
 *
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */
#include <epiload.h>
#include <epiload_int.h>
#include <alloca.h>
int main(int argc, char ** argv, char ** envp)
{
  Monitor_t Monitor ;
  pthread_t Thread ;
  int Terminal ;
  Terminal_t * Terminals ;
  int TerminalQty = TerminalCount() ;
  /*
   * Pick up the locale from the environment
   */
  (void)setlocale(LC_ALL,"");

  /*
   * Allocate space for the per-terminal structures
   */
  Terminals = (Terminal_t *) alloca(TerminalQty*sizeof(Terminal_t)) ;
  {
     int StatCount = StatisticCount ;
     int TermX ;
     for (TermX=0; TermX < TerminalQty; TermX += 1)
     {
        Terminals[TermX].Statistic = (Statistic_t *) alloca(StatCount*sizeof(Statistic_t)) ;
        {
           int StatX ;
           for (StatX = 0 ; StatX < StatisticCount; StatX += 1 )
           {
              clearStatistic(&(Terminals[TermX].Statistic[StatX])) ;
           } /* endfor */
        }
     } /* endfor */
  }

  /*
   * Start the EPI
   */
  CHECK(CICS_EpiInitialize,(CICS_EPI_VERSION_101), CICS_EPI_NORMAL,CICS_EPI_TERM_INDEX_NONE) ;

  /*
   * Start threads to run terminals
   */

  pthread_mutex_init(&Monitor.Mutex, pthread_mutexattr_default);
  pthread_cond_init(&Monitor.AddCond, pthread_condattr_default);
  pthread_cond_init(&Monitor.DeleteCond, pthread_condattr_default);
  pthread_cond_init(&Monitor.AddSeq, pthread_condattr_default);
  Monitor.TerminalsCreated=TerminalCount() ;
  Monitor.TerminalsAdded=-1 ; /* This accounts for the monitoring thread too */
  Monitor.TerminalsAdding=0 ;
  Monitor.TerminalsDeleted=0 ;
  Monitor.RequestFinish=0 ;
  Monitor.FirstTerminal=Terminals ;

  for (Terminal = 0 ; Terminal < TerminalQty ; Terminal += 1 )
  {
    Terminals[Terminal].TransactionCount = 0 ;
    Terminals[Terminal].Monitor = & Monitor ;
    Terminals[Terminal].Terminated = 0 ;
    Terminals[Terminal].TermSequence = Terminal ;
    Terminals[Terminal].Region = RegionName(Terminal) ;
    Terminals[Terminal].Model = DeviceType(Terminal) ;
    CHECKU(
      pthread_create,
      (&Thread,pthread_attr_default,terminalThreadWrap,&(Terminals[Terminal])),
      0
    ) ;
    CHECKU(pthread_detach,(&Thread),0) ;
  } /* endfor */

  /*
   * Run the (user-provided) terminal monitor
   */
  TerminalMonitor(&Monitor,TerminalQty) ;

  /*
   * Wait for terminal threads to complete
   */
  pthread_mutex_lock(&Monitor.Mutex) ;
  while (Monitor.TerminalsCreated != Monitor.TerminalsDeleted )
  {
     pthread_cond_wait(&Monitor.DeleteCond,&Monitor.Mutex) ;
  } /* endwhile */
  pthread_mutex_unlock(&Monitor.Mutex) ;

  /*
   * Clean up the 'threads' resources
   */
  pthread_mutex_destroy(&Monitor.Mutex) ;
  pthread_cond_destroy(&Monitor.AddCond) ;
  pthread_cond_destroy(&Monitor.AddSeq) ;
  pthread_cond_destroy(&Monitor.DeleteCond) ;

  /*
   * Clean up 'epi' resources
   */
  CHECK(CICS_EpiTerminate,(),CICS_EPI_NORMAL,CICS_EPI_TERM_INDEX_NONE) ;

  /*
   * Finish normally
   */
 return 0 ;

}
