/*
 *  NAME: notifyMain.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 *    void notifyMain (Terminal_t * Terminal)
 *
 *  DESCRIPTION:
 *
 *      EPI workload driver function to notify the 'main' thread when
 *      all terminals are signed off, and it is therefore safe
 *      to exit the process.
 *
 *      This would not normally be called by
 *      user code; it would normally be called internally within the
 *      workload driver only.
 *
 *  CAVEATS/WARNINGS:
 *
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */

#include <epiload.h>
#include <epiload_int.h>
void notifyMain(Terminal_t * Terminal)
{
  Monitor_t * Monitor = TerminalMon(Terminal) ;
  pthread_mutex_lock(&(Monitor -> Mutex)) ;
  {
     if ((++(Monitor->TerminalsDeleted)) == Monitor -> TerminalsCreated)
     {
        pthread_cond_signal(&(Monitor -> DeleteCond)) ;
     } /* endif */
  }
  pthread_mutex_unlock(&(Monitor -> Mutex)) ;

}
