/*
 *  NAME: report_failure.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 * void report_failure(int result, const char * text ,
 *                     int expect, int terminal) ;
 *
 *  DESCRIPTION:
 *
 *      Report a failure in the EPI workload simulator
 *
 *
 *  CAVEATS/WARNINGS:
 *
 *      This will exit the process unconditionally. The failure is
 *      assumed to be fatal, and no recovery is attempted.
 *
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */
#include <epiload.h>
#include <epiload_int.h>
void report_failure(int result, const char * text , int expect, int terminal )
{
   CICS_EpiSysError_t Syserr ;
   (void) CICS_EpiGetSysError(terminal, &Syserr) ;
   fprintf(stderr,"Call to %s failed, code %i, expected %i."
                  "Terminal %i Cause %i Value %i Msg %s. Exiting.\n",
                           text,           result,      expect,
                            terminal, Syserr.Cause, Syserr.Value, Syserr.Msg
          ) ;
   exit(1) ;
}
