/*
 *  NAME: runTransaction.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 *    static void startTransaction(Terminal_t * Terminal, void * TranKey)
 *    double runTransaction(Terminal_t * Terminal, void * TranKey)
 *
 *  DESCRIPTION:
 *
 *      EPI workload driver function to run a transaction and report the
 *      response time.
 *
 *
 *  CAVEATS/WARNINGS:
 *
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */

#include <epiload.h>
#include <epiload_int.h>
static void startTransaction(Terminal_t * Terminal, void * TranKey)
{
  int TermIndex=TerminalTermIndex(Terminal) ;
  Datastream_t Datastream ;

  /*
   * Possible default datastream
   */
  defaultDatastream(&Datastream) ;
  /*
   * User exit to change the datastream ...
   */
  BuildData(&Datastream, TranKey ) ;

  /*
   * Diagnostic display of the datastream if required
   */
  if (diagnose || diagnosex)
  {
      fprintf(stderr,
        "Message, size=%i, sending to CICS for transaction <%s>\n"
        ,Datastream.CurrentSize, Datastream.Transaction) ;
      displayData(Datastream.Stream, Datastream.CurrentSize) ;
  } /* endif */

  /*
   * Now send it to CICS
   */
   CHECK(CICS_EpiStartTran,
         (TermIndex,
          Datastream.Transaction,
          Datastream.Stream,
          Datastream.CurrentSize
         ),
         CICS_EPI_NORMAL,TermIndex) ;
}

double runTransaction(Terminal_t * Terminal, void * TranKey)
{
   int s_0, s_1, ns_0, ns_1 ;
   CLOCK(s_0,ns_0) ;
   startTransaction(Terminal, TranKey) ;
   processEventToEnd(Terminal, TranKey ) ;
   CLOCK(s_1,ns_1) ;
   return (s_1-s_0) + 0.000000001 * (ns_1 - ns_0) ;
}
