/*
 *  NAME: sba_3270.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 * void sba_3270(Datastream_t *Data, int Row, int Col)
 *
 *  DESCRIPTION:
 *
 * Set a given screen position for an inbound 3270 datastream
 * Note this will return silently if the location is bad, or if the
 * datastream buffer is full
 *
 * 'Row' and 'Col' are values from the 'POS' macro in the BMS map.
 * The top left hand corner is (1,1), and you point at the attribute byte.
 * This function adjusts to get you ready to write to the data area of the
 * field.
 *
 *  CAVEATS/WARNINGS:
 *
 *
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */
#include <epiload.h>
#include <epiload_int.h>


void sba_3270(Datastream_t *Data, int Row, int Col)
{
   unsigned int ScreenLoc = ((Row-1)*80) + Col ;
   unsigned int CurrentSize = Data->CurrentSize ;
   if (   (CurrentSize <= max3270ds-3 ) /* Check for room in the buffer */
       && (ScreenLoc < 1920) /* Check it is on the screen ! */
      )
   {
     int Pos_Hi = binaryToGraphic[ScreenLoc >> 6   ] ;
     int Pos_Lo = binaryToGraphic[ScreenLoc & 0x3f ] ;
     Data->CurrentSize = CurrentSize+3 ;
     Data->Stream[CurrentSize] = 0x11 ; /* 3270 set-buffer-address */
     Data->Stream[CurrentSize+1] = Pos_Hi ; /* 3270 addressing */
     Data->Stream[CurrentSize+2] = Pos_Lo ; /* 3270 addressing */
   } /* endif */
}
