/*
 *  NAME: syncMonitor.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 * void syncMonitor(Monitor_t * Monitor)
 * void syncTerminal(Terminal_t * Terminal)
 *
 *  DESCRIPTION:
 *
 *      EPI workload driver functions to synchronise (wait) until all
 *      terminals are installed and the terminal monitor is ready to
 *      time a run.
 *
 *      When the terminals and the monitor make their calls, the threads
 *      are suspended until the last one comes in. At that time, all
 *      threads are made runnable again.
 *
 *
 *  CAVEATS/WARNINGS:
 *
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */

#include <epiload.h>
#include <epiload_int.h>
void syncMonitor(Monitor_t * Monitor)
{
   pthread_mutex_lock(&(Monitor -> Mutex)) ;
   /*
    * Check if we're the last one in
    */

   if ((++(Monitor->TerminalsAdded)) == Monitor -> TerminalsCreated)
   {
      /*
       * Yes. Let everyone continue on their way.
       */

      pthread_cond_broadcast(&(Monitor -> AddCond)) ;
   }
   else
   {
      /*
       * No. Wait until someone else (i.e. the last in) releases us.
       */

      do
      {
         pthread_cond_wait(&(Monitor -> AddCond),&(Monitor -> Mutex)) ;
      } while ( Monitor->TerminalsAdded != Monitor -> TerminalsCreated ); /* enddo */
   } /* endif */
   pthread_mutex_unlock(&(Monitor -> Mutex)) ;
}

void syncTerminal(Terminal_t * Terminal)
{
  syncMonitor(TerminalMon(Terminal)) ;
}
