/*
 * NAME:        cicseciexit.h 
 *
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *      This header file contains definitions and structures for the ECI
 *      exits
 */

#if !defined(CICSECIEXIT_H)
#define CICSECIEXIT_H
#include <cics_samples.h>

/*
 * Special return codes from exits.
 * All other non-zero return codes indicate an exit has failed
 * but will not cause special handling in the EPI code.
 */
#define CICS_EXIT_OK			 0
#define CICS_EXIT_NO_EXITS               1 /* do not call any other exits */
#define CICS_EXIT_CANT_INIT_EXITS	 2 /* Can't inisialise exits */
#define CICS_EXIT_GIVE_UP          	 3 /* exit SystemId retry loop */

/*
 * The following values correspond to error cases identified in the
 * exits architecture.  They have no special meaning to the EPI
 */
#define CICS_EXIT_BAD_ANCHOR		 10 /* Bad anchor value */
#define CICS_EXIT_BAD_STORAGE		 11 /* Can't tidy up storage */
#define CICS_EXIT_BAD_PARM		 12 /* Bad parameter */

#define CICS_EXIT_USER_BASE		100 /* exit specific codes start here */

/*
 * Datatypes
 */
typedef void *CICS_EciAnchor_t;
typedef struct CICS_EciToken
{
    cics_ulong_t Pid;
    cics_ulong_t count;
}   CICS_EciToken_t;

/*
 * InitialiseExit
 * Called after EPI internal initialisation is complete
 */
cics_sshort_t CICS_EciInitialiseExit
			( IN  cics_sshort_t	Version
			, OUT CICS_EciAnchor_t *Anchor
			);

/*
 * TerminateExit
 * Called before termination 
 */
cics_sshort_t CICS_EciTerminateExit
			( IN CICS_EciAnchor_t Anchor
			);

/*
 * CICS_EciExternalCallExit1
 * CICS_EciExternalCallExit2
 */
cics_sshort_t CICS_EciExternalCallExit1
			( IN    CICS_EciAnchor_t Anchor
			, IN    CICS_EciToken_t  Token
			, INOUT PECI_PARMS	 ParmPtr
			);
cics_sshort_t CICS_EciExternalCallExit2
			( IN    CICS_EciAnchor_t Anchor
			, IN    CICS_EciToken_t  Token
			, INOUT PECI_PARMS	 ParmPtr
			);

/*
 * SystemIdExit
 */
cics_sshort_t CICS_EciSystemIdExit
		    ( IN    CICS_EciAnchor_t Anchor
		    , IN    CICS_EciToken_t  Token
		    , INOUT PECI_PARMS       ParmPtr
		    , IN    cics_sshort_t    Reason
		    );

/*
 * CICS_EciDataSendExit
 */
cics_sshort_t CICS_EciDataSendExit
			( IN    CICS_EciAnchor_t Anchor
			, IN    CICS_EciToken_t  Token
			);

/*
 * CICS_EciDataReturnExit
 */
cics_sshort_t CICS_EciDataReturnExit
			( IN    CICS_EciAnchor_t Anchor
			, IN    CICS_EciToken_t  Token
			, INOUT PECI_PARMS	 ParmPtr
			);

typedef struct CICS_EciExitList
{
    void *InitialiseExit;
    void *TerminateExit;
    void *ExternalCallExit1;
    void *ExternalCallExit2;
    void *SystemIdExit;
    void *DataSendExit;
    void *DataReturnExit;
}   CICS_EciExitList_t;

#endif /* CICSECIEXIT_H */
