/*
 * NAME:        cicsepiexit.h
 *
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION
 *      This header file contains definitions and structures for the EPI
 *      exits
 */

#if !defined(CICSEPIEXIT_H)
#define CICSEPIEXIT_H
#include <cics_samples.h>

/*
 * Special return codes from exits.
 * All other non-zero return codes indicate an exit has failed
 * but will not cause special handling in the EPI code.
 */
#define CICS_EXIT_OK			 0
#define CICS_EXIT_NO_EXITS               1 /* do not call any other exits */
#define CICS_EXIT_CANT_INIT_EXITS	 2 /* cannot init exit environment */
#define CICS_EXIT_DONT_ADD_TERMINAL	 3 /* abort current AddTerminal */
					   /* or SystemId addterminal retry */

/*
 * The following values correspond to error cases identified in the
 * exits architecture.  They have no special meaning to the EPI
 */
#define CICS_EXIT_BAD_ANCHOR		 10 /* Bad anchor value */
#define CICS_EXIT_BAD_STORAGE		 11 /* Can't tidy up storage */
#define CICS_EXIT_BAD_PARM		 12 /* Bad parameter */

#define CICS_EXIT_USER_BASE		100 /* exit specific codes start here */

/*
 * Datatypes
 */
typedef void *CICS_EpiAnchor_t;

/*
 * InitialiseExit
 * Called after EPI internal initialisation is complete
 */
cics_sshort_t CICS_EpiInitialiseExit
			( IN  cics_ulong_t      Version
			, OUT CICS_EpiAnchor_t *Anchor
			);

/*
 * TerminateExit
 * Called before termination 
 */
cics_sshort_t CICS_EpiTerminateExit
			( IN CICS_EpiAnchor_t Anchor
			);

/*
 * AddTerminalExit
 * NOTE: May not be driven if AddTerminal fails with ERR_MAX_TERMINALS
 */
cics_sshort_t CICS_EpiAddTerminalExit
			( IN    CICS_EpiAnchor_t Anchor
			, INOUT cics_char_t	*NameSpace
			, INOUT cics_char_t	 System[CICS_EPI_SYSTEM_MAX+1]
			, INOUT cics_char_t	 NetName[CICS_EPI_NETNAME_MAX+1]
			, INOUT cics_char_t	 DevType[CICS_EPI_DEVTYPE_MAX+1]
			);

/*
 * StartTranExit
 */
cics_sshort_t CICS_EpiStartTranExit
			( IN CICS_EpiAnchor_t	 Anchor
			, IN cics_char_t         TransId[CICS_EPI_TRANSID_MAX+1]
			, IN cics_ubyte_t	*Data
			, IN cics_ushort_t	 Size
			);

/*
 * ReplyExit
 */
cics_sshort_t CICS_EpiReplyExit
			( IN CICS_EpiAnchor_t	 Anchor
			, IN cics_ushort_t       TermIndex
			, IN cics_ubyte_t	*Data
			, IN cics_ushort_t	 Size
			);

/*
 * DelTerminalExit
 */
cics_sshort_t CICS_EpiDelTerminalExit
			( IN CICS_EpiAnchor_t Anchor
			, IN cics_ushort_t    TermIndex
			);
/*
 * GetEventExit
 */
cics_sshort_t CICS_EpiGetEventExit
			( IN CICS_EpiAnchor_t     Anchor
			, IN cics_ushort_t	  TermIndex
			, IN CICS_EpiWait_t	  Wait
			, IN CICS_EpiEventData_t *Event
			);

/*
 * TranFailedExit
 * This will only be driven when GetEvent returns with a genuine event
 * (ie not after parameter validation errors or an exception)
 */
cics_sshort_t CICS_EpiTranFailedExit
			( IN CICS_EpiAnchor_t     Anchor
			, IN cics_ushort_t	  TermIndex
			, IN CICS_EpiWait_t	  Wait
			, IN CICS_EpiEventData_t *Event
			);

/*
 * SystemIdExit
 * NOTE: Not driven if AddTerminal fails with ERR_MAX_TERMINALS
 */
cics_sshort_t CICS_EpiSystemIdExit
		    ( IN    CICS_EpiAnchor_t Anchor
		    , INOUT cics_char_t	 *NameSpace
		    , INOUT cics_char_t	  System[CICS_EPI_SYSTEM_MAX+1]
		    , IN    cics_char_t	  NetName[CICS_EPI_NETNAME_MAX+1]
		    , IN    cics_char_t	  DevType[CICS_EPI_DEVTYPE_MAX+1]
		    , IN    cics_char_t	  FailedSystem [CICS_EPI_SYSTEM_MAX+1]
		    , IN    cics_sshort_t Reason
		    , IN    cics_ulong_t  SubReason
		    , OUT   cics_char_t	  UserId[64]
		    , OUT   cics_char_t   PassWord[64]
		    );

/*
 * TermIdExit
 * This will be driven at the end of a successful CICS_EpiAddTerminal
 */
cics_sshort_t CICS_EpiTermIdExit
			( IN CICS_EpiAnchor_t     Anchor
			, IN cics_ushort_t	  TermIndex
			, IN cics_char_t	 *System
			);

typedef struct CICS_EpiExitList
{
    void *InitialiseExit;
    void *TerminateExit;
    void *AddTerminalExit;
    void *StartTranExit;
    void *ReplyExit;
    void *DelTerminalExit;
    void *GetEventExit;
    void *TranFailedExit;
    void *SystemIdExit;
    void *TermIdExit;
}   CICS_EpiExitList_t;

#endif /* CICSEPIEXIT_H */

/* EOF */
