/*
 * NAME:	cics_emp_clock.h - Typedefs etc for Monitoring Clock
 * VERSION:	1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 *
 * DESCRIPTION: This file contains definitions for a monitoring clock.
 */

#ifndef CICS_EMP_CLOCK_H

#define CICS_EMP_CLOCK_H


#include <cicstype.h>       /* CICS-supplied datatypes */

/*
 * OBJECT: CICS_EMP_Clock_t - The Monitor Facility Clock Structure.
 */

struct CICS_EMP_Clock
{
    cics_ulong_t	Counter;
    cics_ulong_t	TotalTime;
    cics_ushort_t	ClockStatus;
};

typedef struct CICS_EMP_Clock	CICS_EMP_Clock_t;

/*
 * DESCRIPTION:
 * This structure contains the necessary information to accumulate a monitor
 * clock.  The structure contains all the information that is written to
 * the Monitoring Transient Data Queue for a clock.  The members are defined
 * as follows:
 * :ul.
 * :li.
 * :hp2.Counter:ehp2. - This member contains the number of times the clock
 * has been started.
 * :li.
 * :hp2.TotalTime:ehp2. - This member contains the total time the clock has
 * measured in microseconds.
 * :li.
 * :hp2.ClockStatus:ehp2. - This member contains the status information for
 * this clock (see CICS_EMP_CLOCK* flags)
 * :eul.
 */

/*
 * OBJECT: CICS_EMP_CLOCK* - Clock Status member values in CICS_EMP_Clock 
 * structure.
 */

#define CICS_EMP_CLOCK_STARTED	0x0001	/* Clock started		*/
#define CICS_EMP_CLOCK_STOPPED	0x0002	/* Clock stopped		*/
#define CICS_EMP_CLOCK_OP_START	0x0004	/* Out of phase start occurred	*/
#define CICS_EMP_CLOCK_OP_STOP	0x0008	/* Out of phase stop occurred	*/
#define CICS_EMP_CLOCK_ACTIVE	0x0010	/* Restart clock after write	*/

/*
 * DESCRIPTION:
 * CICS_EMP_CLOCK* defines the status of a Monitor clock.
 */

#endif

