/*
 *
 * VERSION:	1.1
 *
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:	
 *     Definitions used as part of the CICS statistics facility.
 */



#ifndef _CICS_STAT_H_		/* [      Multiple inclusion Lock */

#define _CICS_STAT_H_




#include <cicstype.h>			/* CICS-supplied datatypes	   */
#include <cics_samples.h>		/* CICS-supplied macro definitions */

/*
 * OBJECT: CICSSTAT_CATID(catid) - Statistics Category Identifiers
 */

#define CICSSTAT_CATID_DUMP		(cics_ulong_t) 0x010000
#define CICSSTAT_CATID_FILE		(cics_ulong_t) 0x020000
#define CICSSTAT_CATID_ISCM		(cics_ulong_t) 0x030000
#define CICSSTAT_CATID_JOUR		(cics_ulong_t) 0x040000
#define CICSSTAT_CATID_LUWM		(cics_ulong_t) 0x050000
#define CICSSTAT_CATID_PROG		(cics_ulong_t) 0x060000
#define CICSSTAT_CATID_RTDB		(cics_ulong_t) 0x070000
#define CICSSTAT_CATID_STOR		(cics_ulong_t) 0x080000
#define CICSSTAT_CATID_STOT		(cics_ulong_t) 0x090000
#define CICSSTAT_CATID_TASK		(cics_ulong_t) 0x0a0000
#define CICSSTAT_CATID_TERM		(cics_ulong_t) 0x0b0000
#define CICSSTAT_CATID_TDQU		(cics_ulong_t) 0x0c0000
#define CICSSTAT_CATID_TRAN		(cics_ulong_t) 0x0d0000
#define CICSSTAT_CATID_TSQU             (cics_ulong_t) 0x0e0000


#define CICSSTAT_CATID_INTERVAL		(cics_ulong_t) 0x0f0000
#define CICSSTAT_CATID_ENDOFDAY		(cics_ulong_t) 0x100000
#define CICSSTAT_CATID_UNSOLICITED	(cics_ulong_t) 0x110000
#define CICSSTAT_CATID_REQUESTED_RESET	(cics_ulong_t) 0x120000
#define CICSSTAT_CATID_REQUESTED	(cics_ulong_t) 0x130000
#define CICSSTAT_CATID_ENDOFREGION	(cics_ulong_t) 0x140000
#define CICSSTAT_CATID_END		(cics_ulong_t) 0x150000
#define CICSSTAT_CATID_TIME_NA		(cics_ulong_t) 0x160000
#define CICSSTAT_CATID_TIME_START	(cics_ulong_t) 0x170000
#define CICSSTAT_CATID_TIME_STOP	(cics_ulong_t) 0x180000
#define CICSSTAT_CATID_SUMMARY		(cics_ulong_t) 0x190000
#define CICSSTAT_CATID_ALL_DETAILS	(cics_ulong_t) 0x1a0000
#define CICSSTAT_CATID_ONE_DETAIL	(cics_ulong_t) 0x1b0000

/*
 * DESCRIPTION:
 *	These defines uniquely identify each of the available statistics 
 *      categories.
 */

/*
 * Provide macro that can be used to convert a catid into a cics_slong_t bit Mask type.
 */

#define CICSSTAT_CATID_TO_MASK(CatId)					(cics_ulong_t)(1 << (((cics_ulong_t)(CatId) >> 16) - 1))


/*
 * OBJECT: CICSSTAT_*(statid) - Unique Statistics Identifiers
 */

#define	CICSSTAT_DUMP_DUMPS_WRITTEN    	  (cics_ulong_t)(0x00 | CICSSTAT_CATID_DUMP)
#define	CICSSTAT_DUMP_WRITE_ERRORS     	  (cics_ulong_t)(0x01 | CICSSTAT_CATID_DUMP)


#define	CICSSTAT_PROG_TOTAL_PROGS_RUN     (cics_ulong_t)(0x00 | CICSSTAT_CATID_PROG)
#define	CICSSTAT_PROG_NUM_TIMES_USED      (cics_ulong_t)(0x01 | CICSSTAT_CATID_PROG)
#define	CICSSTAT_PROG_STAT_PROG_TYPE      (cics_ulong_t)(0x02 | CICSSTAT_CATID_PROG)


#define	CICSSTAT_TSQU_READ_MAIN        	  (cics_ulong_t)(0x00 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_WRITE_MAIN       	  (cics_ulong_t)(0x01 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_DELETE_MAIN      	  (cics_ulong_t)(0x02 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_READ_AUX         	  (cics_ulong_t)(0x03 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_WRITE_AUX        	  (cics_ulong_t)(0x04 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_DELETE_AUX       	  (cics_ulong_t)(0x05 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_CHECKPOINT       	  (cics_ulong_t)(0x06 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_ABORT            	  (cics_ulong_t)(0x07 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_COMMIT           	  (cics_ulong_t)(0x08 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_EXHAUSTED        	  (cics_ulong_t)(0x09 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_PEAK_QS          	  (cics_ulong_t)(0x0a | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_CURRENT_QS       	  (cics_ulong_t)(0x0b | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_MOST_ITEMS       	  (cics_ulong_t)(0x0c | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_IO_ERRORS        	  (cics_ulong_t)(0x0d | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_AUTH_FAILURES    	  (cics_ulong_t)(0x0e | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_QS_PURGED        	  (cics_ulong_t)(0x0f | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_CAGE_RUNS        	  (cics_ulong_t)(0x10 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_DELETE_WAITS     	  (cics_ulong_t)(0x11 | CICSSTAT_CATID_TSQU)
#define	CICSSTAT_TSQU_REMOTE_REQS      	  (cics_ulong_t)(0x12 | CICSSTAT_CATID_TSQU)


#define	CICSSTAT_FILE_NOREADS             (cics_ulong_t)(0x00 | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOWRITES            (cics_ulong_t)(0x01 | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOBROWSES           (cics_ulong_t)(0x02 | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NODELETES           (cics_ulong_t)(0x03 | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOUPDATES           (cics_ulong_t)(0x04 | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOOPENS             (cics_ulong_t)(0x05 | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOCLOSES            (cics_ulong_t)(0x06 | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOREADS_TOTAL       (cics_ulong_t)(0x07 | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOWRITES_TOTAL      (cics_ulong_t)(0x08 | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOBROWSES_TOTAL     (cics_ulong_t)(0x09 | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NODELETES_TOTAL     (cics_ulong_t)(0x0a | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOUPDATES_TOTAL     (cics_ulong_t)(0x0b | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOOPENS_TOTAL       (cics_ulong_t)(0x0c | CICSSTAT_CATID_FILE)
#define	CICSSTAT_FILE_NOCLOSES_TOTAL      (cics_ulong_t)(0x0d | CICSSTAT_CATID_FILE)


#define	CICSSTAT_STOT_ALLOCS_SUCCEEDED    (cics_ulong_t)(0x00 | CICSSTAT_CATID_STOT)
#define	CICSSTAT_STOT_DEALLOCS_SUCCEEDED  (cics_ulong_t)(0x01 | CICSSTAT_CATID_STOT)
#define	CICSSTAT_STOT_ALLOCS_FAILED       (cics_ulong_t)(0x02 | CICSSTAT_CATID_STOT)
#define	CICSSTAT_STOT_DEALLOCS_FAILED     (cics_ulong_t)(0x03 | CICSSTAT_CATID_STOT)
#define	CICSSTAT_STOT_ABEND_OCCURRENCE    (cics_ulong_t)(0x04 | CICSSTAT_CATID_STOT)
#define	CICSSTAT_STOT_NUMBER_OF_TASKS     (cics_ulong_t)(0x05 | CICSSTAT_CATID_STOT)
#define	CICSSTAT_STOT_TRANS_ABENDS        (cics_ulong_t)(0x06 | CICSSTAT_CATID_STOT)


#define	CICSSTAT_TDQU_STAT_EP_READS       (cics_ulong_t)(0x00 | CICSSTAT_CATID_TDQU)
#define	CICSSTAT_TDQU_STAT_EP_WRITES      (cics_ulong_t)(0x01 | CICSSTAT_CATID_TDQU)
#define	CICSSTAT_TDQU_STAT_IP_READS       (cics_ulong_t)(0x02 | CICSSTAT_CATID_TDQU)
#define	CICSSTAT_TDQU_STAT_IP_WRITES      (cics_ulong_t)(0x03 | CICSSTAT_CATID_TDQU)
#define	CICSSTAT_TDQU_STAT_RMT_READS      (cics_ulong_t)(0x04 | CICSSTAT_CATID_TDQU)
#define	CICSSTAT_TDQU_STAT_RMT_WRITES     (cics_ulong_t)(0x05 | CICSSTAT_CATID_TDQU)
#define	CICSSTAT_TDQU_STAT_TRIG_READS     (cics_ulong_t)(0x06 | CICSSTAT_CATID_TDQU)
#define	CICSSTAT_TDQU_STAT_TRIG_WRITES    (cics_ulong_t)(0x07 | CICSSTAT_CATID_TDQU)
#define	CICSSTAT_TDQU_STAT_LABEL_READS    (cics_ulong_t)(0x08 | CICSSTAT_CATID_TDQU)
#define	CICSSTAT_TDQU_STAT_LABEL_WRITES   (cics_ulong_t)(0x09 | CICSSTAT_CATID_TDQU)


#define	CICSSTAT_TERM_INPUTS              (cics_ulong_t)(0x00 | CICSSTAT_CATID_TERM)
#define	CICSSTAT_TERM_OUTPUTS             (cics_ulong_t)(0x01 | CICSSTAT_CATID_TERM)
#define	CICSSTAT_TERM_XMITERRS            (cics_ulong_t)(0x02 | CICSSTAT_CATID_TERM)
#define	CICSSTAT_TERM_TRANS               (cics_ulong_t)(0x03 | CICSSTAT_CATID_TERM)
#define	CICSSTAT_TERM_TRANERRS            (cics_ulong_t)(0x04 | CICSSTAT_CATID_TERM)
#define	CICSSTAT_TERM_TOTAL_AUTOINSTALLS  (cics_ulong_t)(0x05 | CICSSTAT_CATID_TERM)
#define	CICSSTAT_TERM_TOTAL_AUTOFAILS     (cics_ulong_t)(0x06 | CICSSTAT_CATID_TERM)
#define	CICSSTAT_TERM_TOTAL_AUTODISCONNECTS						  (cics_ulong_t)(0x07 | CICSSTAT_CATID_TERM)

#define CICSSTAT_TERM_SCHDL_LIMIT         (cics_ulong_t)(0x08 | CICSSTAT_CATID_TERM)
#define CICSSTAT_TERM_SYNC_COUNT          (cics_ulong_t)(0x09 | CICSSTAT_CATID_TERM)
#define CICSSTAT_TERM_ASYNC_COUNT         (cics_ulong_t)(0x0a | CICSSTAT_CATID_TERM)
#define CICSSTAT_TERM_THREAD_HWM          (cics_ulong_t)(0x0b | CICSSTAT_CATID_TERM)
#define CICSSTAT_TERM_THREAD_LIMIT        (cics_ulong_t)(0x0c | CICSSTAT_CATID_TERM)
#define CICSSTAT_TERM_RPC_REQUESTS        (cics_ulong_t)(0x0d | CICSSTAT_CATID_TERM)
#define CICSSTAT_TERM_RPC_ERRORS          (cics_ulong_t)(0x0e | CICSSTAT_CATID_TERM)



#define	CICSSTAT_RTDB_ADD                 (cics_ulong_t)(0x00 | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_GET                 (cics_ulong_t)(0x01 | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_GETP                (cics_ulong_t)(0x02 | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_DELTA               (cics_ulong_t)(0x03 | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_DELETE              (cics_ulong_t)(0x04 | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_BROWSE              (cics_ulong_t)(0x05 | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_NEXT                (cics_ulong_t)(0x06 | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_CHAIN               (cics_ulong_t)(0x07 | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_CHAINHI             (cics_ulong_t)(0x08 | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_DELTAUSE            (cics_ulong_t)(0x09 | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_USE                 (cics_ulong_t)(0x0a | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_ENDUSE              (cics_ulong_t)(0x0b | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_EXCLUSIVE           (cics_ulong_t)(0x0c | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_RECORDS             (cics_ulong_t)(0x0d | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_COLLISIONS          (cics_ulong_t)(0x0e | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_CHAINMAX            (cics_ulong_t)(0x0f | CICSSTAT_CATID_RTDB)
#define	CICSSTAT_RTDB_CHAINHIMAX          (cics_ulong_t)(0x10 | CICSSTAT_CATID_RTDB)


#define	CICSSTAT_STOR_RPOOL_SIZE          (cics_ulong_t)(0x00 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_RPOOL_THRESH        (cics_ulong_t)(0x01 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_RBYTES_USED         (cics_ulong_t)(0x02 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_RALLOCS_SUCCEEDED   (cics_ulong_t)(0x03 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_RALLOCS_FAILED      (cics_ulong_t)(0x04 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_RFREES_SUCCEEDED    (cics_ulong_t)(0x05 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_RFREES_FAILED       (cics_ulong_t)(0x06 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_RSOS_OCCURRENCE     (cics_ulong_t)(0x07 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_RHASH_MAXLEN        (cics_ulong_t)(0x08 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_TPOOL_SIZE          (cics_ulong_t)(0x09 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_TPOOL_THRESH        (cics_ulong_t)(0x0a | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_TBYTES_USED         (cics_ulong_t)(0x0b | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_TALLOCS_SUCCEEDED   (cics_ulong_t)(0x0c | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_TALLOCS_FAILED      (cics_ulong_t)(0x0d | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_TFREES_SUCCEEDED    (cics_ulong_t)(0x0e | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_TFREES_FAILED       (cics_ulong_t)(0x0f | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_TSOS_OCCURRENCE     (cics_ulong_t)(0x10 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_THASH_MAXLEN        (cics_ulong_t)(0x11 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_RHASH_BUCKET        (cics_ulong_t)(0x12 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_THASH_BUCKET        (cics_ulong_t)(0x13 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_RBYTES_HIGHWATER    (cics_ulong_t)(0x14 | CICSSTAT_CATID_STOR)
#define	CICSSTAT_STOR_TBYTES_HIGHWATER    (cics_ulong_t)(0x15 | CICSSTAT_CATID_STOR)
#define CICSSTAT_STOR_RBYTES_ALLOCSTART	  (cics_ulong_t)(0x16 | CICSSTAT_CATID_STOR)
#define CICSSTAT_STOR_TBYTES_ALLOCSTART	  (cics_ulong_t)(0x17 | CICSSTAT_CATID_STOR)
#define CICSSTAT_STOR_RBYTES_FREED	  (cics_ulong_t)(0x18 | CICSSTAT_CATID_STOR)
#define CICSSTAT_STOR_TBYTES_FREED	  (cics_ulong_t)(0x19 | CICSSTAT_CATID_STOR)


#define	CICSSTAT_ISCM_MAXNUMLQSR          (cics_ulong_t)(0x00 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NUMLQSR             (cics_ulong_t)(0x01 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NUMXMITLQSR         (cics_ulong_t)(0x02 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_PURGEDLQSR          (cics_ulong_t)(0x03 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NUMATTEMPTS         (cics_ulong_t)(0x04 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NUMALLOC            (cics_ulong_t)(0x0b | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NUMICES             (cics_ulong_t)(0x0c | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NUMENQRES           (cics_ulong_t)(0x0d | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_MAXICES             (cics_ulong_t)(0x0e | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_ENQWAIT             (cics_ulong_t)(0x0f | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_LOCKWAIT            (cics_ulong_t)(0x10 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NFCIN               (cics_ulong_t)(0x11 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NFCOUT              (cics_ulong_t)(0x12 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NFCXFR              (cics_ulong_t)(0x13 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NTDIN               (cics_ulong_t)(0x14 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NTDOUT              (cics_ulong_t)(0x15 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NTDXFR              (cics_ulong_t)(0x16 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NTSIN               (cics_ulong_t)(0x17 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NTSOUT              (cics_ulong_t)(0x18 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NTSXFR              (cics_ulong_t)(0x19 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NATIIN              (cics_ulong_t)(0x1a | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NATIOUT             (cics_ulong_t)(0x1b | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NATIXFR             (cics_ulong_t)(0x1c | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NDPLIN              (cics_ulong_t)(0x1d | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NDPLOUT             (cics_ulong_t)(0x1e | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NDPLXFR             (cics_ulong_t)(0x1f | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_NDL1IN              (cics_ulong_t)(0x20 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_OUTTRREQ            (cics_ulong_t)(0x21 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_INTRREQ             (cics_ulong_t)(0x22 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_BPSDEFSENT          (cics_ulong_t)(0x23 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_BPSDEFRECEIVED      (cics_ulong_t)(0x24 | CICSSTAT_CATID_ISCM)
#define	CICSSTAT_ISCM_FORWARD             (cics_ulong_t)(0x25 | CICSSTAT_CATID_ISCM)
#define CICSSTAT_ISCM_DYNAMICTRREQ        (cics_ulong_t)(0x26 | CICSSTAT_CATID_ISCM)


#define	CICSSTAT_LUWM_NUM_ABORTED         (cics_ulong_t)(0x01 | CICSSTAT_CATID_LUWM)
#define	CICSSTAT_LUWM_NUM_HITS    	  (cics_ulong_t)(0x03 | CICSSTAT_CATID_LUWM)
#define	CICSSTAT_LUWM_MAX_CHAIN_LENGTH    (cics_ulong_t)(0x04 | CICSSTAT_CATID_LUWM)


#define	CICSSTAT_TASK_NUM_TRANS           (cics_ulong_t)(0x00 | CICSSTAT_CATID_TASK)
#define	CICSSTAT_TASK_NUM_TRANS_ABENDED   (cics_ulong_t)(0x01 | CICSSTAT_CATID_TASK)
#define	CICSSTAT_TASK_NUM_INVALID_ADDRESS (cics_ulong_t)(0x02 | CICSSTAT_CATID_TASK)
#define	CICSSTAT_TASK_NUM_PROT_VIOLATION  (cics_ulong_t)(0x03 | CICSSTAT_CATID_TASK)
#define	CICSSTAT_TASK_NUM_SYSTEM_ERROR    (cics_ulong_t)(0x04 | CICSSTAT_CATID_TASK)
#define	CICSSTAT_TASK_NUM_EXCEPTIONS      (cics_ulong_t)(0x05 | CICSSTAT_CATID_TASK)
#define	CICSSTAT_TASK_NUM_TRANNONS        (cics_ulong_t)(0x06 | CICSSTAT_CATID_TASK)
#define	CICSSTAT_TASK_NUM_APP_SERVERS_FULL 					  (cics_ulong_t)(0x07 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_PEAK_TASKS          (cics_ulong_t)(0x08 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS1_MAXVAL       (cics_ulong_t)(0x09 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS1_MAXLIM       (cics_ulong_t)(0x0a | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS1_CURRENT      (cics_ulong_t)(0x0b | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS1_MAX_TIMES_HIT                                           (cics_ulong_t)(0x0c | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS1_PEAK_TASKS   (cics_ulong_t)(0x0d | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS1_MAXLIM_REJECTS                                           (cics_ulong_t)(0x0e | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS2_MAXVAL       (cics_ulong_t)(0x0f | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS2_MAXLIM       (cics_ulong_t)(0x10 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS2_CURRENT      (cics_ulong_t)(0x11 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS2_MAX_TIMES_HIT                                           (cics_ulong_t)(0x12 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS2_PEAK_TASKS   (cics_ulong_t)(0x13 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS2_MAXLIM_REJECTS                                           (cics_ulong_t)(0x14 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS3_MAXVAL       (cics_ulong_t)(0x15 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS3_MAXLIM       (cics_ulong_t)(0x16 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS3_CURRENT      (cics_ulong_t)(0x17 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS3_MAX_TIMES_HIT                                           (cics_ulong_t)(0x18 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS3_PEAK_TASKS   (cics_ulong_t)(0x19 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS3_MAXLIM_REJECTS                                           (cics_ulong_t)(0x1a | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS4_MAXVAL       (cics_ulong_t)(0x1b | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS4_MAXLIM       (cics_ulong_t)(0x1c | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS4_CURRENT      (cics_ulong_t)(0x1d | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS4_MAX_TIMES_HIT                                           (cics_ulong_t)(0x1e | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS4_PEAK_TASKS   (cics_ulong_t)(0x1f | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS4_MAXLIM_REJECTS                                           (cics_ulong_t)(0x20 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS5_MAXVAL       (cics_ulong_t)(0x21 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS5_MAXLIM       (cics_ulong_t)(0x22 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS5_CURRENT      (cics_ulong_t)(0x23 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS5_MAX_TIMES_HIT                                           (cics_ulong_t)(0x24 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS5_PEAK_TASKS   (cics_ulong_t)(0x25 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS5_MAXLIM_REJECTS                                           (cics_ulong_t)(0x26 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS6_MAXVAL       (cics_ulong_t)(0x27 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS6_MAXLIM       (cics_ulong_t)(0x28 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS6_CURRENT      (cics_ulong_t)(0x29 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS6_MAX_TIMES_HIT                                           (cics_ulong_t)(0x2a | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS6_PEAK_TASKS   (cics_ulong_t)(0x2b | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS6_MAXLIM_REJECTS                                           (cics_ulong_t)(0x2c | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS7_MAXVAL       (cics_ulong_t)(0x2d | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS7_MAXLIM       (cics_ulong_t)(0x2e | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS7_CURRENT      (cics_ulong_t)(0x2f | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS7_MAX_TIMES_HIT                                           (cics_ulong_t)(0x30 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS7_PEAK_TASKS   (cics_ulong_t)(0x31 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS7_MAXLIM_REJECTS                                           (cics_ulong_t)(0x32 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS8_MAXVAL       (cics_ulong_t)(0x33 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS8_MAXLIM       (cics_ulong_t)(0x34 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS8_CURRENT      (cics_ulong_t)(0x35 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS8_MAX_TIMES_HIT                                           (cics_ulong_t)(0x36 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS8_PEAK_TASKS   (cics_ulong_t)(0x37 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS8_MAXLIM_REJECTS                                           (cics_ulong_t)(0x38 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS9_MAXVAL       (cics_ulong_t)(0x39 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS9_MAXLIM       (cics_ulong_t)(0x3a | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS9_CURRENT      (cics_ulong_t)(0x3b | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS9_MAX_TIMES_HIT                                           (cics_ulong_t)(0x3c | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS9_PEAK_TASKS   (cics_ulong_t)(0x3d | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS9_MAXLIM_REJECTS                                           (cics_ulong_t)(0x3e | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS10_MAXVAL      (cics_ulong_t)(0x3f | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS10_MAXLIM       (cics_ulong_t)(0x40 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS10_CURRENT     (cics_ulong_t)(0x41 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS10_MAX_TIMES_HIT                                           (cics_ulong_t)(0x42 | CICSSTAT_CATID_TASK)

#define CICSSTAT_TASK_CLASS10_PEAK_TASKS  (cics_ulong_t)(0x43 | CICSSTAT_CATID_TASK)
#define CICSSTAT_TASK_CLASS10_MAXLIM_REJECTS                                           (cics_ulong_t)(0x44 | CICSSTAT_CATID_TASK)


#define	CICSSTAT_TRAN_TRAN_STARTS         (cics_ulong_t)(0x00 | CICSSTAT_CATID_TRAN)
#define	CICSSTAT_TRAN_TRANS_ABENDS        (cics_ulong_t)(0x01 | CICSSTAT_CATID_TRAN)
#define CICSSTAT_TRAN_DYNAMIC_RETRY_COUNT (cics_ulong_t)(0x02 | CICSSTAT_CATID_TRAN)
#define CICSSTAT_TRAN_DYNAMIC_REMOTE      (cics_ulong_t)(0x03 | CICSSTAT_CATID_TRAN)
#define CICSSTAT_TRAN_DYNAMIC_LOCAL       (cics_ulong_t)(0x04 | CICSSTAT_CATID_TRAN)


#define CICSSTAT_JOUR_CALLBACK_COUNT      (cics_ulong_t)(0x00 | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_NOSPACE_TOTAL       (cics_ulong_t)(0x01 | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_NOSPACE_ITERATIONS_TOTAL                                           (cics_ulong_t)(0x02 | CICSSTAT_CATID_JOUR)

#define CICSSTAT_JOUR_CHECKPOINT_COUNT    (cics_ulong_t)(0x03 | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_SYS_JRNL_PUTS_COUNT (cics_ulong_t)(0x04 | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_BUFFERED_WRITES_TOTAL                                           (cics_ulong_t)(0x05 | CICSSTAT_CATID_JOUR)

#define CICSSTAT_JOUR_FLUSHED_WRITES_TOTAL                                           (cics_ulong_t)(0x06 | CICSSTAT_CATID_JOUR)

#define CICSSTAT_JOUR_IOERRORS_TOTAL      (cics_ulong_t)(0x07 | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_NOTAUTH_ATTEMPTS_TOTAL                                           (cics_ulong_t)(0x08 | CICSSTAT_CATID_JOUR)

#define CICSSTAT_JOUR_BUFFERED_WRITES     (cics_ulong_t)(0x09 | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_FLUSHED_WRITES      (cics_ulong_t)(0x0a | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_IOERRORS            (cics_ulong_t)(0x0b | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_NOTAUTH_ACCESSES    (cics_ulong_t)(0x0c | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_NOSPACE_CONDITIONS  (cics_ulong_t)(0x0d | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_STORAGE_CUSHION_RELEASES  (cics_ulong_t)(0x0e | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_LOG_FORCES  (cics_ulong_t)(0x0f | CICSSTAT_CATID_JOUR)
#define CICSSTAT_JOUR_LOG_TRUNCATES  (cics_ulong_t)(0x10 | CICSSTAT_CATID_JOUR)


/*
 * DESCRIPTION:
 *	These defines are used to uniquely identify all of the possible
 *      statistics gathered by the facility.
 */

/*
 * OBJECT: CICSSTAT_Mesg_t - Message index type.
 */

#define CICSSTAT_Mesg_t 	cics_ulong_t

/* 
 * DESCRIPTION:
 *	The message index into the message catalog is of this type.
 */


/*
 * OBJECT: CICSSTAT_Detail - Individual Statistic Details.
 */

struct CICSSTAT_Detail
{
    cics_ulong_t Id;
    CICSSTAT_Mesg_t Description;
    cics_slong_t Value;
};

typedef struct CICSSTAT_Detail CICSSTAT_Detail_t;

/*
 * DESCRIPTION:
 *	This structure is used as part of the CICSSTAT_Record.
 *	Each CICSSTAT_Detail record contains the :hp2.Id:ehp2.,
 *	:hp2.Description:ehp2. and :hp2.Value:ehp2. of one statistic 
 *	at the time statistics were requested. These details should 
 *	be obtained from the CICSSTAT_Record structure which is 
 *	being used to store the statistic by using either the 
 *	InfST_GetRequestedStat() or InfST_GetIntervalStat() function
 *	depending on the type of statistics required.
 *
 */

/*
 * OBJECT: CICSSTAT_Record - format of statistics record in the output medium
 */

#define CICSSTAT_ITEM_NAME_LEN	 12 

struct CICSSTAT_Record
{
    cics_ulong_t Id;
    CICSSTAT_Mesg_t Category;
    cics_char_t ItemName[CICSSTAT_ITEM_NAME_LEN + 1];
    cics_ushort_t NumStats;
    CICSSTAT_Detail_t Details[1];	/* used as variable size */

};

typedef struct CICSSTAT_Record CICSSTAT_Record_t;

/*
 * DESCRIPTION:
 *	This is the structure of the data records written to the output
 *	medium for later processing as required.
 *	:sl.
 *	:li.:hp2.Id:ehp2. identifies which set of statistics this
 *	statistic belongs to. eg: Transactions or Temporary Storage.
 *	:li.:hp2.Category:ehp2. is the key into the Statistics Message 
 *      Catalogue which identifies the piece of text which can be used 
 *      to describe this statistics set.
 *	:li.:hp2.ItemName:ehp2. is an optional field which can be used
 *	to store a name if the statistics are the details
 *	for a particular item rather than a general category. eg: the
 *	statistics record for Temporary Storage queue TS1 would store
 *	"TS1" in ItemName.
 *	:li.:hp2.NumStats:ehp2. indicates how many items
 *	there are in the :hp2.Details:ehp2. array.
 *	:hp2.Details:ehp2. is the array of statistics for this record. eg: for
 *	a Temporary Storage queue there might be one element in the array
 *	for the number of times the queue was read, one for the number of times
 *	it was written, one for the number of times a record was deleted and
 *	one for the maximum number of records in the queue.
 *	:esl.
 */

/*
 * OBJECT: CICSSTAT_MAX_STAT_DETS - defines the maximum number of statistic
 *                                  details.
 */

#define CICSSTAT_MAX_STAT_DETS 		40

/*
 * DESCRIPTION:
 *	CICSSTAT_MAX_STAT_DETS is used to control the maximum number of
 *      statistic details which can be stored in any one record.
 */

#define CICSSTAT_STATSRECORD(n)				    struct						    {								cics_ulong_t Id;						CICSSTAT_Mesg_t Category; 				cics_char_t ItemName[CICSSTAT_ITEM_NAME_LEN + 1];		cics_ushort_t NumStats;					struct CICSSTAT_Detail Details[n];		    }
/*
 * DESCRIPTION:
 * 	The CICSSTAT_STATSRECORD macro can be used to declare a stats 
 * 	record large enough for use by a particular module's use. 
 *	eg: For a module which will collect 7 statistics 
 *      CICSSTAT_STATSRECORD(7) can be used.
 */


/*
 * OBJECT: CICSSTAT_RecordList - List of statistics records.
 */

struct CICSSTAT_RecordList
{
    cics_ulong_t NumRecords;
    CICSSTAT_Record_t Records[1];	/* used as variable size */
};

/*
 * DESCRIPTION:
 *      This structure describes the format used to return 
 *      statistics information when the specified destination type 
 *      is INFST_OUTDEST_MEMORY i.e. Store statistic in memory.
 *      :sl.
 *	:li.:hp2.NumRecords:ehp2. indicates how many statistics records
 *	there are in the :hp2.Records:ehp2. array.
 *	:li.:hp2.Records:ehp2. is an array of statistics records. Each
 *      record will either contain summary, or resource specific 
 *      statistics for a requested statistics class.
 *	:esl.
 */




#endif 	/* ] 	Inclusion lock */


