/*
 * NAME:	cics_truc.h - DFHTRUC Parameters
 *
 *
 * VERSION:	1.1
 *
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 */

#if !defined(_CICS_TRUC_H)

#define _CICS_TRUC_H


#include <cicstype.h>		/* CICS-supplied datatypes */
#include <limits.h>

/*
 * OBJECT: cics_DFHTRUCParam - Transaction Routing User Conversion Parameters
 */

#define CICS_DFHTRUC_DIR_OUTBOUND   0     /* These values are used with  */
#define CICS_DFHTRUC_DIR_INBOUND    1     /* the Direction parameter.    */

struct cics_DFHTRUCParam
{
    cics_sshort_t         SysIdLength;
    cics_char_t           SysId[5];
    cics_char_t          *COMMAREA;
    cics_sshort_t         COMMAREALength;
    cics_char_t          *TCTUA;
    cics_sshort_t         TCTUALength;
    cics_sshort_t         Direction;
    cics_sshort_t         LocalCodePageLength;
    cics_char_t           LocalCodePage[_POSIX_PATH_MAX];
    cics_sshort_t         RemoteCodePageLength;
    cics_char_t           RemoteCodePage[_POSIX_PATH_MAX];
};

/*
 * DESCRIPTION:
 * Parameters passed in the COMMAREA to the Transaction Routing Use Exit
 * program, DFHTRUC.  These parameters provide the information required
 * for the user exit program to perform conversions on the TCTUA and/or 
 * COMMAREA flowed when the AOR or TOR is Transaction Routing to a 
 * remote region that does not have the same code page as the local region. 
 * A copy of this structure is passed to the user exit in the COMMAREA.
 *
 *     SysIdLength          - The length of SysId.
 *     SysId                - NULL terminated remote SYSID.
 *     COMMAREA             - A pointer to the COMMAREA to be converted.
 *     COMMAREALength       - COMMAREA length.
 *     TCTUA                - A pointer to the TCTUA to be converted.
 *     TCTUALength          - The length of TCTUA.
 *     Direction            - Conversion direction. 0 = Outbound, 1 = Inbound
 *     LocalCodePageLength  - Length of LocalCodePage.
 *     LocalCodePage        - The name of the local regions's code page
 *     RemoteCodePageLength - Length of RemoteCodePage.
 *     RemoteCodePage       - The name of the remote region's code page
 *  
 */


#endif /* _CICS_TRUC_H */

