/*
 * NAME:        Generic User Exit Sample program 
 *
 *
 * VERSION:     1.1 
 *
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * SYNOPSIS:
 *
 * cics_UE_Return_t cics_UE_entry( cics_UE_Header_t *UE_Header, 
 *			               cics_UE_Specific_t *UE_Specific)
 * DESCRIPTION:
 *  
 * This is a generic user exit sample program which illustrates -	
 *
 *      How to write a user exit program
 *       which determines the user exit for which it has been called
 *             IF it's the Task end user exit  (UE014015)
 *                then call a separate function to do task end processing
 *             ELSE IF it's the Dynamic Transaction Routing user exit (UE014025)
 *                then call a separate function to do dynamic transaction routing
 *             ELSE IF its the Dynamic DPL user exit (UE015050)
 *                then call a separate function to do dynamic DPL
 *             ELSE
 *                Process in-line based on the requirements for that user exit 
 *                and using the cics_UE_specific structure for that user exit 
 *             ENDIF
 *
 *      Points to note -
 *        There is no <main> function
 *        The entry point is cics_UE_entry, which is mandated for 
 *          some platforms as it is required when the program is loaded
 *          but can be altered on AIX
 *        The cics_UE_header field UE_TraceOnExit is always UE_No
 *          on entry to a user exit program; it should be set to UE_Yes to
 *          obtain product trace for the UE parameter lists and workarea
 *          on exit from the user exit (if product trace is active)
 *
 *     Refer to the CICS Administration Guide for guidance on
 *     writing user exit programs.
 *
 * To use this sample program -
 *     
 *    Do a make using the supplied  make file 
 *         to produce an executable called <cicsuxit>
 *
 *    Add a PD stanza entry for program CICSUXIT
 *      which has UserExitNumber=17
 *                PathName=<cicsuxit>
 *
 *     Start the CICS region
 *       Ensuring that all the RD stanza entries for dump are set to <yes>
 *       Message ERZ015042W indicates that CICSUXIT is now associated with the 
 *       Dump Request user exit (UE052017)
 *
 *    Issue CEMT PERF SNAP
 *       the dump should be by_passed due to the action of the user exit program
 *
 *    Uninstall PD stanza entry CICSUXIT
 *       message ERZ015044W indicates that CICSUXIT no longer associated with exit 
 * 
 *    Issue CEMT PERF SNAP
 *       the dump should be produced as the user exit is now inactive 
 *
 *   Modify the sample program as required for your customer installation
 *
 */

#include <stdio.h>
#include <errno.h>
#include <cicstype.h>
#include <cicsue.h>

#define UGH       "This inserted by user exit sample program before switch    "
#define CASE15    "This inserted before call user exit 15 function            "
#define CALLED    "This inserted by UE014015                                  "
#define CASE17    "This inserted by UE052017                                  "
#define CASE25    "This inserted before call user exit 25 function            "
#define ROUTE     "This inserted by UE014025                                  "
#define ROUTESEL  "UE014025 called for initial route selection                "
#define RTESELERR "UE014025 called because of a route selection error         "
#define RTEABEND  "UE014025 called because a tran abend occurred              "
#define ROUTETERM "UE014025 called because transaction successfully completed "
#define CASE50    "This inserted before call user exit 50 function            "
#define LINK      "This inserted by UE015050                                  "
#define LINKSEL   "UE015050 called for initial route selection                "
#define LINKUNKN  "UE015050 called because program not defined to CICS        "
#define LINKTERM  "UE015050 called because of a successful program link       "
#define LINKABEND "UE015050 called because of an unsuccessful program link    "
#define CASE51    "This inserted before call user exit 51 function            "
#define SYNC      "This inserted by UE016051                                  "
#define SYNCCOMM  "UE016051 called after successful syncpoint                 "
#define SYNCROLL  "UE016051 called after successful syncpoint rollback        "
#define SYNCABND  "UE016051 called after failed syncpoint/rollback            "
#define SYNCUNKN  "UE016051 called for unknown reason                         "
#define DEFAULT   "This inserted by DEFAULT                                   "


cics_UE_Return_t cics_UE_entry( cics_UE_Header_t *UE_Header,
			        cics_UE_Specific_t *UE_Specific)
{
   cics_UE_Return_t wott;
   strcpy(UE_Header->UE_Workarea,UGH);
   switch(UE_Header->UE_Number)
   {
       /*
	*  Task end user exit            UE014015
	*       
	*/
       case 15:
	    strcpy(UE_Header->UE_Workarea,CASE15);
	    wott=cics_UE014015(UE_Header,
			       (cics_UE014015_t *) UE_Specific);
	    return(wott);
	    break;
       /*
	*  Dump request user exit        UE052017
	*       
	*/
       case 17:
	    strcpy(UE_Header->UE_Workarea,CASE17);
            UE_Header->UE_TraceOnExit=UE_Yes;
	    return(UE_Bypass);
	    break;
       /*
        *  Dynamic transaction routing user exit   UE014025
        *
        */
       case 25:
            strcpy(UE_Header->UE_Workarea,CASE25);
            wott=cics_UE014025(UE_Header,
                               (cics_UE014025_t *) UE_Specific);
            return(wott);
            break;
       /*
        *  Dynamic DPL user exit     UE015050
        *
        */
       case 50:
            strcpy(UE_Header->UE_Workarea,CASE50);
            wott=cics_UE015050(UE_Header,
                               (cics_UE015050_t *) UE_Specific);
            return(wott);
            break;
       /*
        *  Syncpoint user exit     UE016051
        *
        */
       case 51:
            strcpy(UE_Header->UE_Workarea,CASE51);
            wott=cics_UE016051(UE_Header,
                               (cics_UE016051_t *) UE_Specific);
            return(wott);
            break;

       default:
	    strcpy(UE_Header,DEFAULT);
            UE_Header->UE_TraceOnExit=UE_No;
	    return(UE_Normal);
	    break;
    }
}


cics_UE_Return_t cics_UE014015(
			 cics_UE_Header_t *UE_Header,
			 cics_UE014015_t *UE_Specific)
{
    strcpy(UE_Header->UE_Workarea,CALLED);
    if (UE_Specific->UE_Terminationtype ==UE_Normaltermination)
    {
       UE_Header->UE_TraceOnExit=UE_Yes;
    }
    return(UE_Normal);
}

cics_UE_Return_t cics_UE014025(
                         cics_UE_Header_t *UE_Header,
                         cics_UE014025_t *UE_Specific)
{
    strcpy(UE_Header->UE_Workarea,ROUTE);
    UE_Header->UE_TraceOnExit=UE_Yes;

    switch (UE_Specific->UE_Dyrfunc)
    {
        case (UE_ROUTESEL) :   strcpy(UE_Header->UE_Workarea,ROUTESEL);
                               break;
        case (UE_RTESELERR):   strcpy(UE_Header->UE_Workarea,RTESELERR);
                               break;
        case (UE_ROUTETERM):   strcpy(UE_Header->UE_Workarea,ROUTETERM);
                               break;
        case (UE_RTEABEND) :   strcpy(UE_Header->UE_Workarea,RTEABEND);
                               break;
    }

    return(UE_Normal);
}


cics_UE_Return_t cics_UE015050(
                         cics_UE_Header_t *UE_Header,
                         cics_UE015050_t *UE_Specific)
{
    strcpy(UE_Header->UE_Workarea,LINK);
    UE_Header->UE_TraceOnExit=UE_Yes;

    switch (UE_Specific->UE_Dplfunc)
    {
        case (UE_LINKSEL)     : strcpy(UE_Header->UE_Workarea,LINKSEL);
                                break;
        case (UE_LINKUNKNOWN) : strcpy(UE_Header->UE_Workarea,LINKUNKN);
                                break;
        case (UE_LINKTERM)    : strcpy(UE_Header->UE_Workarea,LINKTERM);
                                break;
        case (UE_LINKABEND)   : strcpy(UE_Header->UE_Workarea,LINKABEND);
                                break;
    }

    return(UE_Normal);
}


cics_UE_Return_t cics_UE016051(
                         cics_UE_Header_t *UE_Header,
                         cics_UE016051_t *UE_Specific)
{
    strcpy(UE_Header->UE_Workarea,SYNC);
    UE_Header->UE_TraceOnExit=UE_Yes;

    switch (UE_Specific->UE_SyncResp)
    {
        case (UE_COMMITTED)  : strcpy(UE_Header->UE_Workarea,SYNCCOMM);
                                break;
        case (UE_ROLLEDBACK) : strcpy(UE_Header->UE_Workarea,SYNCROLL);
                                break;
        case (UE_ABEND)      : strcpy(UE_Header->UE_Workarea,SYNCABND);
                                break;
        default              : strcpy(UE_Header->UE_Workarea,SYNCUNKN);
                                break;
    }

    return(UE_Normal);
}
