#@ NAME:    migratescript.t
#@
#@ MODULE:  11
#@ VERSION: 1.3
#@
#@ DESCRIPTION:
#@
#@ This is the template file which is processed by the cicsmigrate
#@ script to generate the script which is run by the user to migrate
#@ a region database.
#@
#@ Tags have the following form %LABEL%
#@    %REGION%     - will get replaced by the region name
#@    %VERSION%    - will get replaced by the current cics version
#@    %LOGFILE%    - will be replaced with the name of the logfile
#@                   specified.
#@    %SCRIPTNAME% - will get replaced by the name of the generated
#@                   script
#@    %BINPATH%    - will get replaced with the path to the directory
#@                   containing the original cicsmigrate command
#@
#@ Lines starting with '#@' are comment lines which will stripped out
#@ by the cicmigrate script.
#@
#!/bin/ksh
######################################################################
#
# NAME: %SCRIPTNAME%
#
# DESCRIPTION: This script has been generated to perform the
Details:R:9999:#              migration of the cics region %REGION%
Details:M:9999:#              migration of all cics regions
Details:M:9999:#              and the ...
Details:S:9999:#              migration of all SFS Servers
Details:M:9999:#              and the ...
Details:P:9999:#              migration of all PPC Gateway Servers
Details:M:9999:#              and the ...
Details:C:9999:#              migration of the default SCD stanza
#
#              Before running the script you should read over the
#              changes which are going to be made. Only then should
#              you run the script with the command ;
#                 'ksh %SCRIPTNAME%'
#
#              The script performs the following tasks.
#
#              1. Attribute additions, deletions, renaming
#
#              2. Attribute value updateing
#
#              3. DCE Names migration
#
#              4. New supplied stanzas
#
######################################################################

######################################################################
# Definitions
# DO NOT CHANGE THIS SECTION
######################################################################
MCATNAME="cics.cat"     # message catalogue
typeset -r MSET=46      # message set number

typeset -r MSN_475=0475
typeset -r MSN_476=0476
typeset -r MSN_477=0477
#
# Import messages and message-ids
#
ERZ4602E=2
ERZ4602E_TEXT='"ERZ046002E: Unable to access region '%1\$s'"'
ERZ46144I=134
ERZ46144I_TEXT='"ERZ046144I: Unable to write to file '%1\$s'"'
ERZ46146E=136
ERZ46146E_TEXT='"ERZ046146E: Unable to add the region principal to the DCE group 'cics_regions'"'

#
# Module what string
#
#@(#)regrm,22:09:15,Apr 08 1998,s211-L980223
#

unset OUTPUT
unset PRINCIPAL

typeset OUTPUT
typeset PRINCIPAL

######################################################################
# Script Initialisation
# DO NOT CHANGE THIS SECTION
######################################################################
# Leave this section unchanged. This just does some initialisation
PROGNAME=${0##*/}
BINDIR=$(cicsname bindir)
LIBDIR=$(cicsname libdir)
SCDDIR=$(cicsname sfsschemadir)
CICSREGIONS=$(cicsname regions)

# Get the script portability layer ...
. $LIBDIR/cicsportable.fnc
# ... and the common functions
. $LIBDIR/cicsadm.fnc
# ... and the migration functions
. $LIBDIR/cicsmig.fnc
# ... and the 1.05 to 1.1 server migration functions
. $LIBDIR/cicsmigserv.fnc


# Check we can write to the log file
touch %LOGFILE% 2> /dev/null
if [ $? != 0 ]
then
   # Display error message and exit
   GeneralDispMsg $MSN_475 ERZ46144I "%LOGFILE%"
   exit 1
fi

#
# Make sure things are tidy if we get signals 1 2 13 and 15
#
trap '
    stty sane                       # reset a reasonable config if we exited
                                    # when entering the password
    exit 1
' 1 2 13 15

# Check the user is logged into the DCE.
CheckLoginAndDce
if [ $? -ne 0 ]
then
  exit 1
fi

# Redirect all standard output to the logfile but leave standard error
exec 1>> %LOGFILE%

# set up a temporary region tree for use by cicscopystanza
# temporary region database is ($BINDIR/cicsname regions)/mig#temp/database
CreateTempMigrationDir
if [ $? -ne 0 ]
then
  exit 1
fi


Details:R:9999:##################################################################
Details:R:9999:##################################################################
Details:R:9999:# Migrate Regions
Details:R:9999:##################################################################
Details:R:9999:##################################################################
Details:M:9999:# For every Region defined in the system
Details:M:9999:for RegionName in $($BINDIR/cicslist | cut -f1)
Details:M:9999:do
Details:M:9999:if [ $RegionName = "mig#temp" ]
Details:M:9999:then
Details:M:9999:   continue            #with next for value
Details:M:9999:fi
Details:M:9999:echo $RegionName
Details:R:4:#
Details:R:4:# Rename CProgramCacheSize to ProgramCacheSize
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% -crd \
                                     -a CProgramCacheSize \
                                     -u ProgramCacheSize
Details:R:4:#
Details:R:4:#Add NameService
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% -crd \
                    -a NameService -n DCE
Details:R:4:#
Details:R:4:#Add AuthenticationService
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% -crd \
                    -a AuthenticationService -n DCE
Details:R:4:#
Details:R:4:#Add AllowDebugging to the RD
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% -crd \
                    -a AllowDebugging -n no
Details:R:4:#
Details:R:4:#Add ECIPasswordCacheSeconds to the RD
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% -crd \
                    -a ECIPasswordCacheSeconds -n 28800
Details:R:4:#
Details:R:4:# Remove EnabledAtShutDown, no longer required.
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% -ctd \
		    -a EnabledAtShutdown -d
Details:R:4:#
Details:R:4:#
Details:R:4:# Update the currect CICS Release Number in the Region
Details:R:4:# Definition from "0120" to "0210".  Do not test for the
Details:R:4:# current value as the user should not have changed it.
Details:R:4:$BINDIR/cicsupdateclass -w -r %REGION% -crd -a \
				     ReleaseNum -n \\\"0210\\\"
Details:R:4:#
Details:R:5:#
Details:R:5:#
Details:R:5:# Update the current CICS release number in the region 
Details:R:5:# definition from "0210" to "0211". Do not test for the 
Details:R:5:# current value as the user should not have changed it.
Details:R:5:$BINDIR/cicsupdateclass -w -r %REGION% -crd -a \
				     ReleaseNum -n \\\"0211\\\"
Details:R:5:#
Details:R:5:#
Details:R:5:# Update the default values of SysTraceBufferSize and
Details:R:5:# TraceFileSize in the region definition.
Details:R:5:$BINDIR/cicsupdateclass -w -r %REGION% -crd -a \
				SysTraceBufferSize -n 163840 -eq 16384
Details:R:5:# Update the default values of SysTraceBufferSize and
Details:R:5:# TraceFileSize in the region definition.
Details:R:5:$BINDIR/cicsupdateclass -w -r %REGION% -crd -a \
				TraceFileSize -n 3276800 -eq 163840
Details:R:5:$BINDIR/cicsupdateclass -w -r %REGION% -cwd -a \
                                TermSubType -n 242
Details:R:5:#
Details:R:4:# Add ListenerName, RemoteTCPAddress, RemoteTCPPort attributes to
Details:R:4:# CD entry
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% \
                            -c cd \
                            -a ListenerName \
                            -n \\\"\\\"
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% \
                            -c cd \
                            -a RemoteTCPAddress \
                            -n \\\"\\\"
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% \
                            -c cd \
                            -a RemoteTCPPort \
                            -n 1435
Details:R:4:#
Details:R:4:# Change RemoteSysType=SNA to RemoteSysType=ppc_gateway
Details:R:4:$BINDIR/cicsupdateclass -w -r %REGION% \
                            -c cd \
                            -a RemoteSysType \
                            -n ppc_gateway \
                            -eq SNA
Details:R:4:#
Details:R:4:# Change RemoteSysType=TCP to RemoteSysType=ppc_tcp
Details:R:4:$BINDIR/cicsupdateclass -w -r %REGION% \
                            -c cd \
                            -a RemoteSysType \
                            -n ppc_tcp \
                            -eq TCP
Details:R:4:#
Details:R:4:# Change RemoteSysType to ConnectionType
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% \
                            -c cd \
                            -a RemoteSysType \
                            -u ConnectionType
Details:R:5:#
Details:R:5:# Add DefaultSNAModeName to CD entry
Details:R:5:$BINDIR/cicsmigrateclass -w -r %REGION% \
			    -c cd \
		       	    -a DefaultSNAModeName \
			    -n \\\"\\\"
Details:R:5:#
Details:R:4:#
Details:R:4:# Update the value of ConnectionType based on its current value and
Details:R:4:# the value of GatewayName.
Details:R:4:CDENTRIES=$($BINDIR/cicsget -c cd -r %REGION% -l | cut -c1-5)
Details:R:4:for KEY in $CDENTRIES
Details:R:4:do
Details:R:4:    #
Details:R:4:    # Get value of ConnectionType and GatewayName
Details:R:4:    #
Details:R:4:    CONNECTIONTYPE=$($BINDIR/cicsgetattribute -r %REGION% \
                                              -c cd \
                                              -k $KEY \
                                              -a ConnectionType)
Details:R:4:    GATEWAYNAME=$($BINDIR/cicsgetattribute -r %REGION% \
                                           -c cd \
                                           -k $KEY \
                                           -a GatewayName)
Details:R:4:    #
Details:R:4:    # Alter ConnectionType=ppc_gateway to ConnectionType=local_sna
Details:R:4:    # if GatewayName=""
Details:R:4:    #
Details:R:4:    if [ $CONNECTIONTYPE = "ppc_gateway" ]
Details:R:4:    then
Details:R:4:        if [ -z "$GATEWAYNAME" -o "$GATEWAYNAME" = "\\\"\\\"" ]
Details:R:4:        then
Details:R:4:            $BINDIR/cicsupdateclass -w -r %REGION% \
                                       -c cd \
                                       -a ConnectionType \
                                       -n local_sna \
                                       -k $KEY
Details:R:4:        fi
Details:R:4:    fi
Details:R:4:done
Details:R:4:#
Details:R:4:#
Details:R:4:# Add a new definition for the new CDCN transaction
Details:R:4:$BINDIR/cicscopystanza -w -ctd -k CDCN -s mig#temp -t %REGION%
Details:R:4:#
Details:R:4:# Add a new definition for the new CCIN log
Details:R:4:$BINDIR/cicscopystanza -w -ctdd -k CCIN -s mig#temp -t %REGION%
Details:R:4:#
Details:R:4:# Add a new definition for the new DFHCDCN0 program
Details:R:4:$BINDIR/cicscopystanza -w -cpd -k DFHCDCN0 -s mig#temp -t %REGION%
Details:R:4:#
Details:R:4:# Add a new definition for the new DFHCDCN mapset
Details:R:4:$BINDIR/cicscopystanza -w -cpd -k DFHCDCN -s mig#temp -t %REGION%
Details:R:4:# Add a new definition for the new MD18 and MD19 terminals.
Details:R:4:# The MD18 and MD19 definitions may already be present so this
Details:R:4:# may produce a warning.
Details:R:4:$BINDIR/cicscopystanza -w -cwd -k MD18 -s mig#temp -t %REGION%
Details:R:4:$BINDIR/cicscopystanza -w -cwd -k MD19 -s mig#temp -t %REGION%
Details:R:5:#
Details:R:5:# Add new definitions for the new MD20-26 terminals
Details:R:5:$BINDIR/cicscopystanza -w -cwd -k MD20 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cwd -k MD21 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cwd -k MD22 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cwd -k MD23 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cwd -k MD24 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cwd -k MD25 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cwd -k MD26 -s mig#temp -t %REGION%
Details:R:4:#
Details:R:4:# Add a new definition for the new CICE transaction
Details:R:4:$BINDIR/cicscopystanza -w -ctd -k CICE -s mig#temp -t %REGION%
Details:R:5:#
Details:R:5:# Add a new definitions for the new CSTD transaction
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CSTD -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CST0 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CST1 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CST2 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CST3 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CST4 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CST5 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CST6 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CST7 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CST8 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CST9 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CSTA -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CSTB -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -ctd -k CSTZ -s mig#temp -t %REGION%
Details:R:5:#
Details:R:5:# Add a new definition for the new DFHCSTM mapset
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCSTM -s mig#temp -t %REGION%
Details:R:5:#
Details:R:5:# Add a new definition for the new DFHCST.. programs
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST0 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST01 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST02 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST03 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST04 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST05 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST06 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST07 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST08 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST09 -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST0A -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST0B -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST0X -s mig#temp -t %REGION%
Details:R:5:$BINDIR/cicscopystanza -w -cpd -k DFHCST0Z -s mig#temp -t %REGION%
Details:R:4:$BINDIR/cicsmigrateclass  -r %REGION% -w -crd \
                        -a FileSystemType -n SFS
Details:R:4:$BINDIR/cicsmigrateclass  -r %REGION% -w -crd \
                        -a RDBMSInstance -n ""
Details:R:4:#
Details:R:4:#Add CICSPassword
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% -cud \
                    -a CICSPassword -n \\\"\\\"
Details:R:4:#
Details:R:4:#Add PrePages
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% -cfd \
                    -a PrePages -n 1
Details:R:4:#
Details:R:4:#Remove UseDebugger from the UD entries
Details:R:4:$BINDIR/cicsmigrateclass -w -r %REGION% -cud \
                    -a UseDebugger -d
Details:R:4:#
Details:R:4:#Rename XAD.auto to XAD.emer
Details:R:4:#Let any error be reported. This may just be becuase the region has
Details:R:4:#not been started yet and there is no auto file.
Details:R:4:mv $($BINDIR/cicsname -r %REGION% regiondbase)/XAD/XAD.%REGION%/XAD.auto \
	       $($BINDIR/cicsname -r %REGION% regiondbase)/XAD/XAD.%REGION%/XAD.emer 
Details:R:4:#
Details:R:4:#Add links from En_US maps to en_US maps
Details:R:4:MigrateEnUSMaps %REGION%
Details:R:9999:# Mark all stanzas with current version stamp
Details:R:9999:for MC in fd jd md pd tdd tsd xad cd rd td ud wd ld gd
Details:R:9999:do
Details:R:9999:$BINDIR/cicsmigrateclass -w -r %REGION% -c $MC
Details:R:9999:done
Details:M:9999:done

Details:S:9999:##################################################################
Details:S:9999:##################################################################
Details:S:9999:# Migrate SFS Servers
Details:S:9999:##################################################################
Details:S:9999:##################################################################
Details:S:4:#Add NameService
Details:S:4:$BINDIR/cicsmigrateclass -w -c ssd -a NameService -n DCE
Details:S:4:#
Details:S:9999:# Migrate version stamp in SSD stanza
Details:S:9999:$BINDIR/cicsmigrateclass -w -c ssd
Details:S:9999:#
Details:S:9999:# Migrate sfsserver SRC entries : change of path for sfs program
Details:S:9999:MigrateSfsPathSRC
Details:S:9999:#
Details:C:9999:#################################################################
#
Details:C:9999:# Migrate the default SCD Server
Details:C:9999:#################################################################
#
Details:C:9999:#################################################################
#
Details:C:9999:# Migrate version stamp in SCD stanza
Details:C:9999:$BINDIR/cicsmigrateclass -w -c scd -f $SCDDIR/SCD.stanza

Details:P:9999:##################################################################
Details:P:9999:##################################################################
Details:P:9999:# Migrate PPC Gateway Servers
Details:P:9999:##################################################################
Details:P:4:#
Details:P:4:#Add NameService
Details:P:4:$BINDIR/cicsmigrateclass -w -c gsd -a NameService -n DCE
Details:P:9999:$BINDIR/cicsmigrateclass -w -c gsd

Details:M:4:#
Details:M:4:#Modify ACLs for /.:/cics/... to allow unauthenticated reads
Details:M:4:SetCICSUnauthenticatedAccess

# Clean up temp regions
RemoveTempMigrationDir
if [ $? -ne 0 ]
then
    exit 1
fi


# End
exit 0
