#!/bin/ksh
#
# NAME: General Shell functions
#
#
# VERSION: 1.5
#
#
# COPYRIGHT:                                                        
# 
#   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
#   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
#   All Rights Reserved
#   Licensed Materials - Property of IBM
#  
#   US Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
#                                                                   
#
# SYNOPSIS:
#
# GeneralLockOptions [LSD|SSD|GSD] PID [-l|-t|-u] Server
#
# GeneralTryLock [LSD|SSD|GSD] PID Server
#
# GeneralUnlock [LSD|SSD|GSD] PID Server 
#
# GeneralLock [LSD|SSD|GSD] PID Server
#
# GeneralDispMsg MSN MSG_NO [parameters]
#
# WhatServerDir [LSD|SSD|GSD] ServerDCEName
#
# NormaliseServer ServerDCEName
#
# GeneralDCEAdmin LSD|SSD|GSD -s server -S shortname { -u user -A|-D} [-I] [-v]
#
# GeneralMkDir Directory Verbosity
#
# GeneralCheckSRC LSD|SSD|GSD ShortName ServerName
#
# GeneralCheckUser W|E UserId
#
# GeneralCheckDir Dir User Perms
#
# GeneralChangeOwner User Dir
#
# GeneralProtStringToNumber ProtLevel
#
# CheckStartServer server_name server_type prot_level verbose ignore_errors
#
# CheckStartPreReqs server_name server_type short_name
#
# StartErrorExit server_name server_running?
#
# CreateLog server_name log_volume cics_raw_log_path logfile userid verbose
#
# MkArchiveDir server_name userid verbose
#
# CheckLog server_name log_volume verbose
#
# SetMediaArchive server_name enable_mra start_type verbose
#
# DESCRIPTION:
#
# :hp2.GeneralLockOptions:ehp2. parses the -l|-t|-u options and calls the
# appropriate lock function.
#
# :hp2.GeneralTryLock:ehp2. takes the
# /var/cics_servers/[LSD|SSD|GSD]/<server_dir>/lock and returns 1 if it cannot
#
# :hp2.GeneralUnlock:ehp2. unlocks the
# /var/cics_servers/[LSD|SSD|GSD]/<server_dir>/lock.
#
# :hp2.Generalock:ehp2. takes the
# /var/cics_servers/[LSD|SSD|GSD]/<server_dir>/lock.
#
# :hp2.GeneralDispMsg:ehp2. displays a message from a catalogue.
#
# :hp2.WhatServerDir:ehp2. returns /var/cics_servers/[LSD|SSD|GSD]/ServerDir.
#
# :hp2.WhatArchiveDir:ehp2. returns /var/cics_servers/archives/<server-name>
#
# :hp2.NormaliseServer:ehp2. removes the prefixed /.:/ or /.../ from the
# server name.
#
# :hp2.GeneralDCEAdmin:ehp2. is a general purpose script to administer DCE
# server # details. It adds or deletes (-A, -D) a principal, account and a
# keytab file.  # The keytab file resides in
# /var/cics_servers/LSD|SSD|GSD/<...>/key. The <...> is derived from the
# server name. E.g. /.:/cics/sfs/blah gives the subdirectory cics/sfs/blah.
# The principal name for the server is derived from the server name with
# the prefixed /.:/, /.../ removed.
#
# :hp2.GeneralMkDir:ehp2. does a mkdir -P of the named directory, and sets
# its mode # (chmod), if it does not already exist. The mode defaults to 770
# if one is not supplied.
#
# :hp2.GeneralCheckSRC:ehp2. makes sure that the SRC entry for the supplied
# short name agrees with the server name given.
#
# :hp2.GeneralCheckUser:ehp2. makes sure the given user exists and that the home
# directory is readable, writable and executable by him. The W|E defines
# whether an error or warning is generated. It also effects the return code.
#
# :hp2.GeneralCheckDir:ehp2. returns 0 if the User has Perms for Dir.
#
# :hp2.GeneralChangeOwner:ehp2. chown's the directory to the supplied user id.
# It logs a message if this is not possible.
#
# :hp2.GeneralProtStringToNumber:ehp2. converts a protection level from
# string format to number format. Returns -1 if the protection level is
# not recognised.
#
# :hp2.CheckStartServer:ehp2. during auto start of a server check that the
# required files / directories are available
#
# :hp2.CheckStartPreReqs:ehp2. during a server start check that the pre-req
# conditions are met - able to exec startsrc etc.
#
# :hp2.StartErrorExit:ehp2. Exit cleanly from a 'Start Encina Server' routine
#
# :hp2.CreateLog:ehp2. Initialize an Encina Server's log
#
# :hp2.MkArchiveDir:ehp2. Create archive directory for Encina Server
#
# :hp2.CheckLog:ehp2. server_name log_volume verbose
#
# :hp2.SetMediaArchive:ehp2. Enable/Disable Media Recovery Archiving
#
# CAVEATS/WARNINGS:
# :p.These functions are private to AdmLG, AdmGY and AdmSF. They are to be
# included by scripts wishing to use them.
# :p.Note that MSET should be set to 36 (AdmLGs message set number) when
# directly using these functions to ensure the correct messages come out.
# :p.GeneralCheckDir assumes the user supplied exists.
#
# RETURNED VALUES:
#
# :dl tsize=5.
# :dt.0
# :dd.Successful
# :dt.1
# :dd.Unsuccessful
# :edl.
#
# RELATED INFORMATION:
# None
#
#*/

unset ENV               # Don't allow 'echos' to confuse parameter passing

MSN_START=04
typeset -r ADMLG_FUNCS_MSN_1=${MSN_START}01
typeset -r ADMLG_FUNCS_MSN_2=${MSN_START}02
typeset -r ADMLG_FUNCS_MSN_3=${MSN_START}03
typeset -r ADMLG_FUNCS_MSN_4=${MSN_START}04
typeset -r ADMLG_FUNCS_MSN_5=${MSN_START}05
typeset -r ADMLG_FUNCS_MSN_6=${MSN_START}06
typeset -r ADMLG_FUNCS_MSN_7=${MSN_START}07
typeset -r ADMLG_FUNCS_MSN_8=${MSN_START}08
typeset -r ADMLG_FUNCS_MSN_9=${MSN_START}09
typeset -r ADMLG_FUNCS_MSN_10=${MSN_START}10
typeset -r ADMLG_FUNCS_MSN_11=${MSN_START}11
typeset -r ADMLG_FUNCS_MSN_12=${MSN_START}12
typeset -r ADMLG_FUNCS_MSN_13=${MSN_START}13
typeset -r ADMLG_FUNCS_MSN_14=${MSN_START}14
typeset -r ADMLG_FUNCS_MSN_15=${MSN_START}15
typeset -r ADMLG_FUNCS_MSN_16=${MSN_START}16
typeset -r ADMLG_FUNCS_MSN_17=${MSN_START}17
typeset -r ADMLG_FUNCS_MSN_18=${MSN_START}18
typeset -r ADMLG_FUNCS_MSN_19=${MSN_START}19
typeset -r ADMLG_FUNCS_MSN_20=${MSN_START}20
typeset -r ADMLG_FUNCS_MSN_21=${MSN_START}21
typeset -r ADMLG_FUNCS_MSN_22=${MSN_START}22
typeset -r ADMLG_FUNCS_MSN_23=${MSN_START}23
typeset -r ADMLG_FUNCS_MSN_24=${MSN_START}24
typeset -r ADMLG_FUNCS_MSN_25=${MSN_START}25
typeset -r ADMLG_FUNCS_MSN_26=${MSN_START}26
typeset -r ADMLG_FUNCS_MSN_27=${MSN_START}27
typeset -r ADMLG_FUNCS_MSN_28=${MSN_START}28
typeset -r ADMLG_FUNCS_MSN_29=${MSN_START}29
typeset -r ADMLG_FUNCS_MSN_30=${MSN_START}30
typeset -r ADMLG_FUNCS_MSN_31=${MSN_START}31
typeset -r ADMLG_FUNCS_MSN_32=${MSN_START}32
typeset -r ADMLG_FUNCS_MSN_33=${MSN_START}33
typeset -r ADMLG_FUNCS_MSN_34=${MSN_START}34
typeset -r ADMLG_FUNCS_MSN_35=${MSN_START}35
typeset -r ADMLG_FUNCS_MSN_36=${MSN_START}36
typeset -r ADMLG_FUNCS_MSN_37=${MSN_START}37
typeset -r ADMLG_FUNCS_MSN_38=${MSN_START}38
typeset -r ADMLG_FUNCS_MSN_39=${MSN_START}39
typeset -r ADMLG_FUNCS_MSN_40=${MSN_START}40
typeset -r ADMLG_FUNCS_MSN_41=${MSN_START}41
typeset -r ADMLG_FUNCS_MSN_42=${MSN_START}42
typeset -r ADMLG_FUNCS_MSN_43=${MSN_START}43
typeset -r ADMLG_FUNCS_MSN_44=${MSN_START}44
typeset -r ADMLG_FUNCS_MSN_45=${MSN_START}45
typeset -r ADMLG_FUNCS_MSN_46=${MSN_START}46
typeset -r ADMLG_FUNCS_MSN_47=${MSN_START}47
typeset -r ADMLG_FUNCS_MSN_48=${MSN_START}48
typeset -r ADMLG_FUNCS_MSN_49=${MSN_START}49
typeset -r ADMLG_FUNCS_MSN_50=${MSN_START}50
typeset -r ADMLG_FUNCS_MSN_52=${MSN_START}52
typeset -r ADMLG_FUNCS_MSN_53=${MSN_START}53
typeset -r ADMLG_FUNCS_MSN_54=${MSN_START}54
typeset -r ADMLG_FUNCS_MSN_55=${MSN_START}55
typeset -r ADMLG_FUNCS_MSN_56=${MSN_START}56
typeset -r ADMLG_FUNCS_MSN_57=${MSN_START}57
typeset -r ADMLG_FUNCS_MSN_58=${MSN_START}58
typeset -r ADMLG_FUNCS_MSN_59=${MSN_START}59
typeset -r ADMLG_FUNCS_MSN_60=${MSN_START}60
typeset -r ADMLG_FUNCS_MSN_61=${MSN_START}61
typeset -r ADMLG_FUNCS_MSN_62=${MSN_START}62
typeset -r ADMLG_FUNCS_MSN_63=${MSN_START}63
typeset -r ADMLG_FUNCS_MSN_64=${MSN_START}64
typeset -r ADMLG_FUNCS_MSN_65=${MSN_START}65
typeset -r ADMLG_FUNCS_MSN_66=${MSN_START}66
typeset -r ADMLG_FUNCS_MSN_67=${MSN_START}67
typeset -r ADMLG_FUNCS_MSN_68=${MSN_START}68
typeset -r ADMLG_FUNCS_MSN_69=${MSN_START}69
typeset -r ADMLG_FUNCS_MSN_70=${MSN_START}70
typeset -r ADMLG_FUNCS_MSN_71=${MSN_START}71
typeset -r ADMLG_FUNCS_MSN_72=${MSN_START}72
typeset -r ADMLG_FUNCS_MSN_73=${MSN_START}73
typeset -r ADMLG_FUNCS_MSN_74=${MSN_START}74
typeset -r ADMLG_FUNCS_MSN_75=${MSN_START}75
typeset -r ADMLG_FUNCS_MSN_76=${MSN_START}76
typeset -r ADMLG_FUNCS_MSN_77=${MSN_START}77
typeset -r ADMLG_FUNCS_MSN_78=${MSN_START}78
typeset -r ADMLG_FUNCS_MSN_79=${MSN_START}79
typeset -r ADMLG_FUNCS_MSN_80=${MSN_START}80
typeset -r ADMLG_FUNCS_MSN_81=${MSN_START}81
typeset -r ADMLG_FUNCS_MSN_82=${MSN_START}82
typeset -r ADMLG_FUNCS_MSN_83=${MSN_START}83

ERZ3607E=2
ERZ3607E_TEXT='"ERZ036007E: Unable to start server '%1\$s'"'
ERZ3608E=3
ERZ3608E_TEXT='"ERZ036008E: Unable to access server definitions as locked with file '%1\$s'"'
ERZ3609W=4
ERZ3609W_TEXT='"ERZ036009W: Waiting for lock"'
ERZ3610I=5
ERZ3610I_TEXT='"ERZ036010I: Principal for server '%1\$s' has been added"'
ERZ3611I=6
ERZ3611I_TEXT='"ERZ036011I: Account for server '%1\$s' has been added"'
ERZ3612I=7
ERZ3612I_TEXT='"ERZ036012I: Keytab file for server '%1\$s' has been created"'
ERZ3613I=8
ERZ3613I_TEXT='"ERZ036013I: Principal for server '%1\$s' has been removed"'
ERZ3614I=9
ERZ3614I_TEXT='"ERZ036014I: Account for server '%1\$s' has been removed"'
ERZ3616E=10
ERZ3616E_TEXT='"ERZ036016E: CDS entry for server '%1\$s' was not found"'
ERZ3617I=11
ERZ3617I_TEXT='"ERZ036017I: Keytab file for server '%1\$s' has been removed"'
ERZ3634E=12
ERZ3634E_TEXT='"ERZ036034E: Unable to execute '%1\$s'"'
ERZ3636E=13
ERZ3636E_TEXT='"ERZ036036E: Either DCE is not started or you have not logged into the DCE"'
ERZ3650E=14
ERZ3650E_TEXT='"ERZ036050E: Unable to contact host '%1\$s'"'
ERZ3652W=15
ERZ3652W_TEXT='"ERZ036052W: Lock file '%1\$s' was not found"'
ERZ3653E=16
ERZ3653E_TEXT='"ERZ036053E: Unable to create lock file '%1\$s'"'
ERZ3654E=17
ERZ3654E_TEXT='"ERZ036054E: Unable to remove lock file '%1\$s'"'
ERZ3656E=18
ERZ3656E_TEXT='"ERZ036056E: Unable to change DCE account for server '%1\$s'"'
ERZ3657W=19
ERZ3657W_TEXT='"ERZ036057W: Principal not found in DCE registry for server '%1\$s'"'
ERZ3659W=20
ERZ3659W_TEXT='"ERZ036059W: Keytab file '%1\$s' for server '%2\$s' does not exist"'
ERZ3660W=21
ERZ3660W_TEXT='"ERZ036060W: Subsystem definition for ShortName '%1\$s' is already used by server '%2\$s'"'
ERZ3661W=22
ERZ3661W_TEXT='"ERZ036061W: Account found in DCE registry for server '%1\$s' - password reset"'
ERZ3662E=23
ERZ3662E_TEXT='"ERZ036062E: Account found in DCE registry for server '%1\$s'"'
ERZ3663W=24
ERZ3663W_TEXT='"ERZ036063W: Principal found in DCE registry for server '%1\$s'"'
ERZ3664E=25
ERZ3664E_TEXT='"ERZ036064E: Principal found in DCE registry for server '%1\$s'"'
ERZ3665W=26
ERZ3665W_TEXT='"ERZ036065W: Keytab file '%1\$s' for server '%2\$s' already exists"'
ERZ3666E=27
ERZ3666E_TEXT='"ERZ036066E: Keytab file '%1\$s' for server '%2\$s' already exists"'
ERZ3667E=28
ERZ3667E_TEXT='"ERZ036067E: Unable to add account to DCE registry for server '%1\$s'"'
ERZ3668E=29
ERZ3668E_TEXT='"ERZ036068E: Unable to add principal to DCE registry for server '%1\$s'"'
ERZ3669E=30
ERZ3669E_TEXT='"ERZ036069E: Unable to initialize keytab file '%1\$s' for server '%2\$s'"'
ERZ3670E=31
ERZ3670E_TEXT='"ERZ036070E: Unable to remove account from DCE registry for server '%1\$s'"'
ERZ3671E=32
ERZ3671E_TEXT='"ERZ036071E: Unable to remove principal from DCE registry for server '%1\$s'"'
ERZ3672E=33
ERZ3672E_TEXT='"ERZ036072E: Unable to remove keytab file '%1\$s' for server '%2\$s'"'
ERZ3683E=34
ERZ3683E_TEXT='"ERZ036083E: Unable to create directory '%1\$s'"'
ERZ36126I=36
ERZ36126I_TEXT='"ERZ036126I: Directory '%1\$s' created"'
ERZ36146W=40
ERZ36146W_TEXT='"ERZ036146W: Unable to access details on user '%1\$s'"'
ERZ36147E=41
ERZ36147E_TEXT='"ERZ036147E: Unable to access details on user '%1\$s'"'
ERZ36148W=42
ERZ36148W_TEXT='"ERZ036148W: Directory '%1\$s' is not accessible by user '%2\$s' or does not exist"'
ERZ36149E=43
ERZ36149E_TEXT='"ERZ036149E: Directory '%1\$s' is not accessible by user '%2\$s' or does not exist"'
ERZ36150E=44
ERZ36150E_TEXT='"ERZ036150E: Unable to change ownership of keytab file '%1\$s' to user '%2\$s'"'
ERZ36152E=45
ERZ36152E_TEXT='"ERZ036152E: Encina server '%1\$s' started, but has not been initialized"'
ERZ36155E=46
ERZ36155E_TEXT='"ERZ036155E: Your current DCE principal '%1\$s' is incorrect"'
ERZ36156E=47
ERZ36156E_TEXT='"ERZ036156E: Unsuccessful attempt at changing owner of '%1\$s' to '%2\$s'"'
ERZ36157E=48
ERZ36157E_TEXT='"ERZ036157E: Subsystem definition '%1\$s' cannot be found"'
ERZ36190W=54
ERZ36190W_TEXT='"ERZ036190W: File ACLs may need manually changing because server ProtectionLevel changed from '%1\$s' to '%2\$s'"'
ERZ36191E=55
ERZ36191E_TEXT='"ERZ036191E: Both '%1\$s' and '%2\$s' are missing for Encina server '%3\$s'"'
ERZ36192W=56
ERZ36192W_TEXT='"ERZ036192W: Restart file '%1\$s' is missing for server '%2\$s'"'
ERZ36194E=57
ERZ36194E_TEXT='"ERZ036194E: Keytab file '%1\$s' for server '%2\$s' does not exist"'
ERZ36195W=58
ERZ36195W_TEXT='"ERZ036195W: The archives directory '%1\$s' for server '%2\$s' does not exist"'
ERZ36196E=59
ERZ36196E_TEXT='"ERZ036196E: The archives directory '%1\$s' for server '%2\$s' does not exist"'
ERZ36197I=60
ERZ36197I_TEXT='"ERZ036197I: ProtectionLevel checked for server '%1\$s'"'
ERZ36198I=61
ERZ36198I_TEXT='"ERZ036198I: Restart files located for server '%1\$s'"'
ERZ36199I=62
ERZ36199I_TEXT='"ERZ036199I: Keytab file checked for server '%1\$s'"'
ERZ36200I=63
ERZ36200I_TEXT='"ERZ036200I: Archives directory located for server '%1\$s'"'
ERZ36202E=64
ERZ36202E_TEXT='"ERZ036202E: Server '%1\$s' is already running"'
ERZ36203E=65
ERZ36203E_TEXT='"ERZ036203E: Unable to create logical volume '%1\$s' for server '%2\$s'"'
ERZ36204I=66
ERZ36204I_TEXT='"ERZ036204I: Created logical volume '%1\$s' for server '%2\$s'"'
ERZ36205E=67
ERZ36205E_TEXT='"ERZ036205E: Unable to initialize logical volume '%1\$s' for logging by server '%2\$s'"'
ERZ36206I=68
ERZ36206I_TEXT='"ERZ036206I: Initialized logical volume '%1\$s' initialized for logging by server '%2\$s'"'
ERZ36207E=69
ERZ36207E_TEXT='"ERZ036207E: Unable to create log file '%1\$s' for server '%2\$s'"'
ERZ36208I=70
ERZ36208I_TEXT='"ERZ036208I: Created log file '%1\$s' for server '%2\$s'"'
ERZ36209E=71
ERZ36209E_TEXT='"ERZ036209E: Unable to access logical volume '%1\$s' for server '%2\$s'"'
ERZ36210I=72
ERZ36210I_TEXT='"ERZ036210I: Server '%1\$s' has been initialized"'
ERZ36211E=73
ERZ36211E_TEXT='"ERZ036211E: Unable to query media archive setting for server '%1\$s'"'
ERZ36212E=74
ERZ36212E_TEXT='"ERZ036212E: Unable to enable media archiving for server '%1\$s'"'
ERZ36213I=75
ERZ36213I_TEXT='"ERZ036213I: Enabled media archiving for server '%1\$s'"'
ERZ36214W=76
ERZ36214W_TEXT='"ERZ036214W: Need to backup server '%1\$s' for media archiving to be effective"'
ERZ36215E=77
ERZ36215E_TEXT='"ERZ036215E: Unable to disable media archiving for server '%1\$s'"'
ERZ36216I=78
ERZ36216I_TEXT='"ERZ036216I: Disabled media archiving for server '%1\$s'"'

#
# Module what string
#
#@(#)admlg,22:07:02,Apr 08 1998,s211-L980223
#

# General values
typeset SERVERS_DIR=$($BINDIR/cicsname serversdir)


#------------------------------------------------------------------------
#
# SYNOPSIS: GeneralDispMsg MSN MSG_NO [parameters]
# DESCIPTION:
# Logs message
#
GeneralDispMsg()
{
    typeset -r _MCATNAME="cics.cat" # message catalogue
    typeset _MSN                    # MSN is the first param
    typeset _MSNO                   # Message Number of the message
    typeset _DEFMSG                 # Default message text corresponding to
                                    # msg id

    _MSN=$1; shift                              # MSN
    _MSNO="`eval echo \\$${1}`"                 # MESSAGE ID
    _DEFMSG="`eval echo \\$${1}_TEXT`" ; shift  # DEFAULT MESSAGE TEXT

    ( dspmsg ${_MCATNAME} -s ${MSET} ${_MSNO} "${_DEFMSG}" "$@" ; echo "" ) |
            sed "s!ERZ[0-9]*[EWI]!&/$_MSN!" >&2

}


#------------------------------------------------------------------------
# SYNOPSIS: GeneralLock [LSD|SSD|GSD] PID ServerDir
GeneralLock()
{
    typeset MSET=36                     
	typeset NORMAL_SERVER=$(NormaliseServer $3)

    typeset MYFILE=$SERVERS_DIR/$1/$NORMAL_SERVER/lock.$2
    typeset LOCKFILE=$SERVERS_DIR/$1/$NORMAL_SERVER/lock

    # create /var/cics_servers/LSD/<server_dir>/lock.$$
    touch $MYFILE 2>/dev/null

    [ $? -ne 0 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_1 ERZ3653E $MYFILE && return 1

    chmod 400 $MYFILE 2>/dev/null

    until CicsSafeLn $MYFILE $LOCKFILE </dev/null 2>/dev/null
    do
        GeneralDispMsg $ADMLG_FUNCS_MSN_2 ERZ3609W
        sleep 20
    done

    #
    # Locked. Remove our temp file
    #
    rm -f $MYFILE 2>/dev/null
    [ $? -ne 0 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_37 ERZ3654E $MYFILE && return 1

    return 0
}


#------------------------------------------------------------------------
#
# SYNOPSIS: GeneralUnlock [LSD|SSD|GSD] PID ServerDir
# DESCRIPTION:
# Unlock lock file
#
GeneralUnlock ()
{
    typeset MSET=36                     
	typeset NORMAL_SERVER=$(NormaliseServer $3)

    typeset LOCKFILE=$SERVERS_DIR/$1/$NORMAL_SERVER/lock
	typeset -l TYPE="$1"

	OUTPUT=$($BINDIR/cicsget -c $TYPE -s $3 2>&1)
	[ $? -ne 0 ] && echo "$OUTPUT" && return 1

    if [ -f $LOCKFILE ]
    then
        rm -f $LOCKFILE 2>/dev/null
        [ $? -ne 0 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_4 ERZ3654E $LOCKFILE \
                 && return 1

    fi
    # Do not log a message if there is no lock file,
    # as this function is idempotent - run it lots of times
    # and you get the same result.

    return 0
}


#------------------------------------------------------------------------
#
# SYNOPSIS: GeneralTryLock [LSD|SSD|GSD] PID ServerDir
# DESCRIPTION:
# Try and take a lock

GeneralTryLock ()
{
    typeset MSET=36                     
	typeset NORMAL_SERVER=$(NormaliseServer $3)

    typeset MYFILE=$SERVERS_DIR/$1/$NORMAL_SERVER/lock.$2
    typeset LOCKFILE=$SERVERS_DIR/$1/$NORMAL_SERVER/lock

    #
    # Create a temp file and try to link it to the lock
    #
    touch $MYFILE 2>/dev/null
    [ $? -ne 0 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_6 ERZ3653E $MYFILE && return 1

    chmod 400 $MYFILE 2>/dev/null

    CicsSafeLn $MYFILE $LOCKFILE </dev/null 2>/dev/null

    #
    # If link failed then lock already existed
    #
    if [ $? -ne 0 ]
    then
        GeneralDispMsg $ADMLG_FUNCS_MSN_7 ERZ3608E $LOCKFILE

        rm -f $MYFILE 2>/dev/null
        [ $? -ne 0 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_39 ERZ3654E $MYFILE
        return 1
    fi

    rm -f $MYFILE 2>/dev/null
    [ $? -ne 0 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_38 ERZ3654E $MYFILE && return 1

    return 0
}


#------------------------------------------------------------------------
#
# SYNOPSIS: GeneralLockOptions [LSD|SSD|GSD] PID [-l|-t|-u] ServerDir
# DESCRIPTION:
# This script parses the -l|-t|-u options and calls the appropriate lock
# function.
#
GeneralLockOptions ()
{
    case $3 in
    -l)
        GeneralLock $1 $2 $4
        return $?
        ;;
    -u)
        GeneralUnlock $1 $2 $4
        return $?
        ;;
    -t)
        GeneralTryLock $1 $2 $4
        return $?
        ;;
    esac
}

#------------------------------------------------------------------------

# Return /var/cics_servers/[LSD|PSD|GSD]/<server>

WhatServerDir()
{
    typeset NORMAL_SERVER=$(NormaliseServer $2)

    echo $SERVERS_DIR/$1/$NORMAL_SERVER
}


#------------------------------------------------------------------------

# Usage: WhatArchiveDir $SERVER
# Return /var/cics_servers/archives/<server-name>

WhatArchiveDir()
{
    typeset SERVER_NAME=$(basename $1)
    echo $SERVERS_DIR/archives/$SERVER_NAME
}

#------------------------------------------------------------------------

# Usage: GeneralChangeMode $MODE $PATH

GeneralChangeMode()
{
    typeset MSET=36                     # switch to AdmLG messages
    typeset MODE=$1
    typeset DIR=$2
    chmod -R $MODE $DIR
    [ $? -ne 0 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_10 ERZ3684E $1 && return 1
    return 0
}

#------------------------------------------------------------------------

# Usage: GeneralChangeOwner $USER $PATH

GeneralChangeOwner()
{
    typeset MSET=36                     # switch to AdmLG messages
    typeset USER=$1
    typeset DIR=$2
    chown $USER $DIR
    [ $? -ne 0 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_49 ERZ36156E $DIR $USER && \
       return 1
    return 0
}

#------------------------------------------------------------------------

# Usage: GeneralMkDir $PATH $VERBOSE [$MODE]

GeneralMkDir()
{
    typeset MSET=36                     # switch to AdmLG messages
    [ $# -lt 2 ] && echo "ASSERT in GeneralMkDir - not enough params" && exit 1

    typeset DIR=$1
    typeset VERBOSE=$2

    typeset MODE=770
    [ $# -ge 3 ] && MODE=$3

    typeset -i TUMASK="8#$MODE^8#777"

    UMASK=${TUMASK##*\#}

    [ $UMASK -lt 10 ] && UMASK="0"$UMASK
    [ $UMASK -lt 100 ] && UMASK="0"$UMASK

    typeset SAVUMASK=$(umask)

    umask $UMASK

    if [ ! -d $DIR ]
    then
        mkdir -p $DIR
        [ $? -ne 0 ] && \
            GeneralDispMsg $ADMLG_FUNCS_MSN_9 ERZ3683E $DIR && return 1
        [ $VERBOSE -eq 1 ] && \
            GeneralDispMsg $ADMLG_FUNCS_MSN_11 ERZ36126I $DIR
        GeneralChangeMode $MODE $DIR || return 1
    fi

    umask $SAVUMASK

    return 0
}

#------------------------------------------------------------------------
#
# Return server without /.:/ or /.../
#
NormaliseServer()
{
    typeset RESULT

    RESULT=${1#/.:/}
    RESULT=${RESULT#/.../}

    echo $RESULT
}



#------------------------------------------------------------------------
#
# Usage message and exit.
# Note: this is an internal function so the usage is not NLS'd
#
DCEUsage()
{
    echo "Usage: GeneralDCEAdmin LSD|SSD|GSD -s server -S shortname -u user-id {-A |-D} [-I] [-v]" 2>&1
    exit 1

}


#------------------------------------------------------------------------
#
# DCEErrorExit
# Logs message and exits 1
#

DCEErrorExit()
{
    GeneralDispMsg "$@"
    exit 1
}


#------------------------------------------------------------------------
#
# SYNOPSIS: GeneralDCEAdmin LSD|SSD|GSD -s server -S shortname {-A |-D}
#                           [-I] [-q]
# Local function to actually do the dce admin
#
#
GeneralDCEAdmin()
{
    typeset MSET=36         # switch to AdmLG messages
    typeset SERVER          # Server name (-s)
    typeset SHORTNAME       # Short name  (-S)
    typeset USERID          # Short name  (-u)
    typeset ASET=0          # -A given
    typeset DSET=0          # -D given
    typeset ISET=0          # -I given
    typeset VSET=0          # -q given
    typeset TYPE            # SSD, GSD or LSD
    typeset PRINCIPAL       # Principal name
    typeset CDSDIR          # /.:/cics/log, /.:/cics/sfs or /.:/cics/ppc/gateway
    typeset GROUP           # either cics_log, cics_ppcgwy or cics_sfs
    typeset KEYTAB          # /var/cics_servers/LSD|SSD|GSD/<path>/key

	typeset DCE_ORG=${CICS_DCE_ORGANISATION:-"none"}


    TYPE=$1             # must be LSD GSD or SSD
    shift

    while getopts :s:S:u:ADh:Iv Opt
    do
        case $Opt in
        s) SERVER="$OPTARG"
           ;;
        S) SHORTNAME="$OPTARG"
           ;;
        u) USERID="$OPTARG"
           ;;
        A) ASET=1
           ;;
        D) DSET=1
           ;;
        I) ISET=1
           ;;
        v) VSET=1
           ;;
        *) DCEUsage
           ;;
        esac
    done

    #
    # Check mandatory options
    #
    [ -z "$SERVER" -o -z "$SHORTNAME" \
      -o \( $ASET -eq 0 -a $DSET -eq 0 \)  \
      -o \( $ASET -eq 1 -a -z "$USERID" \) \
      -o \( $ASET -eq 1 -a $DSET -eq 1 \) ] && DCEUsage

    #
    # Set up general vars
    #
    if [ "$TYPE" = LSD ]
    then
        CDSDIR=/.:/cics/log
        GROUP=cics_log
    elif [ "$TYPE" = GSD ]
    then
        CDSDIR=/.:/cics/ppc/gateway
        GROUP=cics_ppcgwy
    else
        CDSDIR=/.:/cics/sfs
        GROUP=cics_sfs
    fi

    KEYTAB=$(WhatServerDir $TYPE $SERVER)/key

    #
    # Form principal name by removing /.:/, /.../
    #

    PRINCIPAL=$(NormaliseServer $SERVER)

    #
    # Check if cds dirs are there
    #
    OUTPUT=$(cdscp show dir $CDSDIR 2>&1)

    [ $? -ne 0 ] && echo "$OUTPUT">&2 &&
                    DCEErrorExit $ADMLG_FUNCS_MSN_40 ERZ3616E $SERVER

    #
    # Do all the adds ...
    #
    if [ "$ASET" -eq 1 ]
    then
        #
        # Add principal ...
        # ... check if it exists
        #
        OUTPUT=$(RgyEdit -v -p $PRINCIPAL 2>&1)

        if [ $? -eq 0 ]
        then
            # Already exists, either quit or warn
            if [ $ISET -eq 1 ]
            then
                 GeneralDispMsg $ADMLG_FUNCS_MSN_41 ERZ3663W $SERVER
            else
                 echo "$OUTPUT" >&2
                 DCEErrorExit $ADMLG_FUNCS_MSN_42 ERZ3664E $SERVER
            fi
        else
            #
            # .. Create it
            #
            OUTPUT=$(RgyAddPrincipal -p $PRINCIPAL)

            # failed to add it
            [ $? -ne 0 ] && echo "$OUTPUT" >&2 &&
                              DCEErrorExit $ADMLG_FUNCS_MSN_12 ERZ3668E $SERVER

            # Information message
            [ $VSET -eq 1 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_13 ERZ3610I $SERVER
        fi

        #
        # Add account
        # ... check if its there
        OUTPUT=$(RgyEdit -v -a $PRINCIPAL 2>&1)

        if [ $? -eq 0 ]         # 'view' succeeded.
        then
            # Already exists, either quit or warn
            if [ $ISET -eq 1 ]
            then
                GeneralDispMsg $ADMLG_FUNCS_MSN_14 ERZ3661W $SERVER

                # Change passwd
                OUTPUT=$(RgyEdit -a -update << ENDFILE
                change -p $PRINCIPAL -pw $PRINCIPAL -o $DCE_ORG -g $GROUP -mp $ADMLG_DCE_PASSWORD
                quit
ENDFILE
                )
                # Failed to change it
                [ $? -ne 0 ] && echo "$OUTPUT" >&2  &&
                                  DCEErrorExit $ADMLG_FUNCS_MSN_15 ERZ3656E $SERVER

            else
                echo "$OUTPUT" >&1
                DCEErrorExit $ADMLG_FUNCS_MSN_16 ERZ3662E $SERVER
            fi
        else
            #
            # .. add the account
            #
            OUTPUT=$(RgyEdit -a -update <<ENDFILE
            add $PRINCIPAL -g $GROUP -pw $PRINCIPAL -o $DCE_ORG -mp $ADMLG_DCE_PASSWORD -mcl 8 -mcr 40
            quit
ENDFILE
            )

            # Failed to add
            [ $? -ne 0 ] && echo "$OUTPUT" >&2 &&
                              DCEErrorExit $ADMLG_FUNCS_MSN_17 ERZ3668E $SERVER

            # Info message
            [ $VSET -eq 1 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_18 ERZ3611I $SERVER
        fi

        #
        # Add keytab
        # ... check if it is there (using ls !)
        #
        if ls $KEYTAB >/dev/null 2>&1
        then
            # Exists
            if [ $ISET -eq 1 ]          # Ignore errors ?
            then
                GeneralDispMsg $ADMLG_FUNCS_MSN_19 ERZ3665W $KEYTAB $SERVER

                # Remove file
                rm -f $KEYTAB 2>/dev/null ||
                        DCEErrorExit $ADMLG_FUNCS_MSN_20 ERZ3669E $KEYTAB $SERVER
            else
                DCEErrorExit $ADMLG_FUNCS_MSN_21 ERZ3666E $KEYTAB $SERVER
            fi
        fi

        #
        # ... now really add it
        #
        OUTPUT=$(RgyEdit -a -update << ENDFILE
        ktadd -p $PRINCIPAL -pw $PRINCIPAL -f $KEYTAB
ENDFILE
        )

        [ $? -ne 0 ] && echo "$OUTPUT" &&
                        DCEErrorExit $ADMLG_FUNCS_MSN_22 ERZ3669E $KEYTAB $SERVER

        #
        # ... now randomise the password (why ?)
        #
        OUTPUT=$(RgyEdit -a -update << ENDFILE
        ktadd -p $PRINCIPAL -r -a -f $KEYTAB
ENDFILE
        )

        # Had an error ?
        [ $? -ne 0 ] && echo "$OUTPUT" &&
                          DCEErrorExit $ADMLG_FUNCS_MSN_23 ERZ3669E $KEYTAB $SERVER

        # Now chown it and chmod it (chmod not very important)
        OUTPUT=$(chmod 600 $KEYTAB 2>&1)
        OUTPUT=$(chown "$USERID":cics $KEYTAB 2>&1)

        [ $? -ne 0 ] && echo "$OUTPUT" &&
                          DCEErrorExit $ADMLG_FUNCS_MSN_48 ERZ36150E $KEYTAB $USERID


        [ $VSET -eq 1 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_24 ERZ3612I $SERVER

    fi

    #
    # Are we deleteing ?
    #

    if [ $DSET -eq 1 ]
    then
        #
        # Delete the principal
        #
        # ... Check if principal exists
        #
        OUTPUT=$(RgyEdit -v -p $PRINCIPAL 2>&1)

        if [ $? -ne 0 ]
        then
            GeneralDispMsg $ADMLG_FUNCS_MSN_25 ERZ3657W $SERVER
        else
            OUTPUT=$(RgyEdit -p -update <<ENDFILE
            delete $PRINCIPAL
            quit
ENDFILE
            )
            [ $? -ne 0 ] && echo "$OUTPUT" &&
                            DCEErrorExit $ADMLG_FUNCS_MSN_26 ERZ3671E $SERVER

            [ $VSET -eq 1 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_27 ERZ3613I $SERVER
        fi


        #
        # Delete the account
        #
        # Check if account exists (note that the account will not exist
        # if the principal was deleted, so no need to generate a message.)
        #
        OUTPUT=$(RgyEdit -v -a $PRINCIPAL 2>&1)

        if [ $? -eq 0 ]
        then
            # Its there

            OUTPUT=$(RgyEdit -a -update <<ENDFILE
            delete -p $PRINCIPAL
            quit
ENDFILE
            )

            [ $? -ne 0 ] && echo "$OUTPUT" &&
                            DCEErrorExit $ADMLG_FUNCS_MSN_29 ERZ3670E $SERVER

            [ $VSET -eq 1 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_30 ERZ3614I $SERVER
        fi

        #
        # Delete the keytab file
        # .. check if its there
        #
        if ls $KEYTAB >/dev/null 2>&1
        then
            # Exists
            rm -f $KEYTAB 2>/dev/null || DCEErrorExit $ADMLG_FUNCS_MSN_31 ERZ3672E $KEYTAB $SERVER

            [ $VSET -eq 1 ] && GeneralDispMsg $ADMLG_FUNCS_MSN_32 ERZ3617I $SERVER
        else
            GeneralDispMsg $ADMLG_FUNCS_MSN_33 ERZ3659W $KEYTAB $SERVER
        fi
    fi

    return 0
}

GeneralCheckSRC()
{
    typeset MSET=36                     # switch to AdmLG messages
    # $1 - LSD|SSD|GSD
    # $2 - shortname
    # $3 - server name
    typeset SHORTNAME=$2
    typeset SERVERNAME=$3

    typeset TYPE=sfs
    [ $1 = LSD ] && TYPE=log
    [ $1 = GSD ] && TYPE=ppcgwy

    # Check for short name already used for a different server
    EXISTING_SERVER=$(GetServerName cics$TYPE.$SHORTNAME)
    if [ $? -ne 0 ]
    then
        echo "$EXISTING_SERVER"
        GeneralDispMsg $ADMLG_FUNCS_MSN_50 ERZ36157E cics$TYPE.$SHORTNAME
        exit 1
    fi

    if [ -z "$EXISTING_SERVER" -o "$EXISTING_SERVER" = "$SERVERNAME" ]
    then
        # Either entry not there, (not an error) or SRC entry was already
        # existed with the same server name for this shortname.
        return 0
    else
        # SRC entry was created by another server with the same ShortName
        GeneralDispMsg $ADMLG_FUNCS_MSN_43 ERZ3660W $SHORTNAME $EXISTING_SERVER
        return 1
    fi

}


GeneralCheckUser()
{
    typeset MSET=36                     # switch to AdmLG messages
    typeset TYPE=$1
    typeset ID=$2
	typeset DETAILS=""

    # Get home directory

    DETAILS=$(lsuser -a home $ID 2>&1)

    if [ $? -ne 0 ]
    then
        echo "$DETAILS"

        if [ $TYPE = W ]
        then
            GeneralDispMsg $ADMLG_FUNCS_MSN_44 ERZ36146W $ID
            return 0
        else
            GeneralDispMsg $ADMLG_FUNCS_MSN_45 ERZ36147E $ID
            return 1
        fi
    fi

    # Check permissions of directory

    DETAILS=$(echo "$DETAILS" | sed 's/.*=\(.*\)/\1/')

    OUTPUT=$(ls -dl $DETAILS 2>&1)

    RET=$?

    if [ \( $RET -eq 0 -a rwx != $(echo "$OUTPUT" | cut -c2-4) \) \
         -o $RET -ne 0 ]
    then
        echo "$OUTPUT"

        if [ $TYPE = W ]
        then
            GeneralDispMsg $ADMLG_FUNCS_MSN_46 ERZ36148W $DETAILS $ID
            return 0
        else
            GeneralDispMsg $ADMLG_FUNCS_MSN_47 ERZ36149E $DETAILS $ID
            return 1
        fi
    fi

    return 0
}

GeneralCheckDir()
{
    typeset DIR=$1
    typeset USER=$2
    typeset REQUIRED=$3
    typeset LENGTH=${#REQUIRED}         # Length (ie. rw = 2)
    typeset CUT1="-c2-$(expr 1 + $LENGTH)"
    typeset CUT2="-c5-$(expr 4 + $LENGTH)"
    typeset CUT3="-c8-$(expr 7 + $LENGTH)"
    typeset UID
    typeset OWNER
    typeset GROUPS

    typeset PERMS=$(ls -dl $DIR 2>&1)

    if [ $? -ne 0 ]
    then
        return 1
    fi

    #
    # Assumes root has all permissions. Check the uid
    #
    UID=$(lsuser -a id $USER 2>&1)

    [ $? -ne 0 ] && echo "$UID" && return 1     # Had an error

    [ 0 = $(echo $UID | cut -d'=' -f2) ] && return 0

    OWNER=$(echo $PERMS|cut -d' ' -f3)
    GROUP=$(echo $PERMS|cut -d' ' -f4)

    # Check permissions if owner ...
    [ "$OWNER" = "$USER" -a $REQUIRED = $(echo $PERMS | cut $CUT1) ] &&
                                        return 0

    # If user is in the group ...
    GROUPS=$(lsuser -a groups $USER 2>&1)

    [ $? -ne 0 ] && echo "$GROUPS" && return 1  # Had an error

    OUTPUT=$(echo "$GROUPS" | cut -d':' -f2 | grep -w $GROUP 2>&1)

    [ $? -eq 0 -a $REQUIRED = $(echo $PERMS | cut $CUT2) ] && return 0

    # Check 'all' permissions
    [ $REQUIRED = $(echo $PERMS | cut $CUT3) ] && return 0

    return 1
}

GeneralProtStringToNumber()
{
  case $1 in
    default)     echo 0 ;;
    none)        echo 1 ;;
    connect)     echo 2 ;;
    call)        echo 3 ;;
    pkt)         echo 4 ;;
    pkt_integ)   echo 5 ;;
    pkt_privacy) echo 6 ;;
    *)           echo -1 ;;
  esac
}

# ===========================================================================
# FUNCTION: CheckStartServer
#
# Parameter is 1: Server Name
#              2: Server Type
#              3: Protection Level
#              4: Old Protection Level
#              5: Verbose flag 0=not verbose 1=verbose
#              6: Ignore flag 0=error 1=ignore errors
#              7: Name Service
# Returns       : 0 - success
#               : 1 - fail
# ============================================================================

CheckStartServer()
{
    #
    # Declare local variables
    #
    typeset MSET=36          # switch to AdmLG messages
    typeset SERVER=$1        # server name
    typeset SERVERTYPE=$2    # server type 'SSD' or 'GSD'
    typeset PROTLEVEL=$3     # protection level 'default' 'none' 'connect' etc
    typeset OLDPROTLEVEL=$4  # protection level 'default' 'none' 'connect' etc
    typeset -i VERBOSE=$5    # verbose flag
    typeset -i IGNORE=$6     # ignore flag
    typeset NAMESERVICE=$7   # name service
    typeset OLDPROTLEVEL     # protection level from stanza
    typeset RESTARTFILE1     # Primary Restart file
    typeset RESTARTFILE2     # backup Restart file
    typeset KEYFILE          # Keytab file
    typeset ARCHIVEDIR       # archive directory location
    typeset RUNNING=0        # Set status of server

    if [ "$SERVERTYPE" = "SSD" ]
    then
        # If protection level changed to/from none warn user that file ACLS
        # need changing manually.
        if [ ! -z "$OLDPROTLEVEL" -a \
            \( \( "$PROTLEVEL" = "none" -a "$OLDPROTLEVEL" != "none" \) -o  \
               \( "$PROTLEVEL" != "none" -a "$OLDPROTLEVEL" = "none" \) \) ]
        then
            # ERZ36190W Faile ACLs might need changing
            GeneralDispMsg $ADMLG_FUNCS_MSN_52 ERZ36190W "$OLDPROTLEVEL" \
                      "$PROTLEVEL"
        else
            if [ $VERBOSE -eq 1 ]
            then
                # checked protect level
                GeneralDispMsg $ADMLG_FUNCS_MSN_53 ERZ36197I $SERVER
            fi
        fi
    fi

    RESTARTFILE1="$(WhatServerDir $SERVERTYPE $SERVER)/restart"
    RESTARTFILE2="$RESTARTFILE1.bak"

    # check if the restart and restart backup files exist
    # if neither exist exit with error
    # if either exist ok
    if [[ ! -a $RESTARTFILE1 ]]
    then
        if [[ ! -a $RESTARTFILE2 ]]
        then
            # restart & restart.bak file missing
            GeneralDispMsg $ADMLG_FUNCS_MSN_54 ERZ36191E $RESTARTFILE1 \
                           $RESTARTFILE2 $SERVER
            StartErrorExit $SERVER $RUNNING
        else
           # restart file missing
           GeneralDispMsg $ADMLG_FUNCS_MSN_55 ERZ36192W $RESTARTFILE1 $SERVER
        fi
    else
        if [[ ! -a $RESTARTFILE2 ]]
        then
            # restart.bak file missing
            GeneralDispMsg $ADMLG_FUNCS_MSN_56 ERZ36192W $RESTARTFILE2 $SERVER
        else
            if [ $VERBOSE -eq 1 ]
            then
                # checked restart files
                GeneralDispMsg $ADMLG_FUNCS_MSN_57 ERZ36198I $SERVER
            fi
        fi
    fi

    KEYFILE="$(WhatServerDir $SERVERTYPE $SERVER)/key"

    # check if the keytab file exists
    if [ "$NAMESERVICE" = "DCE" ] || [ "$PROTLEVEL" != "none" ]
    then
        if [[ ! -a $KEYFILE ]]
        then
            # keytab file missing
            GeneralDispMsg $ADMLG_FUNCS_MSN_58 ERZ36194E $KEYFILE $SERVER
            StartErrorExit $SERVER $RUNNING
        else
            if [ $VERBOSE -eq 1 ]
            then
                # checked keytab files
                GeneralDispMsg $ADMLG_FUNCS_MSN_59 ERZ36199I $SERVER
            fi
        fi
    fi
    
    ARCHIVEDIR=$(WhatArchiveDir $SERVER)

    # check if archive directory exists
    # if not and ignore errors the make it
    # else exit with error
    if [[ ! -d $ARCHIVEDIR ]]
    then
        if [ $IGNORE -eq 1 ]
        then
            # no archive dir but making it
            GeneralDispMsg $ADMLG_FUNCS_MSN_60 ERZ36195W $ARCHIVEDIR $SERVER
            GeneralMkDir $ARCHIVEDIR $VERBOSE
            # no need to check return as if fails does not return
        else
            # no archive directory
            GeneralDispMsg $ADMLG_FUNCS_MSN_61 ERZ36196E $ARCHIVEDIR $SERVER
            StartErrorExit $SERVER $RUNNING
        fi
    else
        if [ $VERBOSE -eq 1 ]
        then
            # checked archive directory
            GeneralDispMsg $ADMLG_FUNCS_MSN_62 ERZ36200I $SERVER
        fi
    fi

    # return to caller with no errors
    return 0

}

# ========================================================================
# FUNCTION: CheckStartPreReqs
#
# Parameter is 1: Server Name
#              2: Server Type
#              3: Short Name
# Returns       : 0 - success
#               : 1 - fail
# ========================================================================

CheckStartPreReqs()
{
    # Declare local variables

    typeset MSET=36           # switch to AdmLG messages
    typeset SERVER=$1         # server name
    typeset SERVERTYPE=$2     # type of server 'SSD' or 'GSD'
    typeset SHORTNAME=$3      # short name
    typeset OUTPUT            # general variable for output
    typeset SRC_ENTRY         # src entry for startsrc
    typeset RUNNING=0         # Status of Encina Server

    # check we can run startsrc

    CheckCanRun startsrc

    if [ $? -ne 0 ]
    then
        # can not run startsrc
        GeneralDispMsg $ADMLG_FUNCS_MSN_65 ERZ3634E "startsrc"
        StartErrorExit $SERVER $RUNNING
    fi

    # Check the SRC for this server

    GeneralCheckSRC $SERVERTYPE "$IE_SHORTNAME" $SERVER

    if [ $? -ne 0 ]
    then
        # GeneralCheckSRC has its own messages so none needed here
        StartErrorExit $SERVER $RUNNING
    fi

    # return to caller with no errors
    return 0
}

#########################################################################
# StartErrorExit
# Usage: StartErrorExit <SERVER>
#                       <RUNNING>
#
# Synopsis: Exit cleanly from a 'Start Encina Server' routine
#########################################################################
StartErrorExit()
{
    #
    # Local Variables
    #
    typeset MSET=36           # switch to AdmLG messages
    typeset SERVER=$1
    typeset RUNNING=$2

    # If Server is running say so
    if [ $RUNNING -eq 1 ]
    then
        # ERZ36152E Server 'x' has started but not successfully configured.
        GeneralDispMsg $ADMLG_FUNCS_MSN_68 ERZ36152E $SERVER
    else
        # ERZ3607E Cannot start Server 'x'
        GeneralDispMsg $ADMLG_FUNCS_MSN_69 ERZ3607E $SERVER

        # Unlock appropiate server
        if [[ "$SERVER" = *ppc* ]]
        then
            $BINDIR/cicsppcgwylock -u $SERVER
        else
            $BINDIR/cicssfslock -u $SERVER
        fi
    fi
    exit 1
}

#########################################################################
# CreateLog
# Usage: CreateLog <SERVER>
#                  <LOGVOL>
#                  <CICS_RAW_LOG_PATH>
#                  <LOGFILE>
#                  <USERID>
#                  <VERBOSE>
#
# Synopsis: Initialize an Encina Server's log
#########################################################################
CreateLog()
{
    #
    # Local Variables
    #
    typeset MSET=36           # switch to AdmLG messages
    typeset SERVER=$1
    typeset LOGVOL=$2
    typeset CICS_RAW_LOG_PATH=$3
    typeset LOGFILE=$4
    typeset USERID=$5
    typeset VERBOSE=$6
    typeset RUNNING=1
    typeset EXTENTSIZE
    typeset OUTPUT

    # create archive directory
    MkArchiveDir $SERVER $USERID $VERBOSE

    # Map the log volume to the gateway volume
    OUTPUT=$(MapLVol $SERVER $CICS_RAW_LOG_PATH 2>&1)
    if [ $? -ne 0 ]
    then
        echo "$OUTPUT"
        # ERZ36203E cannot map AIX logical volume 'x' to Encina Server 'y'
        GeneralDispMsg $ADMLG_FUNCS_MSN_71 ERZ36203E $LOGVOL $SERVER
        StartErrorExit $SERVER $RUNNING
    else
        if [ $VERBOSE -eq 1 ]
        then
            # ERZ36204I Mapped AIX logical volume to server
            GeneralDispMsg $ADMLG_FUNCS_MSN_72 ERZ36204I $LOGVOL $SERVER
        fi
    fi

    # Initialize the log volume
    if [[ "$SERVER" = *ppc* ]]
    then
        # As the gateway has no long-lived data only needs small extentsize
        EXTENTSIZE="-extentsize 128k"
    else
        # let it default otherwise
        unset EXTENTSIZE
    fi
    OUTPUT=$(tkadmin init logvol -s $SERVER \
             $LOGVOL                        \
             FILE:$(WhatArchiveDir $SERVER) \
             $EXTENTSIZE 2>&1)
    if [ $? -ne 0 ]
    then
        echo "$OUTPUT"
        # ERZ36205E cannot initialize the log volume
        GeneralDispMsg $ADMLG_FUNCS_MSN_73 ERZ36205E $LOGVOL $SERVER
        StartErrorExit $SERVER $RUNNING
    else
        if [ $VERBOSE -eq 1 ]
        then
            # ERZ36206I Initialized log volume
            GeneralDispMsg $ADMLG_FUNCS_MSN_74 ERZ36206I $LOGVOL $SERVER
        fi
    fi

    OUTPUT=$(tkadmin create logfile -s $SERVER $LOGVOL/$LOGFILE 2>&1)
    if [ $? -ne 0 ]
    then
        echo "$OUTPUT"
        # ERZ36207E cannot create logfile
        GeneralDispMsg $ADMLG_FUNCS_MSN_75 ERZ36207E $LOGVOL/$LOGFILE $SERVER
        StartErrorExit $SERVER $RUNNING
    else
        if [ $VERBOSE -eq 1 ]
        then
            # ERZ36208I created logfile
            GeneralDispMsg $ADMLG_FUNCS_MSN_76 ERZ36208I $LOGVOL/$LOGFILE \
                           $SERVER
        fi
    fi

    return 0
}

#########################################################################
# MkArchiveDir
# Usage: MkArchiveDir <SERVER>
#                     <USERID>
#                     <VERBOSE>
#
# Synopsis: Create archive directory for <server>
#########################################################################
MkArchiveDir()
{
    #
    # Local Variables
    #
    typeset MSET=36           # switch to AdmLG messages
    typeset SERVER=$1
    typeset USERID=$2
    typeset VERBOSE=$3
    typeset ArchiveDirectory
    typeset RUNNING=1

    #
    # Set permissions of the $SERVERS_DIR/archives directory to 775 to allow
    # the creation of the server's individual archive directory. If the
    # $SERVERS_DIR/archives directory doesn't already exist, it will be
    # created.
    #
    if [ ! -d $SERVERS_DIR/archives ]
    then
	GeneralMkDir $SERVERS_DIR/archives $VERBOSE
	if [ $? -ne 0 ]
	then
	    StartErrorExit $SERVER $RUNNING
	fi
    fi
    GeneralChangeMode 775 $SERVERS_DIR/archives
    if [ $? -ne 0 ]
    then
        StartErrorExit $SERVER $RUNNING
    fi

    ArchiveDirectory=$(WhatArchiveDir $SERVER)

    GeneralMkDir $ArchiveDirectory $VERBOSE
    if [ $? -ne 0 ]
    then
        StartErrorExit $SERVER $RUNNING
    fi
    GeneralChangeOwner $USERID:cics $ArchiveDirectory
    if [ $? -ne 0 ]
    then
        StartErrorExit $SERVER $RUNNING
    fi

    return 0
}

#########################################################################
# CheckLog
# Usage: CheckLog <SERVER>
#                 <LOGVOL>
#                 <VERBOSE>
#
# Synopsis: Make sure that the Encina server has been initialized
#########################################################################
CheckLog()
{
    #
    # Local Variables
    #
    typeset MSET=36           # switch to AdmLG messages
    typeset SERVER=$1
    typeset LOGVOL=$2
    typeset VERBOSE=$3
    typeset RUNNING=1
    typeset OUTPUT

    # call tkadmin commands to query the log volume
    OUTPUT=$(tkadmin query logvol -s $SERVER $LOGVOL 2>&1)
    if [ $? -ne 0 ]
    then
        echo "$OUTPUT"
        # ERZ36209E Encina Server has not been initialized
        GeneralDispMsg $ADMLG_FUNCS_MSN_77 ERZ36209E $LOGVOL $SERVER
        StartErrorExit $SERVER $RUNNING
    else
        if [ $VERBOSE -eq 1 ]
        then
            # ERZ36210I Encina Server has been initialized
            GeneralDispMsg $ADMLG_FUNCS_MSN_78 ERZ36210I $SERVER
        fi
    fi

    return 0
}

#########################################################################
# SetMediaArchive
# Usage: SetMediaArchive <SERVER>
#                        <ENABLEMRA>
#                        <STARTTYPE>
#                        <VERBOSE>
#
# Synopsis: Enable/Disable Media Recovery Archiving
#########################################################################
SetMediaArchive()
{
    #
    # Local Variables
    #
    typeset MSET=36           # switch to AdmLG messages
    typeset SERVER=$1
    typeset ENABLEMRA=$2
    typeset STARTTYPE=$3
    typeset VERBOSE=$4
    typeset RUNNING=1
    typeset OUTPUT
    typeset MediaArchiveStatus=0

    OUTPUT=$(tkadmin query mediaarchiving -s $SERVER 2>&1)
    if [ $? -ne 0 ]
    then
        echo "$OUTPUT"
        # ERZ36211E Cannot access Encina Server
        GeneralDispMsg $ADMLG_FUNCS_MSN_79 ERZ36211E $SERVER
        StartErrorExit $SERVER $RUNNING
    else
        if [[ "$OUTPUT" = *enabled* ]]
        then
            MediaArchiveStatus=1
        fi
    fi

    # IF MRAArchivingEnabled=yes AND MediaArchiveStatus=disabled
    if [ "$ENABLEMRA" = "yes" -a $MediaArchiveStatus -ne 1 ]
    then
        OUTPUT=$(tkadmin enable mediaarchiving -s $SERVER 2>&1)
        if [ $? -ne 0 ]
        then
            echo "$OUTPUT"
            # ERZ36212E Cannot enable Encina Server
            GeneralDispMsg $ADMLG_FUNCS_MSN_80 ERZ36212E $SERVER
            StartErrorExit $SERVER $RUNNING
        else
            if [ $VERBOSE -eq 1 ]
            then
                # ERZ36213I Enabled MRA Archiving
                GeneralDispMsg $ADMLG_FUNCS_MSN_81 ERZ36213I $SERVER
            fi
            # IF Auto start need to use tkadmin backup for it to be useful
            if [ $STARTTYPE = "auto" ]
            then
                # ERZ36214W Need to backup Encina Server
                GeneralDispMsg $ADMLG_FUNCS_MSN_82 ERZ36214W $SERVER
            fi
        fi
    else
        # IF MRAArchivingEnabled=no AND MediaArchiveStatus=enabled
        if [ "$ENABLEMRA" != "yes" -a $MediaArchiveStatus -eq 1 ]
        then
            OUTPUT=$(tkadmin disable mediaarchiving -s $SERVER 2>&1)
            if [ $? -ne 0 ]
            then
                echo "$OUTPUT"
                # ERZ36215E Cannot disable Encina Server
                GeneralDispMsg $ADMLG_FUNCS_MSN_83 ERZ36215E $SERVER
                StartErrorExit $SERVER $RUNNING
            else
                if [ $VERBOSE -eq 1 ]
                then
                    # ERZ36216I Disabled MRA Archiving
                    GeneralDispMsg $ADMLG_FUNCS_MSN_70 ERZ36216I $SERVER
                fi
            fi
        fi
    fi

    return 0
}

