#!/bin/ksh
#
# NAME:		cicssrc.fnc - HP-UX SRC emulation scripts
#
# VERSION:	1.1
#
# COPYRIGHT:                                                        
# 
#   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
#   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
#   All Rights Reserved
#   Licensed Materials - Property of IBM
#  
#   US Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
#                                                                   
#
# SYNOPSIS:	
#
#	IsChild pid ppid
#	FieldCount rec ifs
#	UserToUID user_name
#	UserToGID user
#	VerifyInactive ssys_name [db_entry]
#	CICS_SRCDBInitTranSvc
#	CICS_SRCDBStartTran
#	CICS_SRCDBCommitTran tran_id
#	CICS_SRCDBAbortTran tran_id
#	CICS_SRCDBCreateDir [db_dir]
#	CICS_SRCDBCreate [db_file]
#	CICS_SRCDBLock
#	CICS_SRCDBUnlock
#	CICS_SRCDBHandleSignal
#	CICS_SRCDBSelect [-v] [-F] [-f db_file] field_index pattern
#	CICS_SRCDBProject fieldnum [...]
#	CICS_SRCDBGet [-q] ssys_name
#	CICS_SRCDBGetGroup [-q] group_name
#	CICS_SRCDBPut ssys_entry
#	CICS_SRCDBDelete [-l] ssys_name
#	CICS_SRCDBUpdate ssys_name ssys_entry
#	CICS_SRCDBGroupToSubsysList group_name
#	CICS_SRCDBAllSubsysList
#	CICS_SRCDBRefreshStatus [ -l ] ssys_name
#	CICS_SRCDBValidatePID pid program user_name [uid]
#
# DESCRIPTION:	
# Refer to function headers.
#
# CAVEATS/WARNINGS:
# Notify not supported. Also, only a limited subset of SRC
# capability has been implemented.
#
# RETURNED VALUES:
#	Unless otherwise noted in individual function header:
#	0	Success/true
#     > 0	Failure/false
#
# RELATED INFORMATION:
# None.
#
#
#*/


#
# NLS error messaging.
#

MSET=58

ERZ5830E=26
ERZ5830E_TEXT='"ERZ058030E: Subsystem '%1\$s' is not defined"'
ERZ5854E=49
ERZ5854E_TEXT='"ERZ058054E: Cannot remove cicssrc database lock file '%1\$s'"'
ERZ5855E=50
ERZ5855E_TEXT='"ERZ058055E: Unable to lock '%1\$s'.  File does not exist."'
ERZ5856E=51
ERZ5856E_TEXT='"ERZ058056E: User lacks write access to '%1\$s'.  Lock failed."'
ERZ5857E=52
ERZ5857E_TEXT='"ERZ058057E: Unable to lock the cicssrc database.  If appropriate, remove '%1\$s' to unlock."'
ERZ5858E=53
ERZ5858E_TEXT='"ERZ058058E: Unable to backout incomplete updates made by a previous command to the cicssrc database, '%1\$s'"'
ERZ5859E=54
ERZ5859E_TEXT='"ERZ058059E: Cannot remove intermediate syncpoint file '%1\$s'"'
ERZ5860E=55
ERZ5860E_TEXT='"ERZ058060E: Cannot create intermediate syncpoint file '%1\$s'"'
ERZ5861E=56
ERZ5861E_TEXT='"ERZ058061E: Cannot syncpoint cicssrc database, '%1\$s', to intermediate syncpoint file, '%2\$s'"'
ERZ5862E=57
ERZ5862E_TEXT='"ERZ058062E: Cannot rename cicssrc database intermediate syncpoint, '%1\$s', to install to actual syncpoint '%2\$s'"'
ERZ5863E=58
ERZ5863E_TEXT='"ERZ058063E: Unable to commit updates to '%1\$s'.  Removal of syncpoint '%2\$s' failed."'
ERZ5864E=59
ERZ5864E_TEXT='"ERZ058064E: Internal shell script or function invoked with incorrect syntax"'
ERZ5865E=60
ERZ5865E_TEXT='"ERZ058065E: Cannot backout cicssrc database updates.  Syncpoint file '%1\$s' does not exist."'
ERZ5866E=61
ERZ5866E_TEXT='"ERZ058066E: Cannot backout cicssrc database updates.  Overlay of database, '%1\$s', with the previous syncpoint, '%2\$s', failed."'
ERZ5868E=62
ERZ5868E_TEXT='"ERZ058068E: Cannot create cicssrc database directory, '%1\$s'"'
ERZ5869E=63
ERZ5869E_TEXT='"ERZ058069E: Cannot set group ownership on directory '%1\$s' to group '%2\$s'"'
ERZ5870E=64
ERZ5870E_TEXT='"ERZ058070E: Cannot create file '%1\$s'"'
ERZ5871E=65
ERZ5871E_TEXT='"ERZ058071E: Cannot set access permissions ('%1\$s') on '%2\$s'"'
ERZ5872E=66
ERZ5872E_TEXT='"ERZ058072E: Cannot set group ownership of file '%1\$s' to group '%2\$s'"'
ERZ5876E=67
ERZ5876E_TEXT='"ERZ058076E: cicssrc database image '%1\$s' does not exist.  Select failed."'
ERZ5877E=68
ERZ5877E_TEXT='"ERZ058077E: Cannot read cicssrc database image '%1\$s' - permission denied.  Select failed."'
ERZ5878E=69
ERZ5878E_TEXT='"ERZ058078E: Error reading cicssrc database image '%1\$s'.  Select failed."'
ERZ5885E=70
ERZ5885E_TEXT='"ERZ058085E: Cannot rename cicssrc database '%1\$s'.  Entry deletion failed."'
ERZ5886E=71
ERZ5886E_TEXT='"ERZ058086E: Cannot create a new cicssrc database file, '%1\$s'.  Restoring original from backup copy.  Entry deletion failed."'
ERZ5887E=72
ERZ5887E_TEXT='"ERZ058087E: Cannot restore original '%1\$s' from backup copy in '%2\$s'"'
ERZ5888E=73
ERZ5888E_TEXT='"ERZ058088E: Cannot rewrite cicssrc database '%1\$s'.  Restoring original from backup copy.  Entry deletion failed."'
ERZ5891I=76
ERZ5891I_TEXT='"ERZ058091I: Retrying the cicssrc database lock for up to '%1\$s' seconds"'
ERZ5892I=77
ERZ5892I_TEXT='"ERZ058092I: The cicssrc database was successfully locked with retries"'

#
# Module what string
#
#@(#)supos,21:43:57,Apr 08 1998,s211-L980223
#

FUNC_MSN_START=08

typeset -r SUPOS_FUNCS_MSN_1=${FUNC_MSN_START}01
typeset -r SUPOS_FUNCS_MSN_2=${FUNC_MSN_START}02
typeset -r SUPOS_FUNCS_MSN_3=${FUNC_MSN_START}03
typeset -r SUPOS_FUNCS_MSN_4=${FUNC_MSN_START}04
typeset -r SUPOS_FUNCS_MSN_5=${FUNC_MSN_START}05
typeset -r SUPOS_FUNCS_MSN_6=${FUNC_MSN_START}06
typeset -r SUPOS_FUNCS_MSN_7=${FUNC_MSN_START}07
typeset -r SUPOS_FUNCS_MSN_8=${FUNC_MSN_START}08
typeset -r SUPOS_FUNCS_MSN_9=${FUNC_MSN_START}09
typeset -r SUPOS_FUNCS_MSN_10=${FUNC_MSN_START}10
typeset -r SUPOS_FUNCS_MSN_11=${FUNC_MSN_START}11
typeset -r SUPOS_FUNCS_MSN_12=${FUNC_MSN_START}12
typeset -r SUPOS_FUNCS_MSN_13=${FUNC_MSN_START}13
typeset -r SUPOS_FUNCS_MSN_14=${FUNC_MSN_START}14
typeset -r SUPOS_FUNCS_MSN_15=${FUNC_MSN_START}15
typeset -r SUPOS_FUNCS_MSN_16=${FUNC_MSN_START}16
typeset -r SUPOS_FUNCS_MSN_17=${FUNC_MSN_START}17
typeset -r SUPOS_FUNCS_MSN_18=${FUNC_MSN_START}18
typeset -r SUPOS_FUNCS_MSN_19=${FUNC_MSN_START}19
typeset -r SUPOS_FUNCS_MSN_20=${FUNC_MSN_START}20
typeset -r SUPOS_FUNCS_MSN_21=${FUNC_MSN_START}21
typeset -r SUPOS_FUNCS_MSN_22=${FUNC_MSN_START}22
typeset -r SUPOS_FUNCS_MSN_23=${FUNC_MSN_START}23
typeset -r SUPOS_FUNCS_MSN_24=${FUNC_MSN_START}24
typeset -r SUPOS_FUNCS_MSN_25=${FUNC_MSN_START}25
typeset -r SUPOS_FUNCS_MSN_26=${FUNC_MSN_START}26
typeset -r SUPOS_FUNCS_MSN_27=${FUNC_MSN_START}27
typeset -r SUPOS_FUNCS_MSN_28=${FUNC_MSN_START}28
typeset -r SUPOS_FUNCS_MSN_29=${FUNC_MSN_START}29
typeset -r SUPOS_FUNCS_MSN_30=${FUNC_MSN_START}30
typeset -r SUPOS_FUNCS_MSN_31=${FUNC_MSN_START}31
typeset -r SUPOS_FUNCS_MSN_32=${FUNC_MSN_START}32
typeset -r SUPOS_FUNCS_MSN_33=${FUNC_MSN_START}33
typeset -r SUPOS_FUNCS_MSN_34=${FUNC_MSN_START}34
typeset -r SUPOS_FUNCS_MSN_35=${FUNC_MSN_START}35
typeset -r SUPOS_FUNCS_MSN_36=${FUNC_MSN_START}36
typeset -r SUPOS_FUNCS_MSN_37=${FUNC_MSN_START}37
typeset -r SUPOS_FUNCS_MSN_38=${FUNC_MSN_START}38
typeset -r SUPOS_FUNCS_MSN_39=${FUNC_MSN_START}39
typeset -r SUPOS_FUNCS_MSN_40=${FUNC_MSN_START}40

#
# Miscellaneous environment variable settings.
#

typeset -i CICS_APPEND_SRC_OUTPUT=0		# startsrc won't append output
typeset -i CICS_NOTIFY_SUPPORTED=1		# notify supported (0=no,1=yes)
typeset -i CICS_SUBSYS_MONITORING=1		# Monitor parent for ssys procs
typeset       MON_SIGNORM=USR1			# Monitor "normal stop" signal
typeset       MON_SIGFORCE=USR2			# Monitor "forced stop" signal
typeset       MON_SIGCANCEL=TERM		# Monitor "cancel stop" signal
typeset       MON_SIGTIMEOUT=SEGV		# Monitor "cancel timeout" sig
typeset       MON_SIGSTARTOK=USR1		# Monitor "ssys start OK" sig
typeset       MON_SIGSTARTFAIL=USR2		# Monitor "ssys start fail" sig
typeset    SRCDB_IFS="|"			# subsys DB field separator
typeset    STDERR=2				# stderr file number
typeset -i CICS_FLOCK_RETRY_INTERVAL=2		# seconds between lock retries
typeset -i CICS_MAX_FLOCK_RETRIES=8		# retry limit on DB lock
typeset -i CICS_PRE_MSG_RETRIES=3		# retries before clue user in
typeset -i CICS_POST_MSG_RETRIES=CICS_MAX_FLOCK_RETRIES-CICS_PRE_MSG_RETRIES
						# retries after clue user in

# File naming convention environment variables.
typeset    CICS_FLOCK_SUFFIX=".lock"
typeset    CICS_FBACK_SUFFIX=".back"
typeset    CICS_FTEMP_SUFFIX=".temp"
typeset    CICS_SYNCPT_SUFFIX=".sync"
typeset    CICS_TMP_SYNCPT_SUFFIX=".stmp"

typeset    CICS_USER=cics			# CICS user  identifier/name
typeset    CICS_GROUP=cics			# CICS group identifier/name

#
# Environment vars describing the SRC Data Base.
#

typeset    CICS_SRCDIR=/var/cics_src		# dir containing SRC DB file
typeset    CICS_SRCDIR_PERMS='ug=rwx,o=rx'	# DB directory permissions

typeset    CICS_SRCDB=$CICS_SRCDIR/cicssrcdb	# absolute path to SRC DB file
typeset    CICS_SRCDB_PERMS='ug=rw,o='		# DB file permissions

typeset    CICS_SRCDB_VERSION="v1"		# current DB record version num

typeset    CICS_SRCDB_SYNCPT=${CICS_SRCDB}${CICS_SYNCPT_SUFFIX}
						# DB image before updates
typeset    CICS_SRCDB_TMP_SYNCPT=${CICS_SRCDB}${CICS_TMP_SYNCPT_SUFFIX}
						# Temp copy of CICS_SRCDB_SYNCPT

#
# Global environment variables controlling SRC signal handling.  CICS_TRAP_SIGS
# defines those signals which are to be caught, while the remaining variables
# are maintained by the shell functions in this file to reflect the state of
# resources held, or tasks partially completed.  The settings of these
# "resource" variables drive the cleanup processing performed by the general
# signal handler, CICS_SRCDBHandleSignal, which is invoked when any of the
# signals given by CICS_TRAP_SIGS (SIGINT, for instance, generated by
# pressing <ctrl-C>) is received.
#
typeset    CICS_TRAP_SIGS="HUP INT QUIT BUS SEGV TERM USR1 USR2"
typeset -i CICS_SRCDB_LOCKED=0			# DB locked? Not yet.
typeset -i CICS_TRAN_IN_PROGRESS=0		# Incomplete DB tran. Not yet.
typeset -i CICS_DBCREATE_IN_PROGRESS=0		# Incomplete DB create. Not yet.
typeset       CICS_DBCREATE_DB=""		# Name of incomplete DB.

#
# Constants defining zero-based position of fields within the DB entry.
#
typeset -i SSYS_DB_VERSION_FLD=0
typeset -i SSYS_HOSTNAME_FLD=1
typeset -i SSYS_SUBSYS_FLD=2
typeset -i SSYS_SRCGROUP_FLD=3
typeset -i SSYS_USER_FLD=4
typeset -i SSYS_UID_FLD=5
typeset -i SSYS_PROGRAM_FLD=6
typeset -i SSYS_ARGS_FLD=7
typeset -i SSYS_STDIN_FLD=8
typeset -i SSYS_STDOUT_FLD=9
typeset -i SSYS_STDERR_FLD=10
typeset -i SSYS_RESERVED_1_FLD=11
typeset -i SSYS_RESERVED_2_FLD=12
typeset -i SSYS_RESERVED_3_FLD=13
typeset -i SSYS_NORMSTOP_SIG_FLD=14
typeset -i SSYS_FORCESTOP_SIG_FLD=15
typeset -i SSYS_CANCELSTOP_WAIT_FLD=16
typeset -i SSYS_NOTIFY_FLD=17
typeset -i SSYS_SHOWINACTIVE_FLD=18
typeset -i SSYS_ALLOW_MULT_ACTIVE_FLD=19
### End of fixed fields. #############
typeset -i SSYS_1st_INSTANCE_FLD=20		# first subsys instance subrec
### End of field index variables. ####
typeset -i CICS_SRCDB_FLD_CNT=20		# Fields minus instance subrecs
						# Should = SSYS_1st_INSTANCE_FLD

Debug()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:Debug()] '  &&
   set -x

   [ -n "$CICS_SRC_DEBUG" ] && print -u$STDERR "$@"
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	IsChild pid ppid
#
# DESCRIPTION:
#	IsChild returns 0 if the process specified by "pid" is the child of
#	the process given by "ppid", else 1.
#
# OPTIONS/ARGUMENTS:
#
#	pid	Process id whose parentage is to be checked against "ppid".
#
#	ppid	Process id to be checked for parenthood of "pid".
#
# RETURNED VALUES:
#
#	0	The process given by "pid" is the child of the process given
#		by "ppid".
#
#     > 0	Process "pid" in not the child of process "ppid"; one
#		or the other process was not specified or does not exist; or
#		an unexpected error was encountered.

IsChild()
{
   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:IsChild()] '  &&
   set -x

   # Return an error if not exactly 2 parameters.
   [ $# -ne 2 ] && return 1

   typeset    ARG_PID="$1"		# PID whose parenage is to be checked
   typeset    ARG_PPID="$2"		# PPID whose parenthood is to be checked
   typeset    ps_DISPLAY		# The entire ps display, including hdr
   typeset    ps_LINE			# A single desired ps display line
   typeset    ps_COL_ARR		# Array to hold "ps -f" field headers
   typeset -i ps_PPID_COL		# Index to PPID column of "ps -f"
   typeset    ps_PPID			# PPID obtained from ps display
   typeset    LANG="$LANG"		# Make local LANG var matching caller's
   typeset    INIT_LANG="$LANG"		# Record initial LANG value

   # Return an error if either parameter is null.
   [ -z "$ARG_PID" -o -z "$ARG_PPID" ] && return 2

   # Get the "full" ps display for $ARG_PID into ps_DISPLAY.  Force English
   # display headers.  It's OK if ps fails and we return without resetting
   # LANG, as the caller's LANG setting remains unchanged.
   unset LANG
   ps_DISPLAY=$(ps -fp $ARG_PID) || return 3
   typeset LANG="$INIT_LANG"

   # Figure out which column contains the PPID.

   # Expand the column headings into ps_COL_ARR array elements.
   set -A ps_COL_ARR $(echo "$ps_DISPLAY" | head -1)

   # Scan through ps_COL_ARR for the element containing "PPID", and remember
   # its index in ps_PPID_COL.

   ps_PPID_COL=0
   while (( ps_PPID_COL < ${#ps_COL_ARR[@]} ))
   do
      [ "${ps_COL_ARR[ps_PPID_COL]}" = "PPID" ] && break
      ps_PPID_COL=ps_PPID_COL+1
   done

   # Return an error if the "PPID" column header wan't found.
   [ $ps_PPID_COL -gt ${#ps_COL_ARR[@]} ] && return 4
   
   # Bump index so 1-based for cut.
   ps_PPID_COL=ps_PPID_COL+1

   # Extract ARG_PID's detail line from the ps display.
   ps_LINE=$(echo "$ps_DISPLAY" | tail -1)

   # Extract ARG_PID's PPID from its detail line.
   ps_PPID=$(echo $ps_LINE | cut -d" " -f$ps_PPID_COL)	# extract PPID

   # If ARG_PID's PPID, as given by ps, doesn't match ARG_PPID, return error.
   [ "$ps_PPID" != "$ARG_PPID" ] && return 5

   # ARG_PPID is ARG_PID's parent.
   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	FieldCount rec ifs
#
# DESCRIPTION:
#	FieldCount counts the number of fields in a record ("rec"), given the 
#	field separator ("ifs"), and returns the field count on stdout.
#
# OPTIONS/ARGUMENTS:
#
#	rec	record whose fields are to be counted.
#		string ("").
#
#	ifs	inter-field separator.
#
# RETURNED VALUES:
#
#	0	The field count was successfully obtained and returned on stdout
#
#     > 0	An error was encountered.
#
#	stdout	When return code is 0, the number of fields in the record is
#		returned on stdout.  Here are a few examples of output:
#
#			Command    rec   ifs    Output
#			====================	======
#			FieldCount ""    "/"	0
#			FieldCount "abc" "/"	1
#			FieldCount "a b" "/"	1
#			FieldCount " "   "/"	1
#			FieldCount "/"   "/"	2
#			FieldCount "a/b" "/"	2
#			FieldCount "/ab" "/"	2
#			FieldCount "//"  "/"	3
#

FieldCount()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:FieldCount()] '  &&
   set -x

   typeset    ARG_REC
   typeset    ARG_IFS
   typeset -i FIELD_COUNT

   # Too many or too few parameters?
   [ $# -ne 2 ] && return 1

   ARG_REC="$1"
   ARG_IFS="$2"

   FIELD_COUNT=$( echo "$ARG_REC" | awk -F"$ARG_IFS" 'NR==1 {print NF}' )
   [ $? -ne 0 ] && return 2

   echo $FIELD_COUNT
   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	UserToUID user_name
#
# DESCRIPTION:
#	UserToUID maps from a user name to its associated integer UID (User 
#       Id), and writes it on stdout.
#
# OPTIONS/ARGUMENTS:
#
#	user_name	Name of the user whose UID is to be obtained.
#
# RETURNED VALUES:
#
#	0	The user's UID was successfully obtained and written to stdout.
#
#	1	pwget failed to locate a passwd entry for "user_name".
#
#	2	Some other error occurred returning the UID for "user_name".
#
#	stdout	The successfully obtained UID is written to stdout.
#

UserToUID()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:UserToUID()] '  &&
   set -x

   typeset ARG_USER
   typeset PASSWD_ENTRY

   ARG_USER=${1:-""}
   PASSWD_ENTRY=$( pwget -n $ARG_USER 2>/dev/null )
   [ $? -ne 0 ] && return 1

   echo "$PASSWD_ENTRY" | cut -d: -f 3
   [ $? -ne 0 ] && return 2

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	UserToGID user_name
#
# DESCRIPTION:
#	UserToGID maps from a user name to its associated integer GID (Group
#	Id), and returns it on stdout.
#
# OPTIONS/ARGUMENTS:
#
#	user_name       Name of the user whose GID is to be obtained.
#
# RETURNED VALUES:
#
#	0	The user's gid was successfully obtained and written to stdout.
#
#	1	pwget failed to locate a passwd entry for "user_name"
#
#	2	Some other error occurred returning the GID for "user_name".
#
#	stdout	The successfully obtained GID is written to stdout.
#

UserToGID()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:UserToGID()] '  &&
   set -x

   typeset ARG_USER
   typeset PASSWD_ENTRY

   ARG_USER=${1:-""}
   PASSWD_ENTRY=$( pwget -n $ARG_USER 2>/dev/null )

   [ $? -ne 0 ] && return 1

   echo "$PASSWD_ENTRY" | cut -d: -f 4
   [ $? -ne 0 ] && return 2

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	VerifyInactive ssys_name [db_entry]
#
# DESCRIPTION:
#	VerifyInactive returns 0 (true) if the subsystem is inactive, 1 (false)
#	if the subsystem is active, and >1 if unable to tell due to errors.
#
# OPTIONS/ARGUMENTS:
#
#	ssys_name	Name of the subsystem whose inactivity is to be checked
#	
#	db_entry	Entry for ssys_name.  If not passed, it will be
#			retrieved from the DB.
#	
# RETURNED VALUES:
#
#	0	The subsystem is inactive.
#
#	1	The subsystem is active.
#
#     > 1	Attempt to verify inactivity failed.
#
# CAVEATS/WARNINGS:
#	The semantics of the return values must not be changed without
#	changing code dependent upon them.  For instance, each caller may
#	react differently depending upon whether 1 or >1 is returned.  When
#	a true error (>1) is returned, some callers may wish to assume the
#	subsystem is active, while others assume the subsystem is inactive
#	while others may bail out altogether.

VerifyInactive()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:VerifyInactive()] '  &&
   set -x

   typeset    ARG_SUBSYS="$1"
   typeset    ARG_SSYS_DB_ENTRY="${2:-$(CICS_SRCDBGet $ARG_SUBSYS 2>/dev/null)}"
   typeset    REFRESHED_SSYS_DB_ENTRY=""
   typeset -i FIELD_COUNT

   FIELD_COUNT=$(FieldCount "$ARG_SSYS_DB_ENTRY" "$SRCDB_IFS")

   # Verify that what was passed has at least the minimum number of fields,
   # CICS_SRCDB_FLD_CNT, else return an error.

   (( FIELD_COUNT < CICS_SRCDB_FLD_CNT )) && return 2	# Invalid entry passed.

   if (( FIELD_COUNT > CICS_SRCDB_FLD_CNT ))
   then
      # There are apparently "pid:ppid:state" fields beyond the fixed
      # fields included in CICS_SRCDB_FLD_CNT.  The subsys appears to
      # to be active.  Use CICS_SRCDBRefreshStatus to clean up any stale
      # "pid:ppid:state" fields for instantiations of the subsystem no longer 
      # running, and to obtain the "refreshed" DB entry so we can see
      # whether the subsystem is in fact still active.

      REFRESHED_SSYS_DB_ENTRY=$(CICS_SRCDBRefreshStatus -l $ARG_SUBSYS)
      [ $? -ne 0 ] && return 3

      FIELD_COUNT=$(FieldCount "$REFRESHED_SSYS_DB_ENTRY" "$SRCDB_IFS")
   fi

   (( FIELD_COUNT > CICS_SRCDB_FLD_CNT )) && return 1	# Subsystem is active.
   return 0						# Subsystem is inactive.
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBInitTranSvc 
#
# DESCRIPTION:
#	CICS_SRCDBInitTranSvc is called by each of the SRC scripts to 
#       initialize transactional services which will later be used within
#	the script and the functions it calls to provide ACID semantics to
#	SRC DB updates.
#	
#	Transaction processing by the scripts should be performed according
#	to the following template:
#
#		parameter/access checking, etc
#		CICS_SRCDBLock
#		CICS_SRCDBInitTranSvc
#		
#		TRAN_ID=$( CICS_SRCDBStartTran )
#		   .
#		Process updates
#		   .
#		If error
#		then
#		   CICS_SRCDBAbortTran  "$TRAN_ID"
#		else
#		   CICS_SRCDBCommitTran "$TRAN_ID"
#		fi
#		CICS_SRCDBUnlock
#	
#	CICS_SRCDBInitTranSvc is invoked after the subsystem database has
#	been locked, but before a transaction has been started via
#	CICS_SRCDBStartTran.  Therefore, no CICS_SRCDBStartTran-created
#	"before image" syncpoint file should exist.  If one does, we must
#	assume it was left there by an earlier transaction which terminated
#	abnormally (system failure, signal such as ctrl-C, etc) prior to
#	commiting or aborting its changes.  We must further assume that the
#	former transaction's changes were incomplete and must be backed out.
#	Therefore, it is CICS_SRCDBInitTranSvc's responsibility to rollback
#	incomplete changes to the subsystem database made by earlier
#	transactions.
#
#	See functions CICS_SRCDBStartTran, CICS_SRCDBCommitTran, and
#	CICS_SRCDBAbortTran for an overview of their functionality.
#	   
# RETURNED VALUES:
#	0		Transaction services initialized successfully.
#
#	1		Abort of previous incomplete transaction failed.
#
# CAVEATS/WARNINGS:
#	The caller must hold the SRC DB lock when calling this function.
#
#	Modified transaction nesting is permitted, with the following caveats:
#	(1) Abort of the top-level transaction results in all changes that
#	    were made within the scope of the top-level transaction
#	    being backed out.
#	(2) Abort or commit of a nested transaction is a no-op.  The top-level
#	    transaction must abort or commit as appropriate based upon success
#	    or failure of the processing within its scope, including any nested
#	    transactions.
#       (3) CICS_SRCDBInitTranSvc should only be called once, at the very
#	    beginning, and may not be called again by shell scripts or 
#	    functions invoked by the current script.

CICS_SRCDBInitTranSvc()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBInitTranSvc()] '  &&
   set -x

   if [ -f $CICS_SRCDB_SYNCPT ] 
   then
      CICS_SRCDBAbortTran TranId_TOP
      if [ $? -ne 0 ]
      then
         CicsDispErr $SUPOS_FUNCS_MSN_5 ERZ5858E $CICS_SRCDB
         return 1
      fi
   fi

   return 0
}

#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBStartTran 
#
# DESCRIPTION:
#	CICS_SRCDBStartTran takes a "syncpoint" of the subsystem database,
#	copying the file in its entirety to another file, before any updates
#	are made to the official copy.  A transaction identifier is written
#	to stdout, indicating whether the transaction started was top-level
#	or nested.  This tran id must later be passed to the corresponding 
#	abort or commit call.  Note that StartTran of a nested transaction
#	is a no-op other than returning a nested transaction id on stdout.
#
#	See the description for CICS_SRCDBInitTranSvc for more detail regarding
#	the transaction services employed by the subsystem DB management
#	scripts.
#
# CAVEATS/WARNINGS:
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBStartTran()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBStartTran()] '  &&
   set -x

   if [ -f $CICS_SRCDB_SYNCPT ]
   then

      # Syncpoint data already exists.  This must be a nested transaction.
      # Simply write the nested transaction id on stdout.

      echo "TranId_NEST"

   else

      # Syncpoint data does not yet exist.  This must be a top-level 
      # transaction.  Syncpoint the "before image" of the subsys DB by
      # creating an intermediate (temp) syncpoint file, copying the subsys
      # DB to this intermediate syncpoint file, and finally, only if all
      # data was successfully copied to the intermediate syncpoint file,
      # rename it to be the official syncpoint file.  The use of an 
      # intermediate syncpoint file guarantees that when the official syncpoint 
      # file exists, it is 100% complete.

      # First, remove any existing copy of the intermediate syncpoint file.
      
      if [ -f $CICS_SRCDB_TMP_SYNCPT ]
      then
         rm -f $CICS_SRCDB_TMP_SYNCPT
         if [ $? -ne 0 ]
         then
            CicsDispErr $SUPOS_FUNCS_MSN_6 ERZ5859E $CICS_SRCDB_TMP_SYNCPT
            return 1
         fi
      fi

      # Now create the intermediate syncpoint file with all the same
      # options, access, and ownership as the actual subsys database.

      CICS_SRCDBCreate $CICS_SRCDB_TMP_SYNCPT
      if [ $? -ne 0 ]
      then
         CicsDispErr $SUPOS_FUNCS_MSN_7 ERZ5860E $CICS_SRCDB_SYNCPT
         return 2
      fi

      # Append the contents of the subsys database to the empty intermediate
      # syncpoint file.

      cat $CICS_SRCDB >> $CICS_SRCDB_TMP_SYNCPT
      if [ $? -ne 0 ]
      then
         CicsDispErr $SUPOS_FUNCS_MSN_8 ERZ5861E $CICS_SRCDB $CICS_SRCDB_TMP_SYNCPT
         rm -f $CICS_SRCDB_TMP_SYNCPT
         return 3
      fi

      # Now that the intermediate syncpoint file is complete, move it into
      # place as the official syncpoint file.  Set CICS_TRAN_IN_PROGRESS so
      # the signal handler can cleanup the transaction if a signal such as
      # SIGINT (ctrl-C) is received prior to commit or abort.

      mv -f $CICS_SRCDB_TMP_SYNCPT $CICS_SRCDB_SYNCPT && CICS_TRAN_IN_PROGRESS=1
      if [ $? -ne 0 ]
      then
         CicsDispErr $SUPOS_FUNCS_MSN_9 ERZ5862E \
		     $CICS_SRCDB_TMP_SYNCPT $CICS_SRCDB_SYNCPT
         rm -f $CICS_SRCDB_TMP_SYNCPT
         return 4
      fi

      # Write the top-level transaction id on stdout.

      echo "TranId_TOP"

   fi

   return 0
}

#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBCommitTran tran_id
#
# DESCRIPTION:
#	CICS_SRCDBCommitTran removes the syncpoint file created by
#	CICS_SRCDBStartTran when commiting the top-level transaction,
#	but does nothing for a nested transaction commit.  Removal of the
#	"before image" contained in the syncpoint file indicates that there 
#	could no longer be any need to back out the transaction's changes,
#	hence the "before image" of the database is no longer required.
#
#	See the description for CICS_SRCDBInitTranSvc for more detail regarding
#	the transaction services employed by the subsystem DB management
#	scripts.
#
# OPTIONS/ARGUMENTS:
#
#	tran_id	Transaction handle written to stdout by CICS_SRCDBStartTran 
#		during initiation of this transaction.  This handle indicates 
#		whether the transaction being committed is top-level or nested.
#
# RETURNED VALUES:
#	0	Transaction tran_id was successfully committed, whether nested
#		or top-level.
#
#	1	Either tran_id was not specified, or it was invalid.
#
#	2	Transaction tran_id could not be committed because the
#		syncpoint file containing the transactions "before image"
#		could not be deleted.
#
# CAVEATS/WARNINGS:
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBCommitTran()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBCommitTran()] '  &&
   set -x

   typeset ARG_TRANID			# Id of transaction to commit

   ARG_TRANID=${1:-""}			# Tran id is arg 1, null if not passed

   case "$ARG_TRANID" in

      TranId_TOP)			# Commit of top-level transaction

         # Commit the transaction by removing the "before image" of the
         # SRC DB.  Clear CICS_TRAN_IN_PROGRESS now that the transaction
         # has completed.

         rm -f $CICS_SRCDB_SYNCPT && CICS_TRAN_IN_PROGRESS=0
         if [ $? -ne 0 -a -f $CICS_SRCDB_SYNCPT ]
         then
            CicsDispErr $SUPOS_FUNCS_MSN_10 ERZ5863E \
			$CICS_SRCDB $CICS_SRCDB_SYNCPT
            return 2
         fi
         ;;

      TranId_NEST)			# Commit of nested transaction.  No-op.
         ;;

      *)				# ARG_TRANID ($1) invalid or missing
         CicsDispErr $SUPOS_FUNCS_MSN_11 ERZ5864E
         return 1
         ;;
   esac

   return 0
}

#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBAbortTran tran_id
#
# DESCRIPTION:
#	CICS_SRCDBAbortTran undoes all the transaction's changes to the
#	subsys database by removing it and re-instating its "before image" 
#	contained in the syncpoint file created by CICS_SRCDBStartTran.  
#	   
#	See the description for CICS_SRCDBInitTranSvc for more detail regarding
#	the transaction services employed by the subsystem DB management
#	scripts.
#
# OPTIONS/ARGUMENTS:
#
#	tran_id	Transaction handle written to stdout by CICS_SRCDBStartTran
#		during initiation of this transaction   This handle indicates 
#		whether the transaction being aborted is top-level or nested.
#
# RETURNED VALUES:
#
#	0	Transaction tran_id was successfully aborted, whether nested
#		or top-level.  Note that the abort of a nested transaction
#		is a no-op; it is presumed that the error condition which
#		caused the abort of a nested transaction will propagate
#		back up the function call chain, ultimately resulting in the
#		top-level transaction being aborted.
#
#	1	Invalid or missing tran_id.
#
#	2	Transaction updates could not be rolled back due to the
#		absence of the syncpoint file containing the "before image".
#
#	3	Transaction updates could not be rolled back due to an error
#		re-instating the "before image" from the syncpoint file.
#
# CAVEATS/WARNINGS:
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBAbortTran()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBAbortTran()] '  &&
   set -x

   typeset    ARG_TRANID		# Id of transaction to abort

   ARG_TRANID=${1:-""}			# Arg 1 if specified, else null string

   case "$ARG_TRANID" in

      TranId_TOP)			# Abort of top-level transaction

         if [ ! -f $CICS_SRCDB_SYNCPT ]
         then
            CicsDispErr $SUPOS_FUNCS_MSN_12 ERZ5865E $CICS_SRCDB_SYNCPT
            return 2
         fi

         # Back out changes to the SRC DB by replacing it with the before
         # image created by CICS_SRCDBStartTran.  Clear CICS_TRAN_IN_PROGRESS
         # now that the transaction has completed.

         mv -f $CICS_SRCDB_SYNCPT $CICS_SRCDB && CICS_TRAN_IN_PROGRESS=0
         if [ $? -ne 0 ]
         then
            CicsDispErr $SUPOS_FUNCS_MSN_13 ERZ5866E \
			"$CICS_SRCDB" "$CICS_SRCDB_SYNCPT"
            return 3
         fi
         ;;

      TranId_NEST)			# Abort of nested transaction.  No-op.
         ;;

      *)				# ARG_TRANID ($1) invalid
         CicsDispErr $SUPOS_FUNCS_MSN_14 ERZ5864E
         return 1
         ;;
   esac

   return 0
}

#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBCreateDir [ db_dir ]
#
# DESCRIPTION:
#	CICS_SRCDBCreateDir creates SRC DB directory "db_dir", which defaults
#	to $CICS_SRCDIR, and assigns appropriate ownership (cics:cics) and
#	access permissions.  If the directory already exists, its ownership
#	and permissions are blindly set to ensure they are as expected.
#
# OPTIONS/ARGUMENTS:
#
#	db_dir	Name of the directory to contain the SRC DB.  
#		Default=$CICS_SRCDIR.
#
# RETURNED VALUES:
#
#	0	Directory already exists or was successfully created and
#		"setup".
#
#	1	Creation of the directory via mkdir failed.
#
#	2	Setting group ownership via chgrp failed.
#
#	3	Setting user ownership via chown failed.
#

CICS_SRCDBCreateDir()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBCreateDir()] '  &&
   set -x

   typeset ARG_DBDIR			# Directory in which to create SRC DB
   ARG_DBDIR=${1:-$CICS_SRCDIR}		# Dir is $1 if passed, else $CICS_SRCDIR

   if [ -d $ARG_DBDIR ]
   then
      # The database file's directory exists.  Blindly assign
      # the desired ownership and access permissions, ignoring errors, just
      # in case an earlier attempt to create it was interrupted.  The only
      # drawback to doing this is that it foils any customer attempts to
      # purposely change the cicssrcdb access policy.
      chmod $CICS_SRCDIR_PERMS $ARG_DBDIR	2>/dev/null
      chgrp $CICS_GROUP $ARG_DBDIR		2>/dev/null
      chown $CICS_USER $ARG_DBDIR		2>/dev/null
   else

      # The database file's directory does not exist.  Create it,
      # assigning desired access permissions in the process.

      mkdir -p -m $CICS_SRCDIR_PERMS $ARG_DBDIR
      if [ $? -ne 0 ]
      then
         CicsDispErr $SUPOS_FUNCS_MSN_15 ERZ5868E "$ARG_DBDIR"
         return 1
      fi

      # Assign the desired ownership on the directory.  It is imperative
      # that at least group ownership be assigned properly.  It is not
      # necessary that the owner be assigned, but we'll give it a try and
      # ignore errors.  
      #
      # Rationale: To exercise these scripts, a user will have to be the
      # superuser, the user $CICS_USER, or a member of the group
      # $CICS_GROUP.  Superuser can access anything.  The user $CICS_USER
      # will be a member of the group $CICS_GROUP, so in order for the
      # user to have all the required accesses, the directory need only be 
      # owned by group $CICS_GROUP, provided, of course, that $CICS_GROUP is
      # granted all necessary accesses.
      #
      # Changing ownership of the directory to $CICS_USER via chown will fail 
      # when the user is not superuser or $CICS_USER on a 
      # _POSIX_CHOWN_RESTRICTED system (POSIX.1, POSIX.2), or is not a 
      # privileged user (POSIX.6).  On HP-UX, for instance, the
      # same will fail unless the CHOWN privilege has been granted, via
      # setprivgrp(1M), to at least one of the groups to which the current 
      # user belongs.

      chgrp $CICS_GROUP $ARG_DBDIR
      if [ $? -ne 0 ]
      then
         CicsDispErr $SUPOS_FUNCS_MSN_16 ERZ5869E "$ARG_DBDIR" "$CICS_GROUP"
         rmdir $ARG_DBDIR
         return 2
      fi

      chown $CICS_USER $ARG_DBDIR 2>/dev/null
   fi

   return 0
}

#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBCreate [db_file]
#
# DESCRIPTION:
#	CICS_SRCDBCreate creates the SRC DB file specified by db_file, which
#	defaults to $CICS_SRCDB, and assigns appropriate ownership (cics:cics)
#	and access permissions (ug=rw,o=).
#
# OPTIONS/ARGUMENTS:
#	db_file	Name of the empty SRC DB file to be created. Defaults to
#		$CICS_SRCDB.
#
# RETURNED VALUES:
#	0	DB file already exists or was successfully created.
#
#	1	SRC DB creation via touch failed.
#
#	2	Setting access permissions via chmod failed.
#
#	3	Setting group ownership via chgrp failed.
#
# CAVEATS/WARNINGS:
#	CICS_SRCDBCreate assumes the CICS SRC directory has already been
#	created and initialized.
#
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBCreate()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBCreate()] '  &&
   set -x

   typeset ARG_SRCDB			# SRC DB to create.  Def = $CICS_SRCDB

   ARG_SRCDB=${1:-$CICS_SRCDB}		# DB is $1 if passed, else $CICS_SRCDB

   if [ -f $ARG_SRCDB ]
   then
      # The database file already exists.  Blindly assign the desired
      # ownership and access permissions, ignoring errors, just in case
      # an earlier attempt to create it was interrupted.
      chmod $CICS_SRCDB_PERMS $ARG_SRCDB	2>/dev/null
      chgrp $CICS_GROUP $ARG_SRCDB		2>/dev/null
      chown $CICS_USER $ARG_SRCDB		2>/dev/null
   else
      # Subsystem database file does not exist.

      # Physically create the database file.  Record the name of the newly-
      # created DB in CICS_DBCREATE_DB and set the CICS_DBCREATE_IN_PROGRESS
      # global resource flag, so that the DB file will be removed by the
      # signal handler if a signal is received before creation/initialization
      # is completed.

      touch $ARG_SRCDB &&
         CICS_DBCREATE_DB="$ARG_SRCDB" &&
         CICS_DBCREATE_IN_PROGRESS=1

      if [ $? -ne 0 ]
      then
         CicsDispErr $SUPOS_FUNCS_MSN_17 ERZ5870E "$ARG_SRCDB"
         return 1
      fi

      chmod $CICS_SRCDB_PERMS $ARG_SRCDB
      if [ $? -ne 0 ]
      then
	 rm -f $ARG_SRCDB
	 CICS_DBCREATE_IN_PROGRESS=0
	 CICS_DBCREATE_DB=""
         CicsDispErr $SUPOS_FUNCS_MSN_18 ERZ5871E "$CICS_SRCDB_PERMS" $ARG_SRCDB
         return 2
      fi

      # Assign desired file ownership.  See comments in CICS_SRCDBCreateDir
      # regarding ownership of the parent directory, as the same applies to
      # the database file.

      chgrp $CICS_GROUP $ARG_SRCDB
      if [ $? -ne 0 ]
      then
	 rm -f $ARG_SRCDB
	 CICS_DBCREATE_IN_PROGRESS=0
	 CICS_DBCREATE_DB=""
         CicsDispErr $SUPOS_FUNCS_MSN_19 ERZ5872E $ARG_SRCDB $CICS_GROUP
         return 3
      fi

      # Outcome here is not critical, so ignore errors and error messages.
      chown $CICS_USER $ARG_SRCDB 2>/dev/null

      # The DB is fully created and initialized.  Signal handler need no
      # longer clean up.  Clear the DB creation resource flag, etc.
      CICS_DBCREATE_IN_PROGRESS=0
      CICS_DBCREATE_DB=""
   fi

   return 0
}

#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBLock 
#
# DESCRIPTION:
#	CICS_SRCDBLock locks the SRC database, $CICS_SRCDB, to synchronize
#	access to it.  An error is returned if the DB is already locked.
#
# CAVEATS/WARNINGS:
#	Nested locking is not supported.
#

CICS_SRCDBLock()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBLock()] '  &&
   set -x

 #
 # Ensure that the parent directory exists, is properly initialised, and
 # is writable.
 #
   CICS_SRCDBCreateDir || return 1

   if [ ! -w $CICS_SRCDIR ]
   then
      CicsDispErr $SUPOS_FUNCS_MSN_3 ERZ5856E $CICS_SRCDIR
      return 2
   fi

 #
 # Create the symlink used to synchronize access to (lock) the SRC database.
 #

   # Defer signal handling until after all processing has been done to ensure
   # that we neither exit without unlocking, nor remove another process' lock.
   typeset -i CAUGHT_SIGNAL=0			# Received any signals? Not yet.
   trap 'CAUGHT_SIGNAL=1' $CICS_TRAP_SIGS	# Record signal receipt.

   typeset    LOCKINGLINK=${CICS_SRCDB}${CICS_FLOCK_SUFFIX}

   typeset -i MAX_LOCK_ATTEMPTS=CICS_MAX_FLOCK_RETRIES+1
   typeset -i ATTEMPTS_REMAINING=MAX_LOCK_ATTEMPTS
   typeset -i RETRY_MSG_SENT=0

   # Try to obtain the lock.  Retry periodically for a while if cannot.
   # Stop retrying if a signal is received.

   while
      [ $CAUGHT_SIGNAL -eq 0 ]
   do
      # Attempt to take the lock.  Exit loop if successful.
      ln -s $CICS_SRCDB $LOCKINGLINK 2>/dev/null &&
      CICS_SRCDB_LOCKED=1 &&			# Set status flag if took lock
      break					# Break out of the while loop

      # Attempt failed.  Decrement ATTEMPTS_REMAINING to reflect failed attempt.
      ATTEMPTS_REMAINING=ATTEMPTS_REMAINING-1

      # If no attempts remain, exit loop.
      [ $ATTEMPTS_REMAINING -le 0 ] && break

      # At least one retry remains.  If CICS_PRE_MSG_RETRIES have been
      # exhausted, leaving CICS_POST_MSG_RETRIES, clue user in on the delay.
      if (( ATTEMPTS_REMAINING == CICS_POST_MSG_RETRIES ))
      then
         typeset -i MAX_WAIT
	 MAX_WAIT=CICS_POST_MSG_RETRIES*CICS_FLOCK_RETRY_INTERVAL
         CicsDispErr $SUPOS_FUNCS_MSN_39 ERZ5891I $MAX_WAIT
	 RETRY_MSG_SENT=1
      fi
      sleep $CICS_FLOCK_RETRY_INTERVAL
   done

   if [ $CICS_SRCDB_LOCKED -eq 1 ]		# Obtained the lock?
   then
      # If the user was notified of the retry delay, follow
      # up with a message indicating the newfound success.
      [ $RETRY_MSG_SENT -eq 1 ] &&
      CicsDispErr $SUPOS_FUNCS_MSN_40 ERZ5892I

      # We now have exclusive access to the SRC database.  Create it and/or
      # initialize it.

      CICS_SRCDBCreate
      if [ $? -ne 0 ]
      then
	 rm -f $LOCKINGLINK
	 CICS_SRCDB_LOCKED=0
      fi
   elif [ $CAUGHT_SIGNAL -eq 0 ]
   then
      # All retry attempts failed.  Log message ERZ5857E.
      CicsDispErr $SUPOS_FUNCS_MSN_4 ERZ5857E $LOCKINGLINK
   fi

   # The lock and its status flag are in sync.  Restore signal handling
   # to its previous state.
   trap - $CICS_TRAP_SIGS

   # If signaled while "critical", invoke the standard signal handler.
   [ $CAUGHT_SIGNAL -eq 1 ] && CICS_SRCDBHandleSignal

   # If the DB isn't locked, return an error
   [ $CICS_SRCDB_LOCKED -eq 0 ] && return 3

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBUnlock 
#
# DESCRIPTION:
#	CICS_SRCDBUnlock unlocks the SRC database, $CICS_SRCDB.
#

CICS_SRCDBUnlock()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBUnlock()] '  &&
   set -x

   typeset LOCKINGLINK=${CICS_SRCDB}${CICS_FLOCK_SUFFIX}

   trap "" $CICS_TRAP_SIGS			# Ignore signals.

   if [ -L "$LOCKINGLINK"        ] &&		# Is the DB locked?
      [ $CICS_SRCDB_LOCKED -eq 1 ]		# By this proc, not some other?
   then
      # Lock file exists.  Remove it and clear the lock status flag.
      rm -f $LOCKINGLINK && CICS_SRCDB_LOCKED=0
      [ $? -ne 0 ] && CicsDispErr $SUPOS_FUNCS_MSN_1 ERZ5854E $LOCKINGLINK
   else
      # Not locked.  Merely clear the global lock status flag.
      CICS_SRCDB_LOCKED=0
   fi

   # The lock and its status flag are in sync.  Restore signal handling
   # to its previous state.
   trap - $CICS_TRAP_SIGS

   # If the DB remains locked, return an error
   [ $CICS_SRCDB_LOCKED -eq 1 ] && return 1

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBHandleSignal
#
# DESCRIPTION:
#	CICS_SRCDBHandleSignal performs the following resource cleanup
#       and then exits (does not return) with an exit code of 1:
#	1) Removes any partially-created SRC DB.
#       2) Aborts any incomplete transaction.
#       3) Unlocks the SRC DB, if locked.
#
# CAVEATS/WARNINGS:
#	CICS_SRCDBHandleSignal relies on the following global resource flags
#	and values to guide its resource deallocation/cleanup:
#
#	CICS_SRCDB_LOCKED		SRC DB lock status flag:
#					0 = unlocked
#					1 = locked
#
#	CICS_TRAN_IN_PROGRESS 		SRC transaction status flag.  
#					0 = no transaction in progress
#					1 = transaction in progress (incomplete)
#
#	CICS_DBCREATE_IN_PROGRESS	SRC DB creation status flag.
#					0 = no DB creation in progress
#					1 = DB creation in progress (incomplete)
#
#	CICS_DBCREATE_DB		Name of incomplete DB, if
#					CICS_DBCREATE_IN_PROGRESS is set.
#
# RETURNED VALUES:
#
#	1	Always exit one, to indicate that the calling script did not
#		terminate normally, but rather due to receipt of a signal.
#

CICS_SRCDBHandleSignal()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBHandleSignal()] '  &&
   set -x

   # Ignore any new signals.
   trap "" $CICS_TRAP_SIGS

   # Is cicssrcdb locked?  If so, perform cleanup and unlock it.
   if [ $CICS_SRCDB_LOCKED -ne 0 ]
   then

      # If DB creation was interrupted before the DB was fully initialized,
      # remove the incomplete DB file.
      [ $CICS_DBCREATE_IN_PROGRESS -ne 0 ] && rm -f $CICS_DBCREATE_DB

      # If a DB transaction is in progress, abort it.
      [ $CICS_TRAN_IN_PROGRESS -ne 0 ] && CICS_SRCDBAbortTran TranId_TOP

      # Unlock cicssrcdb.
      CICS_SRCDBUnlock
   fi
   exit 1
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBSelect [-v] [-F] [-f db_file] field_index pattern
#
# DESCRIPTION:
#	CICS_SRCDBSelect selects and writes on stdout SRC DB entries
#       based upon selection criteria specified by the function call's options 
#	and arguments.  Default behavior is to select and display all entries
#	in the file $CICS_SRCDB whose "field_index"th (first, second, etc) 
#	field contains the value specified by "pattern".
#
# OPTIONS/ARGUMENTS:
#
#	-v		Analagous to "grep -v".  Select only entries NOT
#			matching "pattern".  In the absence of -v, entries
#			matching "pattern" are selected.
#
#	-F		Stop after first selected entry.
#
#	-f db_file	Specify a DB file other than $CICS_SRCDB, the default,
#			from which to perform selection.
#
#	field_index	Zero-based index of the field upon which selection is
#			to be performed.
#
#	pattern		Search pattern to be matched against the value 
#			contained in the "field_index"th field of each DB 
#			entry.
#
# CAVEATS/WARNINGS:
#	Return value of 1 must always be used to indicate that, although no
#	other errors occured, the selection criteria specified resulted in
#	the selection of the empty set.  Other scripts/functions depend upon 
#	this.
#
#	The caller must hold the SRC DB lock when calling this function.
#
# RETURNED VALUES:
#
#	0	One or more DB entries was selected and printed on stdout.
#
#	1	The qualifiers specified resulted in the selection of the
#		empty set.  That is, no entries qualified for selection.
#
#	> 1	A parameter specification or file read error occurred.
#

CICS_SRCDBSelect()
{  

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBSelect()] '  &&
   set -x

   typeset -i ARG_FIELD_NUM		# Zero-based field index to match
   typeset -i CUT_FIELD_NUM		# One-based field index for "cut"
   typeset    ARG_PATTERN		# Match specified field against this
   typeset    ARG_DB_FILE		# Arg to -f opt, else $CICS_SRCDB
   typeset    OPT_COMPARISON		# "!=" if -v specified, else "="
   typeset -i OPT_FIRST_ONLY		# Stop after selecting first entry (-F)
   typeset    SRCDB_ENTRY		# Buffer to hold current DB entry
   typeset    SRCDB_FIELD_VAL		# Value of DB entry field for comparison
   typeset -i TALLY_OF_SELECTED		# Number of entries selected

   ARG_DB_FILE=$CICS_SRCDB		# Default is to use actual DB file
   OPT_COMPARISON='='			# Default is to select matching entries
   OPT_FIRST_ONLY=0			# Default is to select all qualifying,
					#  not just the first encountered
   TALLY_OF_SELECTED=0			# No entries selected yet

   while getopts :vFf: Opt
   do
      case $Opt in
         v)	OPT_COMPARISON='!='	# Select only non-matching entries
		;;
         F)	OPT_FIRST_ONLY=1	# Select only first qualifying entry
		;;
         f)	ARG_DB_FILE="$OPTARG"	# Search this file instead of default
		;;
         *)				# Invalid option or missing option arg
                CicsDispErr $SUPOS_FUNCS_MSN_20 ERZ5864E
                return 2
		;;
      esac
   done
   shift OPTIND-1

   if [ -z "$1" ] 
   then
      # Usage error
      CicsDispErr $SUPOS_FUNCS_MSN_21 ERZ5864E
      return 5
   fi

   if [ -z "$2" ] 
   then
      # Usage error
      CicsDispErr $SUPOS_FUNCS_MSN_22 ERZ5864E
      return 6
   fi

   ARG_FIELD_NUM=$1			# May wish to verify $1 is numeric 1st
   ARG_PATTERN="$2"
   
   CUT_FIELD_NUM=ARG_FIELD_NUM+1	# Convert from 0- to 1-based field index
					#  since "cut" expects 1-based index

   # Check for existence and readability of the DB file.
   if [ ! -f $ARG_DB_FILE ]
   then
       CicsDispErr $SUPOS_FUNCS_MSN_23 ERZ5876E "$ARG_DB_FILE"
       return 3
   fi

   if [ ! -r $ARG_DB_FILE ]
   then
      CicsDispErr $SUPOS_FUNCS_MSN_24 ERZ5877E $ARG_DB_FILE
      return 4
   fi

   while read SRCDB_ENTRY
   do
      SRCDB_FIELD_VAL=$(print -R "$SRCDB_ENTRY" | 
			cut -d"${SRCDB_IFS}" -f $CUT_FIELD_NUM 2>/dev/null)

      if [ "$SRCDB_FIELD_VAL" $OPT_COMPARISON "$ARG_PATTERN" ]
      then
         print -R "$SRCDB_ENTRY"		# Print selected entry on stdout
         TALLY_OF_SELECTED=TALLY_OF_SELECTED+1  # Add to tally of selected ents
         (( OPT_FIRST_ONLY==1 )) && break	# If -F specified, stop here
      fi

      :					# Exit the loop with good return code
   done < $ARG_DB_FILE

   # We exited the while loop for 1 of 3 reasons:
   # 1. The "break" statement terminated the loop.  Since break succeeds,
   #    the return code ($?) will be 0.
   # 2. EOF on $ARG_DB_FILE.  The return code will be 0, which comes from
   #    the last command executed, the ":" at the bottom of the while loop,
   #    which always leaves $? equal to 0.  Had the last statement in the
   #    loop been "false" instead of ":" (note that "true" is an alias for 
   #    ":"), $? would have been left equal to 1.
   # 3. An error was encountered reading $ARG_DB_FILE.  The return code ($?)
   #    will be non-zero.

   if [ $? -ne 0 ]
   then
      # Apparently an error reading $ARG_DB_FILE.
      CicsDispErr $SUPOS_FUNCS_MSN_25 ERZ5878E $ARG_DB_FILE
      return 7
   fi

   (( TALLY_OF_SELECTED == 0 )) && return 1
   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBProject fieldnum [...]
#
# DESCRIPTION:
#	CICS_SRCDBProject implements what in database-land is referred to
#	as the projection of a relation.  It reads SRC DB entries from stdin,
#	and it extracts ands writes on stdout the field(s) specified as 
#       arguments to this function.
#
#	If multiple fields are specified, they will be printed together on a
#	single line, in the same order in which they appear in the entry,
#	separated by the delimiter $SRCDB_IFS.
#
# OPTIONS/ARGUMENTS:
#
#	fieldnum	Zero-based index of the field to be projected.  Any
#			number of fields may be specified.
#
# RETURNED VALUES:
#
#	0	Success.  Note, there may not have been any entries to project.
#
#	1	stdin is a terminal, but must be a file or pipe.
#
#	2	No "fieldnum" arguments were specified.
#
#	3	An error occurred projecting the specified fields.
#
# EXAMPLE(S):
#
#	To project the subsystem's name and SRC group to which it belongs:
#
#	:CICS_SRCDBProject $SSYS_SUBSYS_FLD $SSYS_SRCGROUP_FLD < $CICS_SRCDB
#	cics.guinness|
#	cicslog.Ldover|log
#	cicssfs.Sdover|sfs
#
#	Note that subsystem cics.guinness is not associated with a group.
#

CICS_SRCDBProject()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBProject()] '  &&
   set -x

   typeset -i ARG_FIELD_NUM		# Zero-based field index to project
   typeset    ARG			# To hold each arg in turn
   typeset -i CUT_FIELD_NUM		# Zero-based field index to project
   typeset    CUT_FIELD_LIST		# List of fields to be projected by cut
   typeset    COMMA			# null or comma, depending on need

   # Each entry is expected to be read from stdin, which must therefore
   # be a file or a pipe, not a terminal (interactive input).
   [ -t 0 ] && return 1

   # Make sure at least one field number has been specified.
   [ -z ${1:-""} ] && return 2

   COMMA=""
   for ARG
   do
      ARG_FIELD_NUM=$ARG
      CUT_FIELD_NUM=ARG_FIELD_NUM+1
      CUT_FIELD_LIST="${CUT_FIELD_LIST}${COMMA}${CUT_FIELD_NUM}"
      COMMA=","
   done
   
   cut -d"$SRCDB_IFS" -f $CUT_FIELD_LIST
   [ $? -ne 0 ] && return 3

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBGet [-q] ssys_name
#
# DESCRIPTION:
#	CICS_SRCDBGet locates and displays the entry for subsystem ssys_name,
#	returning an error if the subsystem does not exist.
#
# OPTIONS/ARGUMENTS:
#	
#	-q	Quiet.  Locate, but do not display, the specified subsys entry.
#
# RETURNED VALUES:
#
#	0	Subsystem successfully located, and if -q not specified, 
#		displayed as well.
#
#	1	Non-existent subsystem.
#
#	2	Invalid option specified.
#
#	3	Subsys_name not specified.
#
#	4	Error searching the cicssrcdb.
#
#	5	Subsystem entry was located, but could not be displayed.
#
# CAVEATS/WARNINGS:
#	Return value of 1 must always be used to indicate a non-existent
#	subsystem, while return values greater than 1 shall always imply
#	some processing error occurred.  Callers depend upon this!
#	
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBGet()
{  

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBGet()] '  &&
   set -x

   typeset    ARG_SUBSYS=""		# Subsys name is DB key
   typeset    QUIET=0			# By default, display entry on stdout
   typeset    SRCDB_ENTRY=""		# Temp holding buf for entry retrieved
   typeset -i EXIT_CODE			# Script/function return code holder

   while getopts :q get_opt
   do
      case $get_opt in
	 q)	QUIET=1			# Do not print entry on stdout.
		;;
	 *)				# Invalid option specified.
		CicsDispErr $SUPOS_FUNCS_MSN_26 ERZ5864E
		return 2
		;;
      esac
   done
   shift OPTIND-1			# Discard getopts-handled option parms

   ARG_SUBSYS=${1:-""}			# Subsys is $1, or "" if not passed
   if [ -z "$ARG_SUBSYS" ]
   then
      # Usage error
      CicsDispErr $SUPOS_FUNCS_MSN_27 ERZ5864E
      return 3
   fi

   SRCDB_ENTRY=$(CICS_SRCDBSelect -F $SSYS_SUBSYS_FLD $ARG_SUBSYS)
   EXIT_CODE=$?
   if [ $EXIT_CODE -ne 0 ]
   then
      # CICS_SRCDBSelect returns 1 to indicate the selection of the
      # empty set (non-existent subsys in this case), and >1 for an
      # actual processing error.  CICS_SRCDBGet will do the same.
      [ $EXIT_CODE -eq 1 ] && return 1
      return 4
   fi

   #
   # Unless -q (quiet) was specified, return the entry to the caller by 
   # displaying it on stdout.  It can easily be captured in a variable by
   # calling this function as follows:
   #
   #    SRCDB_ENTRY=$(CICS_SRCDBGet <ssys_name>)
   #
   if [ $QUIET -eq 0 ]
   then
      print -R "$SRCDB_ENTRY" || return 5
   fi

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBGetGroup [-q] group_name
#
# DESCRIPTION:
#	CICS_SRCDBGetGroup locates and displays all subsystem entries
#	which are members of the subsystem grouping group_name.  An error is
#	returned if group_name contains no defined subsystems.
#
# OPTIONS/ARGUMENTS:
#
#	-q	Quiet.  Locate, but do not display, the members of subsystem
#		grouping group_name.
#
# RETURNED VALUES:
#
#	0	Members of the subsystem grouping group_name were located and,
#		unless -q was specified, were displayed on stdout.
#
#	1	Subsystem grouping group_name is not defined.
#
#	> 1	Option/argument or processing error encountered.
#
# CAVEATS/WARNINGS:
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBGetGroup()
{  

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBGetGroup()] '  &&
   set -x

   typeset ARG_GROUP=""			# Group name assoc'd with desired ents
   typeset OPT_QUIET=0			# By default, display entries on stdout
   typeset GROUP_ENTRIES=""		# Temp holding buf for entries retrieved

   while getopts :q get_opt
   do
      case $get_opt in
	 q)	OPT_QUIET=1		# Do not display entries on stdout
		;;
	 *)				# Invalid option specified.
		CicsDispErr $SUPOS_FUNCS_MSN_28 ERZ5864E
		return 2
		;;
      esac
   done
   shift OPTIND-1

   ARG_GROUP=${1:-""}			# Group is $1, or "" if not passed
   if [ -z "$ARG_GROUP" ]
   then
      # Usage error
      CicsDispErr $SUPOS_FUNCS_MSN_29 ERZ5864E
      return 3
   fi

   GROUP_ENTRIES=$(CICS_SRCDBSelect $SSYS_SRCGROUP_FLD $ARG_GROUP)
   if [ $? -ne 0 ]
   then
      # The specified group does not exist in the database.
      return 1
   fi

   # Unless -q (quiet) was specified, return the entries to the caller by 
   # displaying them on stdout.  They can easily be captured in a variable by
   # calling this function as follows:
   #
   #    GROUP_ENTRIES=$(CICS_SRCDBGetGroup <group_name>)

   if [ $OPT_QUIET -eq 0 ]
   then
      print -R "$GROUP_ENTRIES" || return 4
   fi

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBPut ssys_entry
#
# DESCRIPTION:
#	CICS_SRCDBPut transactionally inserts subsystem entry ssys_entry to 
#	the SRC DB.
#
# OPTIONS/ARGUMENTS:
#
#	ssys_entry	Subsystem entry to be added to the DB.
#
# CAVEATS/WARNINGS:
#	No check is made to ensure that the subsystem defined by ssys_entry
#	is currently undefined.  Prevention of duplicates must be performed 
#	by the caller.
#
#	Top-level scripts must perform CICS_SRCDBInitTranSvc() before directly 
#	or indirectly invoking this function.
#
# CAVEATS/WARNINGS:
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBPut()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBPut()] '  &&
   set -x

   typeset    ARG_SSYS_DB_ENTRY="$1"
   typeset    PUT_TRAN_ID
   typeset -i KEY=SSYS_SUBSYS_FLD+1	# Sort DB on subsys fld.  Convert from
					# 0- to 1-based field index.

   (($(FieldCount "$ARG_SSYS_DB_ENTRY" "$SRCDB_IFS") < CICS_SRCDB_FLD_CNT)) &&
      return 1

   PUT_TRAN_ID=$(CICS_SRCDBStartTran) || return 2

   # Use sort to add the new entry to the DB in sorted order.  The specific
   # options and their values are as follows:
   #
   # -o$CICS_SRCDB	writes the sorted output directly back to the DB file
   #
   # -t"$SRCDB_IFS"	tell sort what character delineates/separates fields
   #
   # -k $KEY,$KEY	sort by the subsystem name field, using the POSIX.2 
   #			sort key specification syntax (-k)
   #
   # -			the trailing "-" tells sort that one of the files to 
   #			be sorted is to be read from stdin

   print -R "$ARG_SSYS_DB_ENTRY" | 
      sort -o$CICS_SRCDB -t"$SRCDB_IFS" -k $KEY,$KEY $CICS_SRCDB -
   if [ $? -ne 0 ]
   then
      CICS_SRCDBAbortTran $PUT_TRAN_ID
      return 3
   fi

   CICS_SRCDBCommitTran $PUT_TRAN_ID
   if [ $? -ne 0 ]
   then
      CICS_SRCDBAbortTran $PUT_TRAN_ID
      return 4
   fi

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBDelete [-l] ssys_name
#
# DESCRIPTION:
#	CICS_SRCDBDelete transactionally deletes the DB entry for
#	the subsys whose name is specified by ssys_name.
#
# OPTIONS/ARGUMENTS:
#
#	-l		(ell) Loud.  Display the entry for subsys ssys_name on
#			stdout before deleting it.  By default, the entry is
#			not displayed.
#
#	ssys_name	Name of the subsystem to be removed.
#
# CAVEATS/WARNINGS:
#	Top-level scripts must perform CICS_SRCDBInitTranSvc() before directly 
#	or indirectly invoking this function.
#
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBDelete()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBDelete()] '  &&
   set -x

   typeset    ARG_SUBSYS		# Name of subsys to be deleted
   typeset -i OPT_LOUD			# -l specified?  Write entry to stdout?
   typeset    DBGET_QUIET		# DBGet quiet/loud option
   typeset    DEL_OPT			# holds option from getopts
   typeset    DEL_TRAN_ID		# Delete transaction identifier/handle
   typeset -i RETURN_CODE		# Used to retain value of $? for later
   typeset    TEMP_SRCDB		# holds image of DB just before deletion

   OPT_LOUD=0				# Default = don't display deleted entry

   # Parse options and arguments.
   
   while getopts :l DEL_OPT
   do
      case $DEL_OPT in
	 l)	OPT_LOUD=1		# write deleted entry on stdout
		;;
	 *)	CicsDispErr $SUPOS_FUNCS_MSN_30 ERZ5864E
                return 1
		;;
      esac
   done
   shift OPTIND-1

   ARG_SUBSYS=${1:-""}			# arg 1 if specified, else null string

   # Check that the entry exists.  Unless -l (OPT_LOUD) was specified, pass -q
   # option to CICS_SRCDBGet to prevent it from displaying the entry.
   
   DBGET_QUIET="-q" 			# Default is quiet - no display
   [ OPT_LOUD -eq 1 ] && DBGET_QUIET=""	# -l (loud) specified - display entry

   CICS_SRCDBGet $DBGET_QUIET $ARG_SUBSYS
   if [ $? -ne 0 ]
   then
      CicsDispErr $SUPOS_FUNCS_MSN_31 ERZ5830E "$ARG_SUBSYS"
      return 2
   fi

   DEL_TRAN_ID=$( CICS_SRCDBStartTran ) || return 3

   # We'll be recreating the DB, so let's rename it to be used as input
   # for the re-creation processing.
   
   TEMP_SRCDB="${CICS_SRCDB}${CICS_FTEMP_SUFFIX}"
   mv -f $CICS_SRCDB $TEMP_SRCDB
   if [ $? -ne 0 ]
   then
      CicsDispErr $SUPOS_FUNCS_MSN_32 ERZ5885E $CICS_SRCDB
      CICS_SRCDBAbortTran $DEL_TRAN_ID
      return 4
   fi

   # Create the empty new DB file.

   CICS_SRCDBCreate 
   if [ $? -ne 0 ]
   then
      # DB creation failed - put the original DB back in place.
      CicsDispErr $SUPOS_FUNCS_MSN_33 ERZ5886E $CICS_SRCDB

      mv -f $TEMP_SRCDB $CICS_SRCDB
      RETURN_CODE=$?

      CICS_SRCDBAbortTran $DEL_TRAN_ID

      if [ $RETURN_CODE -ne 0 ]
      then
         CicsDispErr $SUPOS_FUNCS_MSN_34 ERZ5887E "$CICS_SRCDB" "$TEMP_SRCDB"
         return 5
      fi

      return 6
   fi

   # Use "CICS_SRCDBSelect -v" (analagous to "grep -v") to obtain all subsystem
   # entries except the one to be deleted, redirecting output to the new DB 
   # file.

   CICS_SRCDBSelect -v -f $TEMP_SRCDB $SSYS_SUBSYS_FLD $ARG_SUBSYS >> $CICS_SRCDB

   # Check for return codes higher than 1: analagous to grep, 1 indicates that
   # the specified criteria resulted in the selection of the empty set, ie,
   # the entry to be deleted was the only one in the file.

   if [ $? -gt 1 ]
   then
      # DB regeneration failed - put the original DB back in place.
      CicsDispErr $SUPOS_FUNCS_MSN_35 ERZ5888E "$CICS_SRCDB"

      mv -f $TEMP_SRCDB $CICS_SRCDB
      RETURN_CODE=$?

      CICS_SRCDBAbortTran $DEL_TRAN_ID

      if [ $RETURN_CODE -ne 0 ]
      then
         CicsDispErr $SUPOS_FUNCS_MSN_36 ERZ5887E "$CICS_SRCDB" "$TEMP_SRCDB"
         return 7
      fi
      return 8
   fi

   rm -f $TEMP_SRCDB

   # If we've gotten this far, the operation has completed successfully.

   CICS_SRCDBCommitTran $DEL_TRAN_ID
   if [ $? -ne 0 ]
   then
      CICS_SRCDBAbortTran $DEL_TRAN_ID
      return 9
   fi

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBUpdate ssys_name ssys_entry
#
# DESCRIPTION:
#	CICS_SRCDBUpdate transactionally updates the DB entry for
#	the subsys whose name is specified by ssys_name, by replacing
#	its current DB entry with the value of argument ssys_entry.
#
# OPTIONS/ARGUMENTS:
#
#	ssys_name	Name of the subsystem to be updated.
#
#	ssys_entry	Updated entry for subsystem ssys_name, to be used
#			to overlay the subsystem's current entry in the DB.
#
# CAVEATS/WARNINGS:
#	Top-level scripts must perform CICS_SRCDBInitTranSvc() before directly 
#	or indirectly invoking this function.
#
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBUpdate()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBUpdate()] '  &&
   set -x

   typeset ARG_SUBSYS="$1"
   typeset ARG_SRC_DB_ENTRY="$2"
   typeset UPD_TRAN_ID

   UPD_TRAN_ID=$( CICS_SRCDBStartTran ) || return 1

   CICS_SRCDBDelete "$ARG_SUBSYS"       &&
   CICS_SRCDBPut "$ARG_SRC_DB_ENTRY"    &&
   CICS_SRCDBCommitTran $UPD_TRAN_ID

   # If each of the function calls in the conditional-and (&&) command list
   # succeeds, the entire construct returns 0.  Otherwise, the first function
   # to fail (non-0 return) terminates execution of the command list, leaving
   # its return code in place to be tested.

   if [ $? -ne 0 ]
   then
      CICS_SRCDBAbortTran $UPD_TRAN_ID
      return 2
   fi

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBGroupToSubsysList group_name
#
# DESCRIPTION:
#	CICS_SRCDBGroupToSubsysList returns (writes on stdout) the names of
#	the subsystems which are members of the group group_name, one subsystem
#	per line.
#
# CAVEATS/WARNINGS:
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBGroupToSubsysList()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBGroupToSubsysList()] '  &&
   set -x

   typeset GROUP_KEY="$1"
   typeset GROUP_ENTRIES=""

   GROUP_ENTRIES=$( CICS_SRCDBGetGroup "$GROUP_KEY" 2>&1 )
   [ $? -ne 0 ] && return 1

   print -R "$GROUP_ENTRIES" | CICS_SRCDBProject $SSYS_SUBSYS_FLD
   [ $? -ne 0 ] && return 2

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBAllSubsysList 
#
# DESCRIPTION:
#	CICS_SRCDBAllSubsysList returns (writes on stdout) the name of each
#	defined subsystem, one subsystem per line.
#
# CAVEATS/WARNINGS:
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBAllSubsysList()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBAllSubsysList()] '  &&
   set -x

   cat $CICS_SRCDB | CICS_SRCDBProject $SSYS_SUBSYS_FLD
   [ $? -ne 0 ] && return 1

   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBRefreshStatus [ -l ] ssys_name
#
# DESCRIPTION:
#	CICS_SRCDBRefreshStatus refreshes subsystem status values maintained
#	in the DB entry for the specified subsystem.
#
#	During cicsstartsrc, the PID of the process started is recorded in the 
#	subsystem's DB entry and is assigned "active" status.  Any attempt to
#	shutdown that instantiation of the subsystem using cicsstopsrc will
#	result in the PID's status being revised to "stopping".
#
#	When the subsystem PID terminates, either normally, as a result of
#	cicsstopsrc having been issued against it, or abnormally, the PID is
#	not automatically removed from the DB entry, nor is its status updated.
#	
#	CICS_SRCDBRefreshStatus loops through each "active" or "stopping" PID
#	listed in the subsystem's DB entry, checks to see whether the process
#	has terminated since its status was last refreshed, and removes
#	the PID from the DB entry if, in fact, it has terminated.
#	
# OPTIONS/ARGUMENTS:
#
#	-l		(ell) Loud.  Additionally return the "refreshed" DB 
#			entry on stdout.
#
#	ssys_name	Name of the subsystem whose DB entry is to be
#			refreshed with the latest subsystem status.
#
# RETURNED VALUES:
#
# CAVEATS/WARNINGS:
#	The caller must hold the SRC DB lock when calling this function.

CICS_SRCDBRefreshStatus()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBRefreshStatus()] '  &&
   set -x

   # Parse options and arguments.
   
   typeset -i LOUD=0
   typeset    REFRESH_OPT
   typeset    SRC_DB_ENTRY
   typeset    FLDARR[CICS_SRCDB_FLD_CNT+10]

   while getopts :l REFRESH_OPT
   do
      case $REFRESH_OPT in
	 l)	LOUD=1			# return (display) refreshed entry
		;;
	 *)				# Invalid option.
		CicsDispErr $SUPOS_FUNCS_MSN_37 ERZ5864E
		return 1
		;;
      esac
   done
   shift OPTIND-1
	
   typeset ARG_SUBSYS=${1:-""}
   if [ -z "$ARG_SUBSYS" ]
   then
      CicsDispErr $SUPOS_FUNCS_MSN_38 ERZ5864E
      return 2
   fi

   SRC_DB_ENTRY=$(CICS_SRCDBGet "$ARG_SUBSYS")
   [ $? -ne 0 ] && return 3
   
   OLD_IFS="$IFS"			# save original IFS before changing it
   IFS="$SRCDB_IFS"			# set IFS to SRCDB_IFS field delim
   set -A FLDARR $SRC_DB_ENTRY		# Decompose fields into array elements
   IFS="$OLD_IFS"			# Restore original IFS

   typeset -i FLDCNT=${#FLDARR[@]}	# Number of fields in DB entry
   typeset -i FLDCHG=0			# Number of revised fields.

   typeset -i FLDNUM=$SSYS_1st_INSTANCE_FLD # 1st instance field/subrec

   while (( FLDNUM < FLDCNT ))
   do

      # There are more "pid:ppid:state" fields requiring validation.

      # Move select fields from the array FLDARR into suitably named env
      # variables.  Note that the field ${FLDARR[FLDNUM]} contains a 
      # "pid:ppid:state" triple.  Use cut to extract just the pid.

      typeset SSYS_PID=$( echo ${FLDARR[FLDNUM]} | cut -d: -f 1 )
      typeset SSYS_PROGRAM="${FLDARR[SSYS_PROGRAM_FLD]}"
      typeset SSYS_USER="${FLDARR[SSYS_USER_FLD]}"
      typeset SSYS_UID="${FLDARR[SSYS_UID_FLD]}"

      CICS_SRCDBValidatePID $SSYS_PID $SSYS_PROGRAM $SSYS_USER $SSYS_UID
      if [ $? -ne 0 ]
      then
	 # The subsystem is no longer running under that PID.
	 FLDARR[FLDNUM]=""
	 FLDCHG=FLDCHG+1
      fi

      FLDNUM=FLDNUM+1
   done

   if [ $FLDCHG -eq 0 ] 
   then
      # No changes, return with 0 exit status.  If the loud option, -l, was
      # specified, display the unmodified entry before returning.
      [ LOUD -eq 1 ] && print -R "$SRC_DB_ENTRY"
      return 0 				
   fi

   typeset REFRESHED_DB_ENTRY=""

   # Reconstruct the revised entry.  

   # Copy fields up to, but not including, SSYS_1st_INSTANCE_FLD from FLDARR.
   # All but the first field are preceded by the field delimiter $SRCDB_IFS.

   REFRESHED_DB_ENTRY="${FLDARR[0]}"
   FLDNUM=1

   while (( FLDNUM < SSYS_1st_INSTANCE_FLD ))
   do
      REFRESHED_DB_ENTRY="${REFRESHED_DB_ENTRY}${SRCDB_IFS}${FLDARR[FLDNUM]}"
      FLDNUM=FLDNUM+1
   done

   # Append any valid "pid:ppid:state" subsystem active instance fields.

   FLDNUM=SSYS_1st_INSTANCE_FLD

   while (( FLDNUM < FLDCNT ))
   do
      if [ -n "${FLDARR[FLDNUM]}" ]
      then
	 REFRESHED_DB_ENTRY="${REFRESHED_DB_ENTRY}${SRCDB_IFS}${FLDARR[FLDNUM]}"
      fi
      FLDNUM=FLDNUM+1
   done

   # No need to wrap CICS_SRCDBStartTran/CommitTran/AbortTran around this
   # individual update, as CICS_SRCDBUpdate will take care of that.  If we
   # had made multiple modifications as part of our transaction, on the other
   # hand, such transaction management would have been required.

   CICS_SRCDBUpdate "$ARG_SUBSYS" "$REFRESHED_DB_ENTRY"
   [ $? -ne 0 ] && return 4

   [ LOUD -eq 1 ] && print "$REFRESHED_DB_ENTRY"
   return 0
}


#------------------------------------------------------------------------------
#
# SYNOPSIS:
#	CICS_SRCDBValidatePID pid program user_name [uid]
#
# DESCRIPTION:
#	CICS_SRCDBValidatePID returns 0 if process "pid" is running "program"
#	as the user "user_name" or "uid", non-zero otherwise.  
#
#	CICS_SRCDBValidatePID is called by scripts and/or functions wishing 
#	to determine whether subsystem instantiation created via cicsstartsrc 
#	is still running, 
#
# OPTIONS/ARGUMENTS:
#
#	pid		Process id of the subsystem formerly instantiated by 
#			cicsstartsrc.
#
#	program		Absolute path name of subsystem's main program file,
#			invoked by cicsstartsrc at subsystem start-up.
#
#	user_name	Name of user subsystem is configured to run as.
#
#	uid		Integer userid subsystem is configured to run as.
#			Normally, and by default, the userid associated with 
#			user_name.
#
# RETURNED VALUES:
#
#	0		"pid" is a valid instantiation of the subsystem in
#			question
#
#     > 0		Most likely, the subsystem instantiation is no
#			longer running as process id "pid".  However, other
#			processing errors are possible.
#

CICS_SRCDBValidatePID()
{

   # Turn on ksh execution tracing if CICS_SRC_XTRACE is non-null
   [ -n "$CICS_SRC_XTRACE" ]  &&
   typeset PS4='[cicssrc.fnc:CICS_SRCDBValidatePID()] '  &&
   set -x

   typeset ARG_PID="$1"
   typeset ARG_PROGRAM="$2"
   typeset ARG_USER="$3"
   typeset ARG_UID="${4:-$(UserToUID $ARG_USER)}"

   typeset ps_DISPLAY=""
   typeset ps_LINE=""
   typeset ps_USER=""
   typeset ps_COMMAND=""

   ps_DISPLAY=$(ps -fp $ARG_PID 2>/dev/null)
   [ $? -ne 0 ] && return 1
   ps_LINE=$(echo "$ps_DISPLAY" | tail -1)
   ps_USER=$(echo $ps_LINE  | cut -f 1 -d ' ')

   ps_DISPLAY=$(ps -p $ARG_PID 2>/dev/null)
   [ $? -ne 0 ] && return 2
   ps_LINE=$(echo "$ps_DISPLAY" | tail -1)
   ps_COMMAND=$(echo "$ps_LINE" | awk '{print $NF}')

   #
   # Double check that the process has the expected UID and is executing
   # the expected program. 
   [ "$ps_USER" != "$ARG_USER" -a \
     "$ps_USER" != "$ARG_UID"  -a \
     "$(UserToUID $ps_USER)" != "$ARG_UID" ] && return 3
   [ "$ps_COMMAND" != $(basename $ARG_PROGRAM) ] && return 4

   return 0
}
